#!/usr/local/bin/perl5.8.0 -w
#
#
# Designed to;
#   goto ftp.tigr.org (login as cruzi)
#   get  /pub/MySQL/TIGR_SANGER_compare.tar.gz
#
# Author: Paul Mooney (pjm@sanger.ac.uk)
#
use strict;
use Net::FTP;
use Cwd;

if (@ARGV < 2 || @ARGV > 3) {
    &usage();
    exit 1;
}

my $ftp_addr           = "ftp.tigr.org";
my $user               = $ARGV[0];
my $password           = $ARGV[1];

# THIS IS NO LONGER NEEDED?
my $force_download     = $ARGV[2]; # Will downoad DB even if already done today

my $tar_file_exists    = 0;
my $finish_file_exists = 0;
my $finish_file_name   = undef;

my $date_last_downloaded = "0"; # Stores yyyymmdd from finish_mm_dd_yyyy
my $date_last_posted     = "0";


##
# FTP connect, login and cd to correct directory
##
my $ftp = Net::FTP->new($ftp_addr, Debug => 1) 
    or die "Cannot connect to $ftp_addr: $@";

$ftp->login($user, $password)
    or die "Cannot login: ", $ftp->message;

$ftp->cwd("/pub/MySQL/")
    or die "Cannot change working directory: ", $ftp->message;;

my @lines = $ftp->dir();


##
# After changing dir, everything else is in eval{} so can cd back
##
my $old_dir = cwd();
chdir '/nfs/pathdb/prod/coann/data/prod';

eval {
    ##
    # Make sure we download something newer, not the same or older
    ##
    opendir DIR, "." or die "ERROR: Could not open dir '.'";
    my @contents = grep !/^\./, readdir DIR;
    closedir DIR;

    foreach my $entry (@contents) {
        if ($entry =~ /finish_(\d+)_(\d+)_(\d+)/) {
            $date_last_downloaded = sprintf("%4d%02d%02d", $3, $1, $2);
            print "\n\nDATE LAST DOWNLAODED = $date_last_downloaded\n\n";
            last;
        }
    }


    ##
    # Look at `dir` output, check files present on FTP site
    ##
    foreach my $line (@lines) {
        my @parts = split(/\s+/, $line);

        if (@parts > 5){
            my ($month, $day, $file_name) = (@parts[5..6], $parts[8]);

            if ($file_name eq 'TIGR_SANGER_compare.tar.gz') {
                $tar_file_exists = 1;
            }

            #if ($file_name eq $finish_file_name) {
            if ($file_name =~ /^finish_(\d+)_(\d+)_(\d+)/) {
                $date_last_posted   = sprintf("%4d%02d%02d", $3, $1, $2);
                $finish_file_exists = 1;
                $finish_file_name   = $file_name;

                print "DATE LAST POSTED     = $date_last_posted\n\n";
                #print "\n\n*** finish_file name = '$file_name' ***\n\n\n";

                unless ($force_download) {
                    if ($date_last_posted <= $date_last_downloaded) {
                        die "\n\n  No new database to download\n".
                            "    Date last downloaded: $date_last_downloaded\n".
                            "    Date on FTP site    : $date_last_posted\n\n\n";
                    }
                }
            }

            if ($tar_file_exists && $finish_file_exists) {

                $ftp->get("TIGR_SANGER_compare.tar.gz");

                # Do platform independant `touch` for finish_mm_dd_yyyy
                # Remove old ones first
                unlink <finish_*>;
                open FINISHED, ">$finish_file_name";
                print FINISHED "Complete: ", scalar(localtime()), "\n";
                close FINISHED;

                last;
            }
        }
    }
};

my $error = $@;
$ftp->quit;
chdir $old_dir;

if ($error) {
    print STDERR "\nERROR: something bad _might_ have happened; $error";
}

##
# Error checking
##
unless ($tar_file_exists && $finish_file_exists) {
    print STDERR "\nERROR: The tar file could not be downloaded\n";

    unless ($finish_file_exists) {
        print STDERR "       The finish file '$finish_file_name' ".
            "was not present\n";
    }

    unless ($tar_file_exists) {
        print STDERR "       The tar file was not present\n";
    }

    exit 1;
}


exit 1 if $error; # Final catch all, make sure error code ok for calling scripts



################################################################################
#
# usage()
#
################################################################################
sub usage {
    print "\nUsage get-database <user> <password>\n\n";
}

################################################################################
#
# get_numeric_month()
#
################################################################################
sub get_numeric_month {
    my ($month) = @_;
    my %months = ('Jan' =>  1,
                  'Feb' =>  2,
                  'Mar' =>  3,
                  'Apr' =>  4,
                  'May' =>  5,
                  'Jun' =>  6,
                  'Jul' =>  7,
                  'Aug' =>  8,
                  'Sep' =>  9,
                  'Oct' => 10,
                  'Nov' => 11,
                  'Dec' => 12,
                  );
    return $months{$month};
}

################################################################################
#
# This returns todays date for the finish file ie. finish_12_31_2004
# (Note US date)
#
# THIS HAS NOT BEEN TESTED IN ANGER YET...................
#
################################################################################
sub get_todays_finish_time {

    my ($day, $month, $year) = (localtime())[3..5];
    ++$month; # zero based, i.e. Jan == 0
    $year += 1900;

    $finish_file_name = sprintf("finish_%d_%d_%4d", $month, $day, $year);

    return $finish_file_name;
}
