#!/bin/tcsh
#
# Author: Paul Mooney (pjm@sanger.ac.uk)
#
# usage: install.sh [force]
#
# Description;
#
# Without the argument, the database will not be downloaded if it has already
# happended today, hence you can force it to download and install a new DB.
#
# Steps;
#
# . Stop MySQL
# . FTP new tigr db        [This is done by get-database.pl]
# . tar and zip up old one
# . install new one
# . restart MySQL
#


setenv BASE /nfs/pathdb/prod/coann

if (-e ${BASE}/etc/mysql.prod.sock) then
    echo "MySQL database running, shutting down"
    ${BASE}/bin/stop.sh
else
    echo "MySQL databse _NOT_ running"
endif


###
# If get-database.pl did not finish nicely, abort!!!
##
${BASE}/bin/get-database.pl cruzi KRUz1 $1 $2

if ( $status != 0 ) then
    echo "Did not get TIGR DB, restarting MySQL server"
    ${BASE}/bin/start.sh &
    # EMAIL SOMEONE HERE???
    exit 1;
endif



cd ${BASE}/data/prod

echo "Removing old backups (TIGR_SANGER_compare.old.tar.gz)"
rm -f   TIGR_SANGER_compare.old.tar.gz
rm -f   TIGR_SANGER_compare.old.tar

echo "Creating backups of existing DB"
tar -cf TIGR_SANGER_compare.old.tar TIGR_SANGER_compare
gzip    TIGR_SANGER_compare.old.tar

echo "Removing DB"
rm -rf  TIGR_SANGER_compare

echo "Installing new DB"
gunzip  TIGR_SANGER_compare.tar.gz
tar -xf TIGR_SANGER_compare.tar
gzip    TIGR_SANGER_compare.tar

cd -

echo "Starting MySQL server"
${BASE}/bin/start.sh &
