#!/bin/tcsh
#
# Description;
#
# Post XML files to the FTP site so TIGR can grab them to process.
# They will be used to find differences in the annotation between centres.
#
# 1. CVS checkout the EMBL files for T. brucei
# 2. Create a file of filenames (FOF) for the EMBL files we wish to process
#    (some filtering is required here)
# 3. Loop over FOF and generate one XML file for each EMBL file
# 4. Copy XML files to FTP directory, though non OSF1 hosts probably do not have it mounted...
#
setenv COANN_BASE         /nfs/pathdb/prod/coann
setenv CVSROOT            /nfs/pathdata/repository/

setenv TRYP_FTP_DIR       /nfs/disk69/ftp/pub/databases/T.brucei_sequences/Tbrucei

cd /tmp

if (! -d EMBL-files) then
    mkdir EMBL-files
endif

cd EMBL-files


################################################################################
# Get all LEISH EMBL files.
rm -rf annotation/Leishmania/major
cvs co annotation/Leishmania/major

echo "pwd ${PWD}"






# -----------------> DEBUG  HACK  <------------------------

cp -f /nfs/team81/pjm/temp/all_LEISH_cvs_embl_files .

#find ${PWD}/annotation/Leishmania/major -name '*.artemis'  >! all_LEISH_cvs_embl_files


#
#
# Currently all the EMBL files in CVS for Leish (all 3 of them) have /gene as the identifier
# I think Chris has updated all of the files to use /systematic_id
# With out the /systematic_id the xml.properties file need to be changed to understand /gene
#









################################################################################
# Use file of filenames to create an XML file for each EMBL file
##
chdir /tmp
rm -rf XML-LEISH-files
mkdir  XML-LEISH-files
cd     XML-LEISH-files

cp /nfs/pathdata/conf/xml.properties.LEISH ./xml.properties

foreach filename (`cat /tmp/EMBL-files/all_LEISH_cvs_embl_files` )
    echo "filename = $filename"
    echo $filename | fastembl2tigr /tmp/XML-LEISH-files
end



################################################################################
# Copy XML files to FTP directory
#
# FTP dir not mounted on hoyle, other hosts
##

if ( -d /nfs/disk69/ftp/pub/databases/T.brucei_sequences/Tbrucei ) then
    echo ""
    echo "About to copy over all files to FTP directory"
    echo ""

    cd /nfs/disk69/ftp/pub/databases/L.major_sequences/XML/TIGR_DOWNLOAD_AREA
    rm -f *
    rcp /tmp/XML-LEISH-files/*xml .

    touch `${COANN_BASE}/bin/get-finish-today.pl`
else
    echo ""
    echo "The FTP directory is not mounted on this host."
    echo " *** The XML files have not been copied ***"
    echo ""

endif


