#!/bin/tcsh
#
# Description;
#
# Post XML files to the FTP site so TIGR can grab them to process.
# They will be used to find differences in the annotation between centres.
#
# 1. CVS checkout the EMBL files for T. brucei
# 2. Create a file of filenames (FOF) for the EMBL files we wish to process
#    (some filtering is required here)
# 3. Loop over FOF and generate one XML file for each EMBL file
# 4. Copy XML files to FTP directory, though non OSF1 hosts probably do not have it mounted...
#
setenv COANN_BASE         /nfs/pathdb/prod/coann
setenv CVSROOT            /nfs/pathdata/repository/

setenv TRYP_FTP_DIR       /nfs/disk69/ftp/pub/databases/T.brucei_sequences/Tbrucei

cd /tmp

if (! -d EMBL-files) then
    mkdir EMBL-files
endif

cd EMBL-files

################################################################################
# Get all TRYP EMBL files. NOTE: hack to get chr01 file
rm -f  all_TRYP_cvs_embl_files
rm -rf annotation/Trypanosoma/brucei/
cvs co annotation/Trypanosoma/brucei/

# chr11 has a special file that has 54 contigs "bookended". Use psu_splitter to get 54 files again
# THIS WILL CHANGE IN FUTURE - contigs due to be merged in future.
#cd annotation/Trypanosoma/brucei/chr11
#psu_splitter.pl -ossingle Tb927_11_remaining.emblgpi
#mv Tb927_11_remaining.emblgpi Tb927_11_remaining.NOT_MERGED_emblgpi

echo "pwd = ${PWD}"
cd /tmp/EMBL-files

find ${PWD}/annotation/Trypanosoma/brucei/ -name '*.embl*' | grep -v 'chr10/Release1' | grep -v 'chr10/additional_contigs' | grep -v 'SplitSeqs/old' | grep -v 'Tb927_11_remaining.emblgpi' >! all_TRYP_cvs_embl_files
#echo "${PWD}/annotation/Trypanosoma/brucei/chr01/tryp1.whole_chrom" >> all_TRYP_cvs_embl_files


################################################################################
# Use file of filenames to create an XML file for each EMBL file
##
chdir /tmp
rm -rf XML-TRYP-files
mkdir  XML-TRYP-files
cd     XML-TRYP-files

cp /nfs/pathdata/conf/xml.properties.TRYP ./xml.properties

#set array = ( `cat all_embl_files` )
#echo "array size = $#array"
#echo "array[1] = $array[1]"

foreach filename (`cat /tmp/EMBL-files/all_TRYP_cvs_embl_files` )
    echo "filename = $filename"
    echo $filename | fastembl2tigr /tmp/XML-TRYP-files
end



################################################################################
# Copy XML files to FTP directory
#
# FTP dir not mounted on hoyle, other hosts
##

#if ( `uname -s` == 'OSF1' ) then
if ( -d /nfs/disk69/ftp/pub/databases/T.brucei_sequences/Tbrucei ) then
    echo ""
    echo "About to copy over all files to FTP directory"
    echo ""

    cd /nfs/disk69/ftp/pub/databases/T.brucei_sequences/Tbrucei
    rm -f *
    rcp /tmp/XML-TRYP-files/*xml .

    echo "All files copied."

    touch `${COANN_BASE}/bin/get-finish-today.pl`

    echo "Created finish_mm_dd_yyyy."
    echo "COMPLETE"
else
    echo ""
    echo "The FTP directory is not mounted on this host."
    echo " *** The XML files have not been copied ***"
    echo ""

endif


