#!/usr/local/bin/perl5.8.0
#!/usr/local/bin/perl5.6.1
#!/usr/bin/perl5.6.1
#
# This searches the TIGR database for all updates that must be applied to the
# Sanger EMBL files.
# 
# The user can apply all changes or just the ones selected (autoUpdate = 1).
# If the user applies _all_ changes, CONFLICTS still need to be selected as
# it would apply all TIGR's changes regardless.
#
# It does not yet care about organism but hopefully this will not be an issue
# if we have one DB per org.
#
use strict;
use DBI;

$ENV{'PERL5LIB'} = "/nfs/pathsoft/prod/genlib/perl/src";

#use lib "/nfs/pathsoft/prod/genlib/perl/src"; # bio/PSU modules
use lib "/nfs/pathdb/prod/coann/perlLib";     # Bioperl 1.4 in here also
use Mysql_connect;
use ConfigFileReader;
use Gene_obj;
use Data::Dumper; # Remove when this goes "live"
use Storable;
use Cwd 'chdir';

use EMBLMerger::Merger; # traps errors in Merger module otherwise we just know somethings wrong calling Tbrucei...
use EMBLMerger::Tbrucei;

# Stuff to create EMBL files;
# 
# NOTE: a cut down version of Bioperl exists in coann's perlLib directory.
#
use Bio::Location::Split; # NEEDED ANYMORE?
use Bio::Seq;
use Bio::SeqIO;
use Bio::SeqFeature::Generic;


$|++;


my $debug       = 0;
my $config_file = "../cgi-bin/webconfig.conf";

unless (-s $config_file) {
    print "ERROR: can not locate configuration file.\n";
    exit(1);
}

################################################################################
# Get command line args

my $annot_compare_id = -1;
my $organism         = undef;
my $autoUpdate       = undef;

unless (@ARGV == 3) {
    &usage();
    exit 1;
}


# Arg 1 
if ($ARGV[0] =~ /^\d+$/) {
    $annot_compare_id = $ARGV[0];
}

if ($annot_compare_id == -1) {
    &usage();
    exit 1;
}

# Arg 2
if ($ARGV[1] =~ /^\S+$/) {
    $organism = $ARGV[1]; # Tbrucei, Lmajor etc are perl module names (hence no '.')
}

unless (defined $organism) {
    &usage();
    exit 1;
}

# Arg 3 
if ($ARGV[2] =~ /^\S+$/) {
    if ($ARGV[2] eq 'selected'){
        $autoUpdate = "AND    autoUpdate = 1";
    }
    elsif ($ARGV[2] eq 'all'){
        $autoUpdate = "";
    }
    else {
        &usage();
        exit 1;
    }
}

################################################################################
# THIS IS THE _BASE_ DIRECTORY FOR SPARSE EMBL FILES + MERGED EMBL FILES

#my $OUTPUT_DIR     = "/tmp/EMBL_FILES";
my $OUTPUT_DIR      = "/nfs/pathdb/prod/coann/EMBL_FILES";
my $ORG_OUTPUT_DIR  = "$OUTPUT_DIR/$organism";
my $LOG_OUTPUT_FILE = "$ORG_OUTPUT_DIR/all_changes.log";

################################################################################
# Get options set from http://babel:2233/cgi-bin/admin.cgi

my %params   = readConfig($config_file);
my $db       = $params{MYSQL_DB};
my $user     = $params{MYSQL_READ_ONLY_USER};
my $password = $params{MYSQL_READ_ONLY_PASSWORD};
my $server   = $params{MYSQL_SERVER};


my ($dbproc) = &connect_to_db($server,$db,$user,$password);

#
my $feat_change_hash = undef; # Hold all changes for output to log later on - write_out_changes_log()
#
# The structure loks liek this;
#
# 'Tb927_11_18_20040218.embl' => {
#           '3932140808' => [
#                            'Locus: Tb11.18.0011     39321..40808  (TIGR locus = Tb11.18.0011)',
#                            'Change: product name FROM \'thingy\' TO \'whatsit\'. AutoUpdate = 0'
#                            'Change: EC FROM \'1.2.3.4\' To \'2.3.4.5\'
#                           ],
#           '4203645377' => [
#                            'Locus: Tb11.18.0012     42036..45377  (TIGR locus = Tb11.18.0012)',
#                            'Change: product name FROM \'hypothetical\' TO \'no-way\'. AutoUpdate = 0'
#                           ]
#                                },
# 'Tb927_11_01_20040217.embl' => {..........
#


################################################################################
#
# The logic starts here
#
# Get all updates that requires changes to already existign features,
# not adding/deleting genes just yet.
#
#my %updates   = (); # Key is TIGR_gene_store_id
#                    # Holds details of update
my %seq_cache = (); # This holds a Sequence object per EMBL file name


print "About to call get_changed_qualfiers_and_coords()\n";
&get_changed_qualfiers_and_coords($autoUpdate);

#print "About to call get_new_genes()\n";
#&get_new_genes($autoUpdate); # NOT CALLED, THIS NEEDS TO BE A ARGUMENT ABOVE!!!!!!!!!!!!!!!!!!!!!!!!!!!!

print "About to call get_deleted_genes()\n";
&get_deleted_genes($autoUpdate);

#print "About to call get_resolved_conflicts()\n";
#
#&get_resolved_conflicts();

# DELETE _ALL_ PREVIOUS EMBL FILES
print "Purge org directory of aold files\n";
system("rm", "-rf", $ORG_OUTPUT_DIR) == 0
    or die "Could not recurse delete '$ORG_OUTPUT_DIR'";


print "Creating EMBL files.\n";
&write_out_embl_files($organism, $ORG_OUTPUT_DIR, \%seq_cache);
print "EMBL files created.\n";


&merge_EMBL_files($organism, $ORG_OUTPUT_DIR);
print "EMBL files merged.\n";


&write_out_changes_log($organism, $ORG_OUTPUT_DIR, \%seq_cache);
print "Change log created.\n";


print "Done.\n";


################################################################################
#
# get_changed_qualfiers_and_coords
#
# There are 2 SQL statements. The first gets all changes and can just the ones
# selected OR all of them, the later applying all changes regeardless of a
# human pressing the "auto update" button on the web page.
#
# The second gets conflicts. They MUST be selected as we do not wish to apply
# those changes without human being in the loop. A conflict happens because
# 2 annotators have changed the same thing at the same time (roughly). Which is
# correct?
################################################################################
sub get_changed_qualfiers_and_coords {
    my ($autoUpdate) = @_;

    my $query_changes =
        "SELECT TIGR_gene_store_id, Sanger_gene_store_id, autoUpdate, attribute, ".
        "tigr_value, sanger_value, result_id ".
        "FROM   Gene_comparisons, Attributes_analyzed ".
        "WHERE  Gene_comparisons.attribute_id = Attributes_analyzed.attribute_id ".
        "AND    annot_compare_id = ? ".
        "AND    result_id        = 2 ". # TIGR changed, Sanger should update: SELECT * FROM Comparison_result_type ;
        $autoUpdate;

    my $msg2 = "get_changed_qualfiers_and_coords() about to run *changes* query";
    print STDERR "$msg2\n";

    my @results = &do_sql_2D($dbproc, $query_changes, $annot_compare_id);

    print "get_changed_qualfiers_and_coords(): SQL to get changes complete.\n";

    my $query_conflicts =
        "SELECT TIGR_gene_store_id, Sanger_gene_store_id, autoUpdate, attribute, ".
        "tigr_value, sanger_value, result_id ".
        "FROM   Gene_comparisons, Attributes_analyzed ".
        "WHERE  Gene_comparisons.attribute_id = Attributes_analyzed.attribute_id ".
        "AND    annot_compare_id = ? ".
        "AND    result_id        = 1 ". # Conflict!
        "AND    autoUpdate       = 1 "; # User has selected to apply TIGR version

    my $msg2 = "get_changed_qualfiers_and_coords() about to run *conflicts* query";
    print STDERR "$msg2\n";

    my @results_conflicts = &do_sql_2D($dbproc, $query_conflicts, $annot_compare_id);
    push @results, @results_conflicts; 

    ##
    # build features on sequence using tigr and sanger values stored in the DB
    ##
    foreach my $result (@results) {
        my ($TIGR_gene_store_id,
            $Sanger_gene_store_id,
            $auto_update,
            $attribute,
            $tigr_value,
            $sanger_value,
            $result_id) = @{$result};

        my $change = undef;

        if ($result_id != 1) {
            $change = "Change: $attribute from '$sanger_value' to '$tigr_value'. AutoUpdate = $auto_update\n";
        }
        else {
            $change = "Conflict resolved, change: $attribute from '$sanger_value' to '$tigr_value'. AutoUpdate = $auto_update\n";
        }


        ##
        # get TIGR and Sanger Gene_obj objects  from DB

        my $sql_gene_obj        =
            "SELECT annot_store_id, gene_obj, annot_stamp_id, contig_id, xml_filename ".
            "FROM   Annotation_store WHERE annot_store_id = ?";
        my @gene_results_TIGR   = &do_sql_2D($dbproc, $sql_gene_obj, $TIGR_gene_store_id);
        my @gene_results_Sanger = &do_sql_2D($dbproc, $sql_gene_obj, $Sanger_gene_store_id);
    
        my ($tigr_annot_store_id,
            $tigr_gene_obj,  $tigr_annot_stamp_id,
            $tigr_contig_id, $tigr_xml_filename) =  @{$gene_results_TIGR[0]};

        my ($sanger_annot_store_id,
            $sanger_gene_obj,  $sanger_annot_stamp_id,
            $sanger_contig_id, $sanger_xml_filename) =  @{$gene_results_Sanger[0]};

        my $tigr_obj   = Storable::thaw($tigr_gene_obj);
        my $sanger_obj = Storable::thaw($sanger_gene_obj);

        ##
        # Use Sanger object to get oringal Sanger XML file name, put feature on that sequence
        ##
        #my ($embl_file_name) = ($sanger_xml_filename =~ /(.+?)\.xml$/);
        #$embl_file_name     .= '.embl';

        # Temp fix for dodgy chr 1 embl file name
        #if ($sanger_contig_id eq 'null') {
        #    #die "ERROR: sanger_contig_id = null";
        #    print "WARNING: sanger_contig_id = null, changed to Tb927_01\n";
        #    $sanger_contig_id = 'Tb927_01';
        #}

        my $embl_file_name = $sanger_contig_id . '.embl'; # Contig name and file name should match up

        # DEBUG
        if ($debug) {
            print $change;
            print sprintf(" locus = %15s, TIGR contig   = %-25s, filename = %-20s\n", $tigr_obj->{locus},   $tigr_contig_id, $tigr_xml_filename);
            print sprintf(" locus = %15s, SANGER contig = %-25s, filename = %-20s\n", $sanger_obj->{locus}, $sanger_contig_id, $sanger_xml_filename);
        }

        #
        my ($seq, $feat, $coord_hash) = &get_feature_on_sequence($embl_file_name, \%seq_cache, $sanger_obj);

        unless ($feat){
            # Uses Sanger coords for new feature, alter below if need be...
            $feat = create_feat($seq, $tigr_obj, $sanger_obj, $coord_hash);
        }

        # use TIGR coords?
        if ($attribute =~ /gene coordinates/) {
            my ($strand, $split_location) = &get_split_location_for($tigr_obj);

            #print "Location change: BEFORE: ", $feat->location()->to_FTstring(), "\n";
            $feat->location($split_location);
            #print "Location change: AFTER : ", $feat->location()->to_FTstring(), "\n";
        }
        else {
            add_annotation($feat, $attribute, $tigr_value);
        }

        ##
        # LOG CHANGES
        ##
        my $locus_info = sprintf("Locus: %-15s %6d..%-6d (TIGR locus = %5s)",
                                 $sanger_obj->{locus}, $feat->start(), $feat->end(), $tigr_obj->{locus});
        my $coord      = $feat->start() . $feat->end();

        unless (defined $feat_change_hash->{$embl_file_name}->{$coord}) {
            # [0] = locus name, [1] onwards are the changes (or conflict change)
            $feat_change_hash->{$embl_file_name}->{$coord} = [$locus_info, $change];
        }
        else {
            push @{$feat_change_hash->{$embl_file_name}->{$coord}}, $change;
        }

        print "\n" if $debug;
    }
}

sub location_to_string {
    my ($split_location) = @_;

    my @sublocs = $split_location->sub_Location();
    my $string  = "";

    # print the start/end points of the sub locations
    foreach my $location ( sort { $a->start <=> $b->start } @sublocs ) {
        printf "sub feature %d [%d..%d]\n", $location->start,$location->end;
        $string .= $location->start ."..". $location->end .", ";
    }

    chop($string);
    return $string;
}

################################################################################
#
# get_new_genes
#
# Uses the TIGR contig ID and attempts to convert it to a  Sanger contig ID.
#
#
#
################################################################################
sub get_new_genes {
    my ($autoUpdate) = @_;

    my $query =
        "SELECT TIGR_gene_store_id, Sanger_gene_store_id, autoUpdate, attribute, ".
        "tigr_value, sanger_value ".
        "FROM   Gene_comparisons, Attributes_analyzed ".
        "WHERE  Gene_comparisons.attribute_id = Attributes_analyzed.attribute_id ".
        "AND    annot_compare_id = ? ".
        "AND    result_id        = 5 ". # new TIGR gene
        $autoUpdate;

    my @results = &do_sql_2D($dbproc, $query, $annot_compare_id);

    if (@results) {
        print "**************************\n";
        print "New genes created by TIGR.\n";
        print "**************************\n";
    }
    else {
        print "*******************************\n";
        print "No new genes created by TIGR\n";
        print "*******************************\n";
        return;
    }

    foreach my $result (@results) {
        my ($TIGR_gene_store_id,
            $Sanger_gene_store_id,
            $auto_update,
            $attribute,
            $tigr_value,
            $sanger_value) = @{$result};

        my $sql_gene_obj        =
            "SELECT annot_store_id, gene_obj, annot_stamp_id, contig_id, xml_filename ".
            "FROM   Annotation_store WHERE annot_store_id = ?";
        my @gene_results_TIGR   = &do_sql_2D($dbproc, $sql_gene_obj, $TIGR_gene_store_id);
    
        my ($tigr_annot_store_id,
            $tigr_gene_obj,  $tigr_annot_stamp_id,
            $tigr_contig_id, $tigr_xml_filename) =  @{$gene_results_TIGR[0]};

        my $tigr_obj   = Storable::thaw($tigr_gene_obj);


        ##
        # Try to re-create the Sanger contig ID - no 100% match between TIGR and Sanger names
        ##
        my $sanger_contig_id  = $tigr_contig_id;
        $sanger_contig_id     =~ s/\./_/; # Tb927.1.1 becomes Tb927_1_1
        my ($first, $chromo, $last)  = ($sanger_contig_id =~ /(.+?)_(\d+)_(.+)/);

        if (length($chromo) == 1) {
            #print "Chromo BEFORE: $sanger_contig_id\n ";
            $sanger_contig_id = $first . '_0'. $chromo . '_' .$last;
            #print "Chromo AFTER : $sanger_contig_id\n ";
        }
        else {
            #print "Chromo already 2 digits ($sanger_contig_id)\n";
        }

        my $embl_file_name = $sanger_contig_id . '.embl'; # Contig name and file name should match up

        # DEBUG
        if ($debug) {
            print sprintf(" locus = %15s, TIGR contig   = %-25s, filename = %-20s\n", $tigr_obj->{locus},   $tigr_contig_id, $tigr_xml_filename);
        }

        # It should no be on the sequence unless an error has taken place!
        my ($seq, $feat, $coord_hash) = &get_feature_on_sequence($embl_file_name, \%seq_cache, $tigr_obj);

        unless ($feat){
            # Uses TIGR coords for new feature, hence "$tigr_obj, $tigr_obj"
            $feat = create_feat($seq, $tigr_obj, $tigr_obj, $coord_hash);

            ##
            # Add /product, /pseudo, EC_number, otherwise will have a CDS with coords and a name _only_
            ##
            if (defined($tigr_obj->{'com_name'})) {
                if ($tigr_obj->{'com_name'} !~ /\*to be named\*/) {
                    print "Setting product name to ", $tigr_obj->{'com_name'},"\n" if $debug;
                    add_annotation($feat, 'product name', $tigr_obj->{'com_name'});
                }
                else {
                    #print "Product name not being set, it is: ", $tigr_obj->{'com_name'},"\n";
                }
            }

            if (defined($tigr_obj->{'is_pseudogene'}) && $tigr_obj->{'is_pseudogene'} != 0) {
                print "Setting pseudogene to ", $tigr_obj->{'is_pseudogene'},"\n" if $debug;
                add_annotation($feat, 'pseudogene status', $tigr_obj->{'is_pseudogene'});
            }

            if (defined($tigr_obj->{'ec_num'}) && $tigr_obj->{'ec_num'} != 0) {
                print "Setting EC_number to ", $tigr_obj->{'ec_num'},"\n" if $debug;
                add_annotation($feat, 'ec number', $tigr_obj->{'ec_num'});
            }
        }
        else {
            die "ERROR: feature already on sequence: ", $tigr_obj->{locus}, " - quitting now!";
        }

        add_annotation($feat, $attribute, $tigr_value);        

        ##
        # LOG CHANGES
        ##
        my $locus_info = sprintf("Locus: %-15s %6d..%-6d (TIGR locus = %5s)",
                                 $tigr_obj->{locus}, $feat->start(), $feat->end(), $tigr_obj->{locus});
        my $change     = "*** ADDED ***";
        my $coord      = $feat->start() . $feat->end();

        # [0] = locus name, [1] onwards are the changes
        $feat_change_hash->{$embl_file_name}->{$coord} = [$locus_info, $change];
    }
}


################################################################################
#
# get_deleted_genes
#
#
################################################################################
sub get_deleted_genes {
    my ($autoUpdate) = @_;

    my $query =
        "SELECT TIGR_gene_store_id, Sanger_gene_store_id, autoUpdate, attribute, ".
        "tigr_value, sanger_value ".
        "FROM   Gene_comparisons, Attributes_analyzed ".
        "WHERE  Gene_comparisons.attribute_id = Attributes_analyzed.attribute_id ".
        "AND    annot_compare_id = ? ".
        "AND    result_id        = 7 ". # TIGR gene deleted
        $autoUpdate;

    my @results = &do_sql_2D($dbproc, $query, $annot_compare_id);

    if (@results && $debug) {
        print "*** Genes To Be Deleted ***\n";
    }

    foreach my $result (@results) {
        my ($TIGR_gene_store_id,
            $Sanger_gene_store_id,
            $auto_update,
            $attribute,
            $tigr_value,
            $sanger_value) = @{$result};

        my $sql_gene_obj        =
            "SELECT annot_store_id, gene_obj, annot_stamp_id, contig_id, xml_filename ".
            "FROM   Annotation_store WHERE annot_store_id = ?";
        my @gene_results_Sanger = &do_sql_2D($dbproc, $sql_gene_obj, $Sanger_gene_store_id);

        my ($sanger_annot_store_id,
            $sanger_gene_obj,  $sanger_annot_stamp_id,
            $sanger_contig_id, $sanger_xml_filename) =  @{$gene_results_Sanger[0]};

        my $sanger_obj = Storable::thaw($sanger_gene_obj);

        if ($debug) {
            print sprintf(" locus = %15s, SANGER contig = %-25s, filename = %-20s\n", $sanger_obj->{locus}, $sanger_contig_id, $sanger_xml_filename);
        }


        # Temp fix for dodgy chr 1 embl file name
        #if ($sanger_contig_id eq 'null') {
        #    #die "ERROR: sanger_contig_id = null";
        #    print "WARNING: sanger_contig_id = null, changed to Tb927_01\n";
        #    $sanger_contig_id = 'Tb927_01';
        #}

        my $embl_file_name = $sanger_contig_id . '.embl'; # Contig name and file name should match up

        my ($seq, $feat, $coord_hash) = &get_feature_on_sequence($embl_file_name, \%seq_cache, $sanger_obj);

        unless ($feat){
            # HACK: passes in sanger_obj twice so code does not complain, TIGR obj does not exist!
            $feat = create_feat($seq, $sanger_obj, $sanger_obj, $coord_hash);
            $feat->add_tag_value('DELETED', 'DELETED');
        }
        else {
            my $msg = "ERROR: did not create \$feat for pub_locus =".$sanger_obj->{'pub_locus'}.
                ", locus = ". $sanger_obj->{'locus'};
            print $msg; # This is web based so show user _and_ write to logs (STDOUT & SDTERR)
            die $msg;
        }

        ##
        # LOG DELETION
        ##
        my $locus_info = sprintf("Locus: %-15s %6d..%-6d",
                                 $sanger_obj->{locus}, $feat->start(), $feat->end());
        my $change     = "*** DELETED ***  AutoUpdate = $auto_update";
        my $coord      = $feat->start() . $feat->end();

        unless (defined $feat_change_hash->{$embl_file_name}->{$coord}) {
            # [0] = locus name, [1] onwards are the changes
            $feat_change_hash->{$embl_file_name}->{$coord} = [$locus_info, $change];
        }
        else {
            push @{$feat_change_hash->{$embl_file_name}->{$coord}}, $change;
        }
    }
}


################################################################################
#
# get_feature_on_sequence
#
# Get existing sequence _or_ create it.
# Attempt to retrieve already create feature via coord_hash and return it.
#
#
# Returns: ($seq, $feat, $coord_hash)
#  $seq        : The sequencw
#  $feat       : The feature object on this sequence OR undef if not yet created
#  $coord_hash
#
################################################################################

sub get_feature_on_sequence {
    my ($embl_file_name, $seq_cache, $tigr_obj) = @_;

    # NOTE: for deleted genes the tigr_obj is really the Sanger obj, TIGR gene no longer exists!

    unless (defined $seq_cache->{$embl_file_name}) {
        my $seq = Bio::Seq->new(-seq              => '',
                                -desc             => "$embl_file_name",
                                -display_id       => "$embl_file_name",
                                -accession_number => 'accnum_unknown',
                                -alphabet         => 'dna' );
        $seq_cache{$embl_file_name} = [$seq, {}]; # array containing sequence and coord hash
    }

    # NOTE: coord_hash gets poulated by create_feat()
    my ($seq, $coord_hash) = @{$seq_cache{$embl_file_name}};
    my $feat               = feature_on_seq($tigr_obj, $coord_hash);

    return ($seq, $feat, $coord_hash);
}

################################################################################
#
# feature_on_seq
#
#
# returns: the feature obj if in cache
#          undef if feature not found on sequence
#
################################################################################

sub feature_on_seq {
    my ($tigr_obj, $coord_hash) = @_;

    my $strand = 0;
    $strand    =  1 if $tigr_obj->{strand} eq "+";
    $strand    = -1 if $tigr_obj->{strand} eq "-";

    # get coord of whole feat
    #
    my ($start, $end) = (undef, undef);
    my @exons         = $tigr_obj->get_exons();

    foreach my $exon (@exons) {
        my $cds           = $exon->get_CDS_obj();
        my ($end5, $end3) = $cds->get_coords();


        if ($strand == -1) {
            #($start, $end) = ($end3, $end5);
            unless (defined $start) {
                $start = $end3;
            }

            $end = $end5;
        }
        else {
            unless (defined $start) {
                $start = $end5;
            }

            $end = $end3;
        }
    }

    my $coord = $start.",".$end;

    #my $coord = $feat->start . "," .  $feat->end; # This is what creat_feat does

    #if ($tigr_obj->{'locus'} eq 'Tb03.26J7.570') {
    #    print STDERR "featur_on_seq(): Tb03.26J7.570 \$coord = $coord\n";
    #    print STDERR "featur_on_seq(): Dump pf coord_hash; \n";
    #    print STDERR Data::Dumper::Dumper($coord_hash);
    #}

    if (defined $coord_hash->{$coord}) {
        print "  $tigr_obj [$coord] _IS_ in the coord_cache\n" if $debug;
        return $coord_hash->{$coord};
    }

    #foreach my $key (sort keys %{$coord_hash}){
    #    print "feature_on_seq(): $key != $coord\n";
    #}

    print "  $tigr_obj [$coord] is _NOT_ in the coord_cache\n" if $debug;
    return undef;
}

################################################################################
#
#  add_annotation
#
# Only add tag/value 
################################################################################
sub add_annotation {
    my ($feat, $attribute, $tigr_value) = @_;

    if ($attribute !~ /presence\/absence of gene/ && $attribute !~ /gene coordinates/){
        #my $val = new Bio::Annotation::SimpleValue(-value => $tigr_value); 
        my $semied = $tigr_value;
        $semied    =~ s/$;/; /g;   # separate multi valued attributes by semicolon.

        if ($attribute =~ /product name/) {
            #print STDERR "PRODUCT: \$tigr_value = $tigr_value\n";
            #print STDERR "         \$semied     = $semied\n";

            $feat->add_tag_value('product', $semied); #$tigr_value);
        }

        if ($attribute =~ /pseudogene status/) {
            # If /pseudo needs to be removed "not a pseudogene" will be the value
            $feat->add_tag_value('pseudo', $tigr_value); # Does this need a value at all????
        }

        if ($attribute =~ /gene symbol/) {
            $feat->add_tag_value('primary_name', $semied); #$tigr_value);
        }

        if ($attribute =~ /ec number/) {
            $feat->add_tag_value('EC_number', $semied); #$tigr_value);
        }

        if ($attribute =~ /gene name/) {

            # FIXME - what is this. LOCUS, PUB_LOCUS OR ALT_LOCUS???????? <-----------------------

            # ----- > /synoym ?????????  <------------------------

            $feat->add_tag_value('gene_name', $semied); #$tigr_value);
        }
    }
}

################################################################################
# 
# create_feat
#
# Uses the tigr_obj to create a feature and attach it to the sequence.
# Uses Sanger coords for the feature, TIGR coords _may_ be applied later...
#
#
# It returns a reference to the feature.
#
################################################################################
sub create_feat {
    my ($seq, $tigr_obj, $sanger_obj, $coord_hash) = @_;


    my ($strand, $split_location) = &get_split_location_for($sanger_obj);
 
    my $feat = new Bio::SeqFeature::Generic(-location => $split_location,
                                            -strand   => $strand,
                                            -primary  => 'CDS',
                                            -source   => 'internal' );
    ##
    # These mappings should be retrieved from the annotation.properties file
    # FIXME
    ##
    if (defined $sanger_obj->{'pub_locus'} && $sanger_obj->{'pub_locus'} ne '') {
        $feat->add_tag_value('systematic_id', $sanger_obj->{'pub_locus'});
    }
    elsif (defined $sanger_obj->{'locus'} && $sanger_obj->{'locus'} ne '') {
        $feat->add_tag_value('temporary_systematic_id', $sanger_obj->{'locus'});
    }

    if (defined $sanger_obj->{'alt_locus'} && $sanger_obj->{'alt_locus'} ne '') {
        if ($feat->has_tag('systematic_id') || $feat->has_tag('temporary_systematic_id')) {
            $feat->add_tag_value('previous_systematic_id', $sanger_obj->{'alt_locus'});
        }
        else {
            $feat->add_tag_value('temporary_systematic_id', $sanger_obj->{'alt_locus'});
        }
    }

    ##
    # /note 
    if (defined $tigr_obj->{'pub_comment'} && $tigr_obj->{'pub_comment'} ne '0' && $tigr_obj->{'pub_comment'} ne '') {
        print "Adding /note='", $tigr_obj->{'pub_comment'}, "'\n" if $debug;
        $feat->add_tag_value('note', $tigr_obj->{'pub_comment'});
    }

    ##
    #
    $seq->add_SeqFeature($feat);

    my $coord = $feat->start . "," .  $feat->end;
    $coord_hash->{$coord} = $feat;

    return $feat;
}

################################################################################
# 
# get_split_location_for
#
# For a given Gene_obj it returns a Bioperl representation of the exons
#
# Returns: Bio::Location::Split();
#
################################################################################
sub get_split_location_for {
    my ($gene_obj) = @_;

    my $strand = 0;
    $strand    =  1 if $gene_obj->{strand} eq "+";
    $strand    = -1 if $gene_obj->{strand} eq "-";

    my $split_location = new Bio::Location::Split();
    my @exons          = $gene_obj->get_exons();

    foreach my $exon (@exons) {
        my $cds           = $exon->get_CDS_obj();
        my ($end5, $end3) = $cds->get_coords();
        my ($start, $end) = ($end5, $end3);

        if ($strand == -1) {
            ($start, $end) = ($end3, $end5);
        }

        print "  STRAND: $strand, $start, $end\n" if $debug;

        $split_location->add_sub_Location(new Bio::Location::Simple(-start  => $start,
                                                                    -end    => $end,
                                                                    -strand => $strand));
    }

    return ($strand, $split_location);
}


################################################################################
# 
# write_outembl_files
#
# 
#
################################################################################
sub write_out_embl_files {
    my ($organism, $dir, $seq_cache) = @_;
 
    unless (-e $dir){
        print "Making $dir for EMBL files\n";
        #mkdir $dir;
        my $merger = EMBLMerger::Merger->new();
        $merger->create_directories_for("${dir}/fake_file");
    }

    foreach my $embl_file_name (keys %{$seq_cache}){

        print "Creating ${dir}/${embl_file_name}\n";

        my ($seq, $coord_hash) = @{$seq_cache->{$embl_file_name}};

        #if ($embl_file_name eq 'null.embl') {
        #unless ($embl_file_name) {
        #    my $msg = "    WARNING: CHANGING null to tryp1\n";
        #    print STDERR $msg;
        #    print        $msg;
            #$embl_file_name = 'Tb927_01_01';
        #    $embl_file_name = 'tryp1';
        #}

        my $out = Bio::SeqIO->new('-file'   => ">${dir}/${embl_file_name}",
                                  '-format' => 'EMBL');

        # DEBUG
        #foreach my $feat ( $seq->get_SeqFeatures() ) {
        #    foreach my $ann ($feat->annotation->get_Annotations) {
        #        print "  ", $ann->tagname, " = ",$ann->as_text, "\n";
        #    }
        #}

        $out->write_seq($seq);
    }

    return undef;
}

################################################################################
# 
# write_out_changes_log
#
# 
#
################################################################################
sub write_out_changes_log {
    my ($organism, $dir, $seq_cache) = @_;
 
    unless (-e $dir){
        mkdir $dir;
    }

    open LOG, ">$LOG_OUTPUT_FILE" or die "ERROR: could not open $LOG_OUTPUT_FILE";

    foreach my $embl_file_name (sort keys %{$seq_cache}){

        my ($seq, $coord_hash) = @{$seq_cache->{$embl_file_name}};

        # IS THIS NEEDED ANY MORE? I DON'T THINK SO
        #if ($embl_file_name eq 'null.embl') {
        #    my $msg = "    WARNING: CHANGING null to tryp1\n";
        #    print STDERR $msg;
        #    print        $msg;
        #    $embl_file_name = 'tryp1';
        #}

        print LOG "\n";
        print LOG "#"x80, "\n";
        print LOG "${embl_file_name}\n";
        print LOG "#"x80, "\n";

        my $coord_hash = $feat_change_hash->{$embl_file_name};

        foreach my $coord_key (sort {$a <=> $b} keys %{$coord_hash}) {
            my $array_ref = $coord_hash->{$coord_key};

            foreach my $line (@{$array_ref}) {
                print LOG $line, "\n";;
            }

            print LOG "\n";
        }

        # DEBUG
        #foreach my $feat ( $seq->get_SeqFeatures() ) {
        #    foreach my $ann ($feat->annotation->get_Annotations) {
        #        print "  ", $ann->tagname, " = ",$ann->as_text, "\n";
        #    }
        #}

    }

    print LOG "\n\n*** ALL DONE ***\n";
    close LOG;

    return undef;
}

################################################################################
#
################################################################################
sub usage {
    print "\nUsage: update_embl.pl comparison_number Organism <all|selected>\n";
    print "\nExample: update_embl.pl 1 Tbrucei selected\n";
    print "\nIf you are not sure what numebr to choose, goto;\n\n";
    print "     http://babel:2233/cgi-bin/entryPage.cgi?db=TIGR_SANGER_compare\n\n";
}

################################################################################
#
# merge_EMBL_files
#
# The EMBL files have been created.
#
################################################################################
sub merge_EMBL_files {
    my ($organism, $sparse_embl_dir) = @_;
    
    #local $CWD = $dir;
    #print "I've moved to $CWD to checkout CVS files!!!";


    # Module could;
    #  . Know that the org specific CVS directory structure is
    #  . Handle any specail cases within that structure i.e. Release2, Release3 for T. brucei

    # Strip out '.' from T.brucei - Perl does not like modules names with '.'
    my $org_no_dot    = $organism;
    $org_no_dot       =~ s/\.//;
    my $merger_module = "EMBLMerger::$org_no_dot";

    eval "require $merger_module";

    # This lets us know about compile errors in the Parse modules
    if ($@) {
        print STDERR "ERROR: Could not 'require' $merger_module\n       $@\n";
        print "ERROR: Could not 'require' $merger_module\n       $@";
    }
    else {
        print "\n\nMerging files in : $sparse_embl_dir\n\n";

        # Eval just to capture all bugs and send to STDOUT (webpage)
        eval {

            my $merger = $merger_module->new({
                'sparse_embl_dir' => $sparse_embl_dir,
            });

            $merger->output_version();
            $merger->merge();
        };

        if ($@) {
            print "$@";
            die "$@";
        }
    }

    
}
