#!/usr/local/bin/perl

use lib "perlLib";
use DBI;
use CGI;
use CGI::Carp qw(fatalsToBrowser);
use Mysql_connect;
use strict;
use ConfigFileReader;
use Data::Dumper;

$|++;

unless (-s "webconfig.conf") {
    print "Sorry, cannot locate configuration file.  Please create one by visiting the <a href=\"admin.cgi\">admin </a>page.\n";
    exit(1);
}

my %params = readConfig("webconfig.conf");
my $db = $params{MYSQL_DB};
my $user = $params{MYSQL_READ_ONLY_USER};
my $password = $params{MYSQL_READ_ONLY_PASSWORD};
my $server = $params{MYSQL_SERVER};

my ($dbproc) = &connect_to_db($server,$db,$user,$password);

my $cgi = new CGI;
print $cgi->header('text/html');
my $annot_compare_id = $cgi->param('annot_compare_id');
our $SEE = $cgi->param('SEE') || $cgi->param('DEBUG');
my $updateList = $cgi->param('updateList');

my $attType = $cgi->param('ATT_TYPE');

unless ($db && $annot_compare_id) {
    my %vars = $cgi->Vars();
    print "<pre>Params: " . Dumper (\%params);
    print "CGI vars: " .  Dumper (\%vars);
    
    die "Must set the db & annot_compare_id  parameters.\n";
}


## get URL to link genes.
my $TIGR_geneURLtemplate = $params{TIGR_URL_TEMPLATE};
my $SANGER_geneURLtemplate = $params{SANGER_URL_TEMPLATE};


print $cgi->start_html(-title=>"TIGR/Sanger Annotation Comparison, compare_id: $annot_compare_id");

print <<_EOJS_;

<script language="javascript">
<!--
    function flagChange (row_ID, checkBox) {
	var cellIdentifier = "row_" + row_ID;
	var checked = checkBox.checked;
	// alert ("checked: " + checked);
	
	var myURL;
	
	var now = new Date();
	
	var myTime = now.toLocaleString();
	
	myTime = myTime.replace(/\\W+/g, "_");
	
	if (checked) {
	    myURL = "updateToggle.cgi?row_ID=" + row_ID + "&autoUpdate=1&time=" + myTime;
	    
	} else {
	    myURL = "updateToggle.cgi?row_ID=" + row_ID + "&autoUpdate=0&time=" + myTime;
	    
	}
	
	// alert (row_ID + " " + myURL);
	document.getElementById(cellIdentifier).style.backgroundImage = "url(" + myURL + ")";
	
    }
// -->

</script>

_EOJS_

    ;

## populate controlled vocab items:

#+--------------+--------------------------+
#| attribute_id | attribute                |
#+--------------+--------------------------+
#|            1 | presence/absence of gene |
#|            2 | product name             |
#|            3 | gene coordinates         |
#|            4 | pseudogene status        |
#|            5 | gene symbol              |
#|            6 | ec number                |
#|            7 | gene name                |
#+--------------+--------------------------+


my %attributes;
my $query = "select attribute_id, attribute from Attributes_analyzed";
my @results = &do_sql_2D($dbproc, $query);
foreach my $result (@results) {
    my ($attribute_id, $attribute) = @$result;
    $attributes{$attribute_id} = $attribute;
}

#+-----------+------------------------------+
#| result_id | result_type                  |
#+-----------+------------------------------+  ## Colors:  background   text
#|         1 | conflict                     |              #ff0000      #ffffff
#|         2 | TIGR constant, SANGER update |              
#|         3 | TIGR update, SANGER constant |
#|         4 | same annotation              |
#|         5 | new TIGR gene                |
#|         6 | new SANGER gene              |
#|         7 | TIGR gene deleted            |
#|         8 | SANGER gene deleted          |
#+-----------+------------------------------+





my %result_types;
my $query = "select result_id, result_type from Comparison_result_type";
my @results = &do_sql_2D($dbproc, $query);
foreach my $result (@results) {
    my ($result_id, $result_type) = @$result;
    $result_types{$result_id} = $result_type;
}


## Get annot stamp_ids for comparison.
my $query = "select TIGR_annot_stamp_id, SANGER_annot_stamp_id from Annot_compare where annot_compare_id = $annot_compare_id ";
my @results = &do_sql_2D($dbproc, $query);
unless (@results) {
    print "No data to report.\n";
    exit(0);
}

my ($TIGR_annot_stamp_id, $SANGER_annot_stamp_id) = @{$results[0]};


## Populate list of gene_ids:
my %Gene_IDs;
my $query = "select annot_store_id, gene_id, annot_stamp_id from Gene_identifier where annot_stamp_id in ($TIGR_annot_stamp_id, $SANGER_annot_stamp_id) ";
my @results = &do_sql_2D($dbproc, $query);
foreach my $result (@results) {
    my ($annot_store_id, $gene_id, $annot_stamp_id) = @$result;
    my $gene_key = "$annot_store_id,$annot_stamp_id";
    my $existing_gene_id = $Gene_IDs{$gene_key};
    if ($existing_gene_id) {
	$existing_gene_id .= ",$gene_id";
    } else {
	$existing_gene_id = $gene_id;
    }
    $Gene_IDs{$gene_key} = $existing_gene_id;
}




## Colors:
## Conflict: red background, white text
## Tigr action items: blue, black text
## Sanger action items: green, black text

my $conflictColor = { bg => "#ff0000",
		      text => "#ffffff"};
my $tigrActionItem = { bg => "#89f1ff",
		       text => "#000000"};
my $sangerActionItem = {bg => "#9fff8c",
			text => "#000000"};

# map result_id to color
my %updateColors = ( 1 => $conflictColor,
		     2 => $sangerActionItem,
		     3 => $tigrActionItem,
		     5 => $sangerActionItem,
		     6 => $tigrActionItem,
		     7 => $sangerActionItem,
		     8 => $tigrActionItem );

my %TIGR_must_update = ( #1 => 1, 
			 3 => 1, 6 => 1, 8 => 1);
my %SANGER_must_update = (#1 => 1, 
			  2 => 1, 5 => 1, 7 => 1 );



## Retrieve/display data from database:

print "<CENTER>\n";
print "<h1>Annot_compare_id: $annot_compare_id, Database: $db</h1>\n";


my (%selected, %selectedUpdateList);
$selected{$attType} = "selected";

$selectedUpdateList{$updateList} = "selected";

print <<_EOFORM_;

<form action='showComparisonResults.cgi'>
<TABLE>
<TR>
 <TD>Attributes</TD> <TD>For Center</TD> <TD>&nbsp;</TD>
</TR>
<TR>
<TD>
    <select name='ATT_TYPE' >
    <option $selected{0} value='0'>All</option>
    <option $selected{2} value='2'>Product Names</option>
    <option $selected{7} value='7'>Gene Names</option>
    <option $selected{1} value='1'>Presence/Absence of Gene</option>
    <option $selected{3} value='3'>Gene coordinates</option>
    <option $selected{4} value='4'>Pseudogene status</option>
    <option $selected{5} value='5'>Gene Symbols</option>
    <option $selected{6} value='6'>EC numbers</option>
    </select>
</TD>
<TD>
    <select name='updateList'>
    <option $selectedUpdateList{'all'}     value='all'>All</option>
    <option $selectedUpdateList{'TIGR'}     value='TIGR'>TIGR</option>
    <option $selectedUpdateList{'SANGER'}   value='SANGER'>SANGER</option>
    <option $selectedUpdateList{'CONFLICT'} value='CONFLICT'>CONFLICT</option>
    </select>
</TD>
 <TD> <input type="submit" name="submit" value="Submit"> </TD>
</TR>
<!-- <input type='hidden' name='updateList' value= "$updateList" /> -->
<input type='hidden' name='annot_compare_id' value="$annot_compare_id" />
<input type='hidden' name='db' value="$db" />

</TABLE>
</form>

<BR />

_EOFORM_

    ;



my $query = "select rowID, TIGR_gene_store_id, SANGER_gene_store_id, attribute_id, result_id, tigr_value, sanger_value, autoUpdate from Gene_comparisons where annot_compare_id = $annot_compare_id order by TIGR_gene_store_id";

my @results = &do_sql_2D($dbproc, $query);

if (@results) {

    ##
    # Dodgy select all javascript
    ##
    my @row_ids = ();

    foreach my $result (@results) {

	my ($row_ID, $TIGR_gene_store_id, $SANGER_gene_store_id, $attribute_id, $result_id, $tigr_value, $sanger_value, $autoUpdate) = @$result;

	push @row_ids,$row_ID; 
    }

    print <<_EOJS2_;

<script language="javascript">
<!--
    function flagChangeAll (checkBox) {

    	var checked = checkBox.checked;
	// alert ("checked: " + checked);

        var allRowIds = new Array
        


	var cellIdentifier = "row_" + row_ID;
	
	var myURL;
	
	var now = new Date();
	
	var myTime = now.toLocaleString();
	
	myTime = myTime.replace(/\\W+/g, "_");
	
	if (checked) {
	    myURL = "updateToggle.cgi?row_ID=" + row_ID + "&autoUpdate=1&time=" + myTime;
	    
	} else {
	    myURL = "updateToggle.cgi?row_ID=" + row_ID + "&autoUpdate=0&time=" + myTime;
	    
	}
	
	// alert (row_ID + " " + myURL);
	document.getElementById(cellIdentifier).style.backgroundImage = "url(" + myURL + ")";
	
    }
// -->

</script>

_EOJS2_


    print "<FORM>\n"
        ."<table border=\"1\" width=\"100%\" >\n"
            . "<tr>"
		. "<th>#</th>"
                . "<th>Auto<br>Update</th>"
                    . "<th>TIGR gene_id(s)</th>"
                        . "<th>TIGR value</th>"
                            . "<th>Sanger gene_id(s)</th>"
                                . "<th>Sanger value</th>"
                                    . "<th>Attribute type</th>"
                                        . "<th>Comparison result</th>"
                                            . "</tr>\n";
    

 
    my $count = 0;
    foreach my $result (@results) {

	my ($row_ID, $TIGR_gene_store_id, $SANGER_gene_store_id, $attribute_id, $result_id, $tigr_value, $sanger_value, $autoUpdate) = @$result;
	
	
	$tigr_value = &fit_in_cell($tigr_value);
	$sanger_value = &fit_in_cell($sanger_value);

	## separate mult valued attributes by newlines.
	$tigr_value =~ s/$;/<br><br>/g;
	$sanger_value =~ s/$;/<br><br>/g;
	
	if ($updateList eq 'TIGR' && !$TIGR_must_update{$result_id}) { next;}
	if ($updateList eq 'SANGER' && !$SANGER_must_update{$result_id}) { next;}
	if ($updateList eq "CONFLICT" && $result_id != 1) { next;}
	
	if ($attType && $attribute_id != $attType) { next;}

	$count++;
	my $tigr_gene_ids = &get_gene_ids($TIGR_annot_stamp_id, $TIGR_gene_store_id);
	my $sanger_gene_ids = &get_gene_ids($SANGER_annot_stamp_id, $SANGER_gene_store_id);

	## Take the first gene_id and generate a URL
	my @gene_ids = sort split (/,/, $tigr_gene_ids);
	my $gene_id = shift @gene_ids;

	my $tigr_url = $TIGR_geneURLtemplate;
	$tigr_url =~ s/__GENE_ID__/$gene_id/;
	
	my @sangerGeneIDs = sort split (/,/, $sanger_gene_ids);
	my $sangerGeneID = shift @sangerGeneIDs;
	my $sanger_url = $SANGER_geneURLtemplate;
	$sanger_url =~ s/__GENE_ID__/$sangerGeneID/;
	
	my $colorInfo = $updateColors{$result_id};
	my $bgcolor = $colorInfo->{bg};
	my $textColor = $colorInfo->{text};


	$tigr_gene_ids =~ s/,/,<br> /g;
	$sanger_gene_ids =~ s/,/,<br> /g;

	my ($cellColor, $checked) = ($autoUpdate) ? ("#006666", "checked") : ("#ffffff","");
	
	print "<tr>\n"
	    . "<td>$count</td><td id=\"row_$row_ID\" align=\"center\" bgcolor=\"$cellColor\" ><input type=checkbox $checked onClick=\"flagChange($row_ID, this);\"></td>"
	    . "<td><a href=\'$tigr_url\' target=_blank >$tigr_gene_ids </a> </td>"
		. "<td>$tigr_value</td>"
		    . "<td><a href=\'$sanger_url\' target=_blank >$sanger_gene_ids</a></td>"
		    	. "<td>$sanger_value</td>"
			    ."<td>$attributes{$attribute_id}</td>"
				. "<td bgcolor = \"$bgcolor\" ><font color=\"$textColor\">$result_types{$result_id}"
				    . "</font></td></tr>\n";
	#if ($count > 10) { last;} ## debugging
    }
    print "</table>\n</FORM>\n";

} else {
    print "Sorry, no comparison results for compare_id: $annot_compare_id\n";
}


print $cgi->end_html();


exit(0);




####
sub get_gene_ids {
    my ($annot_stamp_id, $annot_store_id) = @_;
    my $key = "$annot_store_id,$annot_stamp_id";
    return ($Gene_IDs{$key});
}


sub fit_in_cell {
    my ($inString) = shift;
    my @words = split (/\s/, $inString);
    foreach my $word (@words) {
	if (length $word > 25) {
	    $word =~ s/(.{25})/$1 /g;
	}
    }
    my $retString = join (" ", @words);
    
    return ($retString);
}

