################################################################################
#
##

package EMBLMerger::Tbrucei;
@ISA = qw(EMBLMerger::Merger);

use strict;
use Cwd;

################################################################################
#
#
sub initialize {
    my ($self) = @_;

    $self->{'cvs_co_path'} =  'annotation/Trypanosoma/brucei';
}

################################################################################
#
##
sub output_version {
    print 'Using module: $Revision: 1.2 $', "\n";
}

################################################################################
#
# merge - Take sparse EMBL files and merge with cvs files, match on filename
#
#  . cvs co the relavent EMBL files
#  . Merge them based on file name
#
##
sub merge {
    my ($self) = @_;

    my $pwd = getcwd(); #$ENV{'PWD'};

    eval {
        #print "chdir = '", $self->{'sparse_embl_dir'}, "'\n";

        chdir $self->{'sparse_embl_dir'};
        mkdir $self->{'annotation.merged.dir'};

        ##
        # CVS CHECKOUT
        ##
        $self->set_cvs_root();
        my $cvs_co_path = $self->{'cvs_co_path'};

        #system("cvs", "co", $cvs_co_path) == 02004/06/04
        system("cvs", "co", "-D", "June 4, 2004 06:00 GMT", $cvs_co_path) == 0
            or die "Could not checkout $cvs_co_path";

        ##
        # Scan sparse EMBL dir
        ##

        opendir(DIR, '.')
            or die "ERROR: Could not open current directory for reading (", $self->{'sparse_embl_dir'}, " ?)";
        my @sparse_files_arr = grep -T, readdir DIR; # can use -T, in current directory so no need to map path on ro front of file
        close DIR;

        @sparse_files_arr = grep !/annotation_map.properties/, @sparse_files_arr;
        @sparse_files_arr = grep !/no_sequence/, @sparse_files_arr;

        my %sparse_files; # set value to true when it has been processed/merged

        foreach my $sparse_file (@sparse_files_arr) {
            $sparse_files{$sparse_file} = undef;
        }
        
        ##
        # Scan cvs dirs
        ##
        my $cvs_files = get_cvs_dirs(); ###

        foreach my $sparse_file (@sparse_files_arr) {
            print "\n--------------------------------------------------------------------------------\n";
            print "FILE: $sparse_file\n";

            # Strip file extension from sparse file to match key in $cvs_files->{}
            # The extension $ext might not start 'embl', i.e. chr01/tryp1.whole_chrom
            my ($sparse_no_dot, $ext) = ($sparse_file =~ /(.+)\.(.+)/);

            if (defined $cvs_files->{$sparse_no_dot}) {
                print "MATCH: EXACT: '$sparse_file;  to be merged with '", $cvs_files->{$sparse_no_dot},"'\n";

                #  MERGE MERGE MERGE
                my $new_file_path = $cvs_files->{$sparse_no_dot};
                $new_file_path    =~ s/annotation/$self->{'annotation.merged.dir'}/;

                $self->merge_two_files('TRYP', $sparse_file, $cvs_files->{$sparse_no_dot}, $new_file_path);
            }
            else {
                # Perhaps date on end of contig different, it has '_0' on the end of it...
                my ($s1, $s2, $s3) = ($sparse_file =~ /^(.+?)[_|\.](.+?)[_|\.](.+?)[_|\.]/);

                print "$sparse_file = \$s1, \$s2, \$s3 = $s1, $s2, $s3\n";

                unless (defined($s1)) {
                    print "\$s1 was not defined, trying simpler reg exp.\n";
                    ($s1, $s2, $s3) = ($sparse_file =~ /^(.+?)_(.+?)[_|\.](.+?)/);
                    print "\$s1, \$s2, \$s3 = $s1, $s2, $s3\n";
                }

                #unless (defined($s1)) {
                #    print "\$s1 was not defined, trying simpler reg exp.\n";
                #    ($s1, $s2, $s3) = ($sparse_file =~ /^(.+?)_(.+?)\.(.+?)/);
                #    print "\$s1, \$s2, \$s3 = $s1, $s2, $s3\n";
                #}


                # Find $s1_$s2_s3 in hash
                # IF FAIL, find $s1_$s2 in hash, PROVIDING $s3 is non-numeric (not contig number) and only 1 match!
                # yes, this is horrible.
                $sparse_no_dot = only_one_entry_in_hash_for("${s1}_${s2}_${s3}", $cvs_files);

                if ($sparse_no_dot) {
                    #  MERGE MERGE MERGE
                    my $new_file_path = $cvs_files->{$sparse_no_dot};
                    $new_file_path    =~ s/annotation/$self->{'annotation.merged.dir'}/;

                    $self->merge_two_files('TRYP', $sparse_file, $cvs_files->{$sparse_no_dot}, $new_file_path);
                }
                #elsif ($s3 =~ /\d+/) {
                else {
                    $sparse_no_dot = only_one_entry_in_hash_for("${s1}_${s2}", $cvs_files);

                    if ($sparse_no_dot) {
                        #  MERGE MERGE MERGE
                        my $new_file_path = $cvs_files->{$sparse_no_dot};
                        $new_file_path    =~ s/annotation/$self->{'annotation.merged.dir'}/;

                        $self->merge_two_files('TRYP', $sparse_file, $cvs_files->{$sparse_no_dot}, $new_file_path);
                    }
                    else {
                        ## file not processed if it gets here
                        my $msg = "ERROR: '$sparse_file' could not be matched to any file in CVS\n";
                        print STDERR $msg;
                        print        $msg;
                    }
                }
            }
        }
    };

    if ($@) {
        print $@;
        print STDERR $@;
    }

    chdir $pwd;
}

################################################################################
# only_one_entry_in_hash_for
#
# Hack for matching ITGR and Sanger contigs together.
##
sub only_one_entry_in_hash_for {
    my ($contig_name, $cvs_files_hash) = @_;

    # Make array of the keys
    # loop through array, =~ // a match
    # store full key if find a match
    # if find second match, print out warnin and return null
    # return value of $match

    print "only_one_entry_in_hash_for() \$contig_name = $contig_name\n";

    my @cvs_files_array = sort keys %{$cvs_files_hash};
    my $match;

    foreach my $key (@cvs_files_array) {
        if ($key =~ /^$contig_name/) {
            if (defined $match) {
                print STDERR "Found 2 or more matches to '$contig_name' ['$match' and '$key'].";
                return undef;
            }
            else { 
                $match = $key;
            }
        }
    }

    return $match;
}

################################################################################
# T.brucei implementation for gettting CVS EMBL files
#
# NOTE: IGNORE ANYTHING IN 'RELEASE1' DIR
# 
# NOTE: psu_splitter used to create 53 (?) files from one EMBL file thus 1 per contig
#       Igmore anyhting in the "old" directory crete by psu_splitter
#
# Returns hash, the keys being the file names with the extensions stripped off
##

sub get_cvs_dirs {
    my ($self) = @_;

    #&run_psu_splitter();

    my @cvs_files = `find annotation -name '*.embl*' -print`;

    push @cvs_files, "annotation/Trypanosoma/brucei/chr01/tryp1.whole_chrom"; # <---- SPECIAL CASE!!! not called __.embl__


    @cvs_files = grep !/Release1/, @cvs_files; # <------ IGNORE ANYTHING IN 'RELEASE1' DIR!!!
    @cvs_files = grep !/\/old/,    @cvs_files; # <------ IGNORE ANYTHING WITH /old IN IT
    @cvs_files = grep !/\/chr10\/additional_contigs_May04/, @cvs_files; # <------ IGNORE

    my %cvs_files = {};

    foreach my $full_path (@cvs_files){
        chomp($full_path);
        my ($basename)     = ($full_path =~ /\/.+\/(.+?)$/);
        #my ($no_extension) = ($basename  =~ /(.+)\.embl.*$/);# no ./embl___ cos tryp1.whole_chrom
        my ($no_extension) = ($basename  =~ /(.+)\..+?$/);

        print "\$basename = $basename, cvs_files{$no_extension} = $full_path\n";

        $cvs_files{$no_extension} = $full_path;
    }

    return \%cvs_files;
}

################################################################################
#
# THIS SHOULD BE REDUNDANT.
#
################################################################################
sub run_psu_splitter {

    my $old_pwd = getcwd(); #$ENV{'PWD'};

    chdir 'annotation/Trypanosoma/brucei/chr11';

    print STDERR "run_psu_splitter(): pwd = ", getcwd(),"\n";

    system("psu_splitter.pl", "-ossingle",  "Tb927_11_remaining.emblgpi") == 0
        or die "Could not split Tb927_11_remaining.emblgpi";


    chdir $old_pwd;
}

1;
