#!/usr/local/bin/perl

package main;
use strict;
our $SEE;


### TIGR-SANGER Compare (TSC) db API
package TSCdbAPI;
use strict;
use Mysql_connect;
use Gene_obj;
use Storable qw (thaw);

####
sub insert_annotation_version {
    my ($dbproc, $organization, $organism, $version_name) = @_;
    my $query = "insert Annotation_version (organization, version_name, organism, timestamp) values (?,?,?, now() )";
    &RunMod($dbproc, $query, $organization, $version_name, $organism);
    
    my $query = "select LAST_INSERT_ID()";
    my $annot_stamp_id = &very_first_result_sql($dbproc, $query);
    
    return ($annot_stamp_id);

}

####
sub load_annotation_store {
    my ($dbproc, $blob, $annot_stamp_id, $xml_filename, $contig_id) = @_;
    
    my $query = "insert Annotation_store (gene_obj, annot_stamp_id, contig_id, xml_filename) values (?,?,?,?)";
    &RunMod($dbproc, $query, $blob, $annot_stamp_id, $contig_id, $xml_filename);
    
    my $query = "select LAST_INSERT_ID()";
    my $annot_store_id = &very_first_result_sql($dbproc, $query);
    
    return ($annot_store_id);

}


####
sub insert_gene_identifier {
    my ($dbproc, $annot_stamp_id, $annot_store_id, $gene_id) = @_;
    
    my $query = "insert Gene_identifier (annot_stamp_id, annot_store_id, gene_id) values (?,?,?)";
    &RunMod($dbproc, $query, $annot_stamp_id, $annot_store_id, $gene_id);
}


####
sub get_latest_annot_version {
    my ($dbproc, $organization, $organism) = @_;
    my $query = "select annot_stamp_id from Annotation_version where organization = ? and organism = ? order by annot_stamp_id desc" ;
    my $latest_annot_stamp_id = &very_first_result_sql($dbproc, $query, $organization, $organism);

    return ($latest_annot_stamp_id);

}


####
sub get_previous_annot_version {
    my ($dbproc, $organization, $organism) = @_;
    my $query = "select annot_stamp_id from Annotation_version where organization = ? and organism = ? order by annot_stamp_id desc" ;
    my @results = &do_sql_2D($dbproc, $query, $organization, $organism);
    shift @results;
    my $second_entry = shift @results;
    unless (ref $second_entry) {
	die "No second entry available! ??? Error Fatal.\n";
    }
    my $annot_stamp_id = $second_entry->[0];
    return ($annot_stamp_id);
}


####
## Adds the {ANNOT_STORE_ID} attribute to a Gene_obj
sub get_annotated_genes {
    my ($dbproc, $annot_stamp_id) = @_;
    my $query = "select annot_store_id, gene_obj from Annotation_store where annot_stamp_id = ?";
    my @results = &do_sql_2D($dbproc, $query, $annot_stamp_id);
    
    my @gene_objs;
    foreach my $result (@results) {
	my ($annot_store_id, $gene_obj) = @$result;
	my $thawed_gene_obj = thaw($gene_obj);
	$thawed_gene_obj->{ANNOT_STORE_ID} = $annot_store_id;
	push (@gene_objs, $thawed_gene_obj);
    }

    return (@gene_objs);
}

####
sub get_mapping_between_annot_versions_via_identifiers {
    my ($dbproc, $annot_stamp_1, $annot_stamp_2) = @_;
    
    my %mapping;
    
    my $query = "select distinct as1.annot_store_id, as2.annot_store_id from Annotation_store as1, Annotation_store as2, Gene_identifier g1, Gene_identifier g2 where as1.annot_store_id = g1.annot_store_id and g1.gene_id = g2.gene_id and g2.annot_store_id = as2.annot_store_id and as1.annot_stamp_id = $annot_stamp_1 and as2.annot_stamp_id = $annot_stamp_2";

    my @results = &do_sql_2D($dbproc, $query);
    foreach my $result (@results) {
	my ($store_id_1, $store_id_2) = @$result;
	if ( (exists ($mapping{$store_id_1}) && $mapping{$store_id_1} != $store_id_2)
	     ||
	     (exists ($mapping{$store_id_2}) && $mapping{$store_id_2} != $store_id_1) )  
	{
	    die "Multiple mappings for $store_id_1.  Fatal Error. \n";
	}
	$mapping{$store_id_1} = $store_id_2;
	$mapping{$store_id_2} = $store_id_1;
    }

    return (%mapping);
}




####
sub get_new_compare_id {
    my ($dbproc, $TIGR_annot_stamp_id, $SANGER_annot_stamp_id) = @_;
    
    my $query = "insert Annot_compare (TIGR_annot_stamp_id, SANGER_annot_stamp_id, currentTime) values (?,?, now() )";
    &RunMod($dbproc, $query, $TIGR_annot_stamp_id, $SANGER_annot_stamp_id);

    my $query = "select LAST_INSERT_ID()";
    my $annot_compare_id = &very_first_result_sql($dbproc, $query);
    
    return ($annot_compare_id);
}


####
sub insert_gene_comparisons {
    my ($dbproc, $tigr_gene_id, $sanger_gene_id, $attribute_id, $result_id, $annot_compare_id, $tigr_value, $sanger_value) = @_;
    
    my $query = "insert Gene_comparisons (TIGR_gene_store_id, SANGER_gene_store_id, attribute_id, result_id, annot_compare_id, tigr_value, sanger_value) values (?,?,?,?,?,?,?)";
    &RunMod($dbproc, $query, $tigr_gene_id, $sanger_gene_id, $attribute_id, $result_id, $annot_compare_id, $tigr_value, $sanger_value);
    
}














1; #EOM
