/*
 * This file is part of Gemini. Copyright (c) 2001, 2002 Genome
 * Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.psu.gemini;

import java.awt.Color;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.SwingConstants;

import org.xml.sax.SAXException;

import uk.ac.sanger.psu.gemini.gui.GeminiUI;

/**
 * <code>GeminiApp</code> is the application proper and contains
 * instances of the <code>DataHandler</code> and <code>GeminiUI</code>
 * classes. Its function is to coordinate loading and parsing of the
 * input data by the <code>DataHandler</code>, its display and
 * manipulation by the <code>GeminiUI</code> and the display of
 * information, warning and error dialogs if exceptions are thrown.
 *
 * @author Keith James
 */
public final class GeminiApp
{
    private static final String TITLE =
        Gemini.PROGRAM + " " + Gemini.VERSION;

    private GeminiUI    geminiUI;
    private JDialog     progress;

    /**
     * Creates a new <code>GeminiApp</code>.
     *
     * @exception Exception if an error occurs.
     */
    public GeminiApp() throws Exception
    {
        geminiUI    = new GeminiUI(TITLE);
    }

    public void startUI()
    {
        geminiUI.setSize(600, 600);
        geminiUI.setVisible(true);
    }
}
