/*
 * This file is part of Gemini. Copyright (c) 2001, 2002 Genome
 * Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.psu.gemini.gui;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.border.BevelBorder;
import javax.swing.border.Border;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;

import uk.ac.sanger.util.gui.ImageFactory;
import uk.ac.sanger.util.gui.ImageFactoryDefaultImpl;

import uk.ac.sanger.psu.gemini.Gemini;

/**
 * <code>AboutDialog</code> is a simple information dialog.
 *
 * @author Keith James
 */
final class AboutDialog extends JDialog
{
    // Images from central image factory
    private ImageFactory imFactory;
    private Image geminiLogo;

    /**
     * Creates a new <code>AboutDialog</code>.
     *
     * @param geminiUI a <code>GeminiUI</code> parent.
     *
     * @exception Exception if an error occurs.
     */
    public AboutDialog(GeminiUI geminiUI) throws Exception
    {
        super(geminiUI, "About " + Gemini.PROGRAM, true);
        initComponents();
    }

    private void initComponents() throws Exception
    {
        imFactory = ImageFactoryDefaultImpl.getInstance();
        geminiLogo = imFactory.createImage("gemini_small");

        JPanel p = new JPanel();
        JLabel logo = new JLabel(new ImageIcon(geminiLogo));
        Border bb = new BevelBorder(BevelBorder.LOWERED);
        Border eb = new EmptyBorder(5, 5, 5, 5);
        logo.setBorder(new CompoundBorder(bb, eb));
        p.add(logo);
        getContentPane().add(p, BorderLayout.WEST);

        p = new JPanel();
        p.setLayout(new BoxLayout(p, BoxLayout.Y_AXIS));

        String message = Gemini.PROGRAM
            + " "
            + Gemini.VERSION
            + "\n"
            + Gemini.AUTHOR
            + "\n"
            + Gemini.AFFILIATION
            + "\n"
            + "Copyright 2001, 2002 Genome Research Limited.\n\n"
            + "Icons are provided by the KDE Project (www.kde.org)\n"
            + "and are copyright of their respective authors.\n\n"
            + "Created using the BioJava library (www.biojava.org)";

        JTextArea info = new JTextArea();
        info.setBackground(getBackground());
        info.setBorder(new EmptyBorder(5, 10, 5, 10));
        info.setFont(new Font("helvetica", Font.BOLD, 12));
        info.setText(message);
        info.setEditable(false);
        p.add(info);

        message = "An interactive pairwise sequence comparison viewer\n"
            + "with integral annotation display.";

        JTextArea detail = new JTextArea();
        detail.setBackground(getBackground());
        detail.setBorder(new EmptyBorder(5, 10, 5, 10));
        detail.setFont(new Font("helvetica", Font.PLAIN, 12));
        detail.setText(message);
        p.add(detail);

        getContentPane().add(p, BorderLayout.CENTER);

        p = new JPanel();
        JButton okAbout = new JButton("OK");
        okAbout.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent ae)
                {
                    setVisible(false);
                }
            });
        p.add(okAbout);
        getContentPane().add(p, BorderLayout.SOUTH);
        pack();
        setResizable(false);

        Rectangle screen = getGraphicsConfiguration().getBounds();
        setLocation((screen.width - getWidth()) / 2,
                    (screen.height - getHeight()) / 2);
    }
}
