/*
 * This file is part of Gemini. Copyright (c) 2001, 2002 Genome
 * Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.psu.gemini.gui;

import java.awt.Toolkit;
import java.text.NumberFormat;
import java.text.ParseException;
import javax.swing.JTextField;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.PlainDocument;

/**
 * <code>IntegerField</code> is a text field which only accepts
 * integer values.
 *
 * @author Keith James
 */
final class IntegerField extends JTextField
{
    private NumberFormat integerFormat;
    private Toolkit toolkit;

    public IntegerField()
    {
        integerFormat = NumberFormat.getNumberInstance();
        integerFormat.setParseIntegerOnly(true);
        toolkit = Toolkit.getDefaultToolkit();
    }

    public int getValue()
    {
        int val = 0;

        try
        {
            val = integerFormat.parse(getText()).intValue();
        }
        catch (ParseException pe)
        {
            toolkit.beep();
        }

        return val;
    }

    public void setValue(int value)
    {
        setText(integerFormat.format(value));
    }

    protected Document createDefaultModel()
    {
        return new IntegerDocument();
    }

    protected class IntegerDocument extends PlainDocument
    {
        public void insertString(int offset, String s, AttributeSet as) 
            throws BadLocationException
        {
            char [] source = s.toCharArray();
            char [] dest = new char[source.length];

            int j = 0;
            for (int i = 0; i < source.length; i++)
            {
                if (Character.isDigit(source[i]))
                    dest[j++] = source[i];
                else
                    toolkit.beep();
            }

            super.insertString(offset, new String(dest, 0, j), as);
        }
    }
}
