/*
 * This file is part of Gemini. Copyright (c) 2001, 2002 Genome
 * Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.psu.gemini.gui;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.io.BufferedOutputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

import com.sun.image.codec.jpeg.JPEGCodec;
import com.sun.image.codec.jpeg.JPEGEncodeParam;
import com.sun.image.codec.jpeg.JPEGImageEncoder;

import org.biojava.bio.gui.sequence.PairwiseSequencePanel;
import org.biojava.bio.gui.sequence.TranslatedSequencePanel;

/**
 * <code>JPEGExporter</code> exports an image of the application to a
 * JPEG format image file.
 *
 * @author Keith James
 */
class JPEGExporter extends AbstractExporter
{
    /**
     * Creates a new <code>JPEGExporter</code> for the specified UI.
     *
     * @param geminiUI a <code>GeminiUI</code>.
     */
    JPEGExporter(GeminiUI geminiUI)
    {
        super(geminiUI);
    }

    public void run()
    {
        try
        {
            exportRasterImage();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    private void exportRasterImage() throws IOException
    {
        if (exportFile == null)
            throw new FileNotFoundException("The export file was set to null");

        PairwiseSequencePanel    plotPanel = geminiUI.getPairwisePanel();
        TranslatedSequencePanel horizPanel = geminiUI.getXSequencePanel();
        TranslatedSequencePanel  vertPanel = geminiUI.getYSequencePanel();

        int plotWidth = plotPanel.getWidth();
        int vertWidth = vertPanel.getWidth();
        int  plotHeight = plotPanel.getHeight();
        int horizHeight = horizPanel.getHeight();

        BufferedImage bi = new BufferedImage(plotWidth + vertWidth,
                                             plotHeight + horizHeight,
                                             BufferedImage.TYPE_INT_RGB);

        Graphics2D g2 = bi.createGraphics();
        g2.setColor(Color.white);
        g2.fillRect(0, 0, plotWidth + vertWidth, plotHeight + horizHeight);
        paintSections(g2);

        BufferedOutputStream bo =
            new BufferedOutputStream(new FileOutputStream(exportFile));
        JPEGImageEncoder encoder = JPEGCodec.createJPEGEncoder(bo);
        JPEGEncodeParam jep = encoder.getDefaultJPEGEncodeParam(bi); 
        jep.setQuality(0.9F, false);

        encoder.encode(bi, jep);
    }
}
