/*
 * This file is part of Gemini. Copyright (c) 2001, 2002 Genome
 * Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.psu.gemini.gui;

import java.io.*;

import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;

import org.apache.batik.dom.GenericDOMImplementation;
import org.apache.batik.svggen.SVGGraphics2D;
import org.apache.batik.svggen.SVGGraphics2DIOException;

import org.biojava.bio.gui.sequence.PairwiseSequencePanel;
import org.biojava.bio.gui.sequence.TranslatedSequencePanel;

import uk.ac.sanger.psu.gemini.gui.GeminiUI;

/**
 * <code>SVGExporter</code> exports an image of the application to an
 * SVG format XML file. Requires the Batik SVG toolkit.
 *
 * @author Keith James
 */
class SVGExporter extends AbstractExporter
{
    /**
     * Creates a new <code>SVGExporter</code> for the specified UI.
     *
     * @param geminiUI a <code>GeminiUI</code>.
     */
    SVGExporter(GeminiUI geminiUI)
    {
        super(geminiUI);
    }

    public void run()
    {
        try
        {
            exportSVG();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    private void exportSVG()
        throws FileNotFoundException, UnsupportedEncodingException,
               SVGGraphics2DIOException
    {
        if (exportFile == null)
            throw new FileNotFoundException("The export file was set to null");

        DOMImplementation domImpl =
            GenericDOMImplementation.getDOMImplementation();

        Document document = domImpl.createDocument(null, "svg", null);
        SVGGraphics2D svgGenerator = new SVGGraphics2D(document);
        paintSections(svgGenerator);

        // Use UTF-8 character-to-byte encoding and CSS
        boolean useCSS = true;

        BufferedOutputStream bo =
            new BufferedOutputStream(new FileOutputStream(exportFile));
        Writer out = new OutputStreamWriter(bo, "UTF-8");
        svgGenerator.stream(out, useCSS);
    }
}
