#!/usr/local/bin/perl5.6.1 -w

use strict;
use Getopt::Std;

use Bio::PSU::SeqFactory;
use Bio::PSU::Seq;

my $_debug = 0;

my (%opts);
my ($fasta, $tab, $output) = undef;
getopts('s:t:o:', \%opts);
defined $opts{s} and $fasta  = $opts{s};
defined $opts{t} and $tab = $opts{t};
defined $opts{o} and $output = $opts{o};

if (not (defined $fasta && defined $tab)) {
  die "Usage :
addSequence_to_embl.pl -s [sequences in fasta file] -t [input tab file] -o [output tab file]
";
}

my $sinfasta = Bio::PSU::SeqFactory->make(
					  -file   => "$fasta",
					 );
my $sintab   = Bio::PSU::SeqFactory->make(
					  -file   => "$tab",
					 );
my $sout;
if (defined $output) {
  $sout = Bio::PSU::SeqFactory->make(
				     -file   => ">$output",
				     -format => 'embl'
				    );
}
else {
  $sout = Bio::PSU::SeqFactory->make(
				     -fh   => \*STDOUT,
				     -format => 'embl',
				    );
}

# Fasta file processing

if ($_debug) {
  print STDERR "Fasta file processing...\n";
}

my %seqs;
while (my $seqobj = $sinfasta->next_seq ()) {
  my $seqid  = $seqobj->id;
  my $seqstr = $seqobj->str;

  $seqs{$seqid} = $seqstr;
}

# Initialise this hash to figure out later which sequences are missing from the output embl (ie those without any features) and add the missing sequence entries in the output file.

my %seqs_processed;
my @sequence_ids = keys (%seqs);
foreach my $seqid (@sequence_ids) {
  $seqs_processed{$seqid} = 0;
}

if ($_debug) {
  print STDERR "Fasta file processing done!\n";
}

# Tab file processing

if ($_debug) {
  print STDERR "tab file processing...\n";
}

while (my $seqobj = $sintab->next_seq ()) {
  my $seqid  = $seqobj->id;
  my $seqstr = $seqs{$seqid};

  my $newseqobj = $seqobj->clone (-str => $seqstr);

  $sout->write_seq ($newseqobj);

  $seqs_processed{$seqid} = 1;

}

# Now for all the sequences which are missing in the output file, add this missing sequence entry

foreach my $seqid (@sequence_ids) {
  if ($seqs_processed{$seqid} == 0) {
    my $seqstr = $seqs{$seqid};
    my $newseqobj = Bio::PSU::Seq->new (
					-id  => $seqid,
					-str => $seqstr,
				       );
    $sout->write_seq ($newseqobj);
  }
}

if ($_debug) {
  print STDERR "tab file processing done!\n";
}

# Done
