#!/usr/local/bin/perl5.6.1 -w

=head1 NAME

 annotate_by_tribe - create MSPCrunch file showing how annotation
		 needs transferring, due to tclustering results

=head1 SYNOPSIS

Examples:

 annotate_by_tribe seq_embl_file mspcrunch_file

=head1 DESCRIPTION

This scribt takes an EMBL file that has been annotated with cluster
qualifiers by the tribe2embl script.  Finds members from a cluster
and finds any product lines within the cluster.  Writes out an 
MSPCrunch format file with the product-annotated id against
each of the other members in the cluster.

The command line requires a sequence EMBL file that has been
annotated with cluster qualifiers and a filename for the crunch
file to be written out to.

=head1 METHODS

None

=head1 AUTHOR

Ellen Schofield (es2@sanger.ac.uk)

=head1 COPYRIGHT

Copyright (C) 2002 Genome Research Limited. All Rights Reserved.

=head1 DISCLAIMER

This software is provided "as is" without warranty of any kind. It may
be used, redistributed and/or modified under the same conditions as
Perl itself.

=cut

use strict;
use Bio::SeqIO;

my $usage = "usage : $0 <IN-embl_contig_file> <OUT-mspcrunch_format_file_of_genes_to_transfer_annotation\n";

my $embl_file = shift or die $usage;
my $crunch_file = shift or die $usage;

my $embl_in = Bio::SeqIO->new(-file => "$embl_file" , '-format' => 'EMBL');
open (FILEOUT, ">$crunch_file");

my %cluster_hash;
my %gene_info;

while (my $seq = $embl_in->next_seq){
    foreach my $feat ($seq->all_SeqFeatures){
	    
	next unless ($feat->primary_tag eq 'CDS');
	next unless ($feat->has_tag('cluster'));

	my @tags = $feat->get_tag_values('temporary_systematic_id');
	my $name = $tags[0];

	my @clusters = $feat->get_tag_values('cluster');
	my ($cluster) = $clusters[0] =~ /:(\d+)\s/;

	my @products;
	if ($feat->has_tag('product')){
	    @products = $feat->get_tag_values('product');
	}
	else{
	    @products = "none";
	}
	
	$cluster_hash{$cluster}{$name} = $products[0];

	my $start;
	my $end;
	my $strand;
	
	if ($feat->location->isa('Bio::Location::SplitLocationI')){
	    $start = $feat->location->start;
	    $end =  $feat->location->end;
	    $strand = $feat->location->strand;
	}
	else{
	    $start = $feat->start;
	    $end = $feat->end;
	    $strand = $feat->strand;
	}
	
	$gene_info{$name} = {"Start" => $start,
			     "End" => $end,
			     "Strand" => $strand};
    }
}

my %hash;

foreach my $cluster (sort by_number keys %cluster_hash){
    my ($g1_name, $g1_start, $g1_end) = ""; 
    my ($g2_name, $g2_start, $g2_end) = ""; 
    my $product;

    foreach my $name (keys %{$cluster_hash{$cluster}}){
	if ($g1_name eq "" && $cluster_hash{$cluster}{$name} ne "none"){
	    $g1_name = $name;
	    $product = $cluster_hash{$cluster}{$name};
	    $g1_start = $gene_info{$g1_name}{"Start"};
	    $g1_end = $gene_info{$g1_name}{"End"};
	}
    }
    
    next if ($g1_name eq "");
    print "$cluster : $product\n";

    foreach my $name (keys %{$cluster_hash{$cluster}}){
	$g2_name = $name;
	next if ($g2_name eq $g1_name);
	$g2_start = $gene_info{$g2_name}{"Start"};
	$g2_end = $gene_info{$g2_name}{"End"};
	print FILEOUT "100.0 100 $g1_start $g1_end $g1_name $g2_start $g2_end $g2_name\n";
    }
}







sub by_number
{$a <=> $b}
