#!/usr/local/bin/perl

# External Modules

# Constraint variables declaration before using them
use strict;

use Cwd;

# Bio modules
# PSU
use Bio::PSU::SeqFactory;

# Benchmark Module
use Benchmark;
##################################################################

=head1 NAME

 - clustering_report.pl input_fasta_file 

=head1 SYNOPSIS


=head1 DESCRIPTION

  this script works only to report phrap assembly
  @author axk

=cut

my $t1 = Benchmark->new ();

my $_contig  = "CONTIG.";
my $_singlet = "SINGLET";

my $input_name = "";
my $output_name = "report.txt";
my @bounce = (99, 499, 999, 1499, 1999);

# Check if in the command line there is an input file name
my $lgth = @ARGV + 0;
# if no args, display the help message
if ($lgth == 0 || $lgth > 1) {
  print "specify an input fasta file and no more than one argument\n";
  print "the results file will be automatically picked up - [fasta file].results\n";
  exit 1;
}
else {
  $input_name = $ARGV[0];
}
my $curdir = cwd;
my $input_name_path = $curdir . "/" . $input_name;

my $nb_seq = qx/cat $input_name_path | grep \">\" | wc -l/;
$nb_seq = $nb_seq + 0;

my $nb_char = qx/cat $input_name_path | grep -v \">\" | wc -c/;
# $nb_char = $nb_char + 0;
my $nb_line = qx/cat $input_name_path | grep -v \">\" | wc -l/;
# $nb_line = $nb_line + 0;
my $nb_bases = $nb_char - $nb_line;

print STDOUT "\nhere the results : \n";

print STDOUT "clustering of a set of $nb_seq sequences ($nb_bases bases)\n";
print STDOUT "\t * unmasked :\n";

my ($nb_contigs_unmasked, @report_unmasked) = reporting ($input_name_path, @bounce);

# if masking give the same statistics ....

my $input_name_masked = $input_name . ".masked";
my $input_name_masked_path = $curdir . "/" . $input_name_masked;
my $nb_contigs_masked = 0;
my @report_masked = ();
if (-f $input_name_masked) {
  print STDOUT "\t * masked :\n";
  ($nb_contigs_masked, @report_masked) = reporting ($input_name_masked_path, @bounce);
}
else {
  print STDOUT "\t * masked : no report available yet\n\n";
}

print STDOUT "size distribution of the contig consensus sequences (bp) - unmasked: \n";
display_report ($nb_contigs_unmasked, \@report_unmasked, \@bounce);
if (-f $input_name_masked) {
  print STDOUT "size distribution of the contig consensus sequences (bp) - masked: \n";
  display_report ($nb_contigs_masked, \@report_masked, \@bounce);
}
my $t2 = Benchmark->new ();

print STDERR "\nprocessing completed : ", timestr (timediff ($t2, $t1)), "\n";

#######################################################
# END
#######################################################

sub reporting {
  my ($input_name_path, @bounce) = @_;

  my $nb_contigs = qx/cat $input_name_path.results | grep \">\" | grep "$_contig" | wc -l/;
  $nb_contigs = $nb_contigs + 0;
  my $nb_singlets = qx/cat $input_name_path.results | grep \">\" | grep "$_singlet" | wc -l/;
  $nb_singlets = $nb_singlets + 0;
  
  my $pattern = "$_contig" . "1";

  my $line = qx/cat $input_name_path.results|grep \">\" | grep "$pattern"/;
  $line =~ />\D+\d+\s+(\d+)\s+.+/;
  my $nb_members_min = $1;

  $pattern = "$_contig" . "$nb_contigs";
  
  $line = qx/cat $input_name_path.results|grep \">\" | grep "$pattern"/;
  $line =~ />\D+\d+\s+(\d+)\s+.+/;
  my $nb_members_max = $1;
  
  my $nb_mb_per_contig = ($nb_seq-$nb_singlets)/$nb_contigs;
  my $singlets_percent = ($nb_singlets*100)/$nb_seq;

  print STDOUT "\t\t * $nb_contigs clusters - ", sprintf ("%.2f", $nb_mb_per_contig), " members in average foreach cluster (from $nb_members_min to $nb_members_max members)\n";
  print STDOUT "\t\t * $nb_singlets singletons (", sprintf ("%.2f", $singlets_percent), " %)\n"; 

  # counting

  print STDERR ("distribution processing...\n");

  my $seqi = Bio::PSU::SeqFactory->make(-file   => "<$input_name_path.contigs",
					-format => 'fasta');
  my $i = 0;
  my $percent_memorized = -1;
  
  my @report = (0, 0, 0 ,0 , 0, 0);
  
  while (my $sequence = $seqi->next_seq) {
    
    # print report - the percentage 0% 5% 10%...
    
    my $current = int ($i*100/$nb_contigs);
    if (($current % 5) == 0 && $current != $percent_memorized) {
      print STDERR "$current% ";
      $percent_memorized = $current;
    }
    
    my $lgth_seq = length ($sequence->str);
    if ($lgth_seq < $bounce[0]) {
      $report[0] ++;
    }
    elsif ($lgth_seq < $bounce[1]) {
      $report[1] ++;
    }
    elsif ($lgth_seq < $bounce[2]) {
      $report[2] ++;
    }
    elsif ($lgth_seq < $bounce[3]) {
      $report[3] ++;
    }
    elsif ($lgth_seq < $bounce[4]) {
      $report[4] ++;
    }
    else {
      $report[5] ++;
    }
    $i++;
  }
  print STDERR "100%\n";

  return ($nb_contigs, @report);
  
}


sub display_report {

  my ($nb_contigs, $report_ref, $bounce_ref) = @_;
  
  my @report = @$report_ref;
  my @bounce = @$bounce_ref;
  
  print STDOUT "Size 0-99 bp\t\t: $report[0] contigs\n";
  print STDOUT "Size 100-499 bp\t\t: $report[0] contigs\n";
  print STDOUT "Size 500-999 bp\t\t: $report[1] contigs\n";
  print STDOUT "Size 1000-1499 bp\t: $report[2] contigs\n";
  print STDOUT "Size 1500-1999 bp\t: $report[3] contigs\n";
  print STDOUT "Size more than 2kb\t: $report[4] contigs\n";
  print STDOUT "Overall\t\t\t  $nb_contigs contigs\n";
}
