#!/usr/local/bin/perl5.6.0 -w

=head1 NAME

create_ids - Creates identifier strings for EMBL features

=head1 SYNOPSIS

create_ids < <filename>

=head1 DESCRIPTION

This script takes EMBL files or feature tables on STDIN and creates
IDs for all requested features. IDs consist of a base string with a
numeric extension.

=head1 METHODS

None

=head1 AUTHOR

Keith James (kdj@sanger.ac.uk)

=head1 COPYRIGHT

Copyright (C) 2002 Genome Research Limited. All Rights Reserved.

=head1 DISCLAIMER

This software is provided "as is" without warranty of any kind. It may
be used, redistributed and/or modified under the same conditions as
Perl itself.

=cut

use strict;
use Getopt::Std;
use Bio::PSU::SeqFactory;

my (%opts, $key, $qual, $base);
my $usage = "Usage: create_ids -k <key> -q <qualifier> -b <base_string> < <filenames>

\t-k Key of features to have identifier added
\t-q Qualifier under which to add the new identifier
\t-b Base string for new identifier
";

getopts('hb:k:q:', \%opts);

defined $opts{h} and die $usage;
defined $opts{k} and $key  = $opts{k};
defined $opts{q} and $qual = $opts{q};
defined $opts{b} and $base = $opts{b};

unless (defined $key && defined $qual && defined $base)
{
    die $usage;
}

my $ei = Bio::PSU::SeqFactory->make(-fh     => \*STDIN,
                                    -format => 'embl');
my $eo = Bio::PSU::SeqFactory->make(-fh     => \*STDOUT,
                                    -format => 'embl');

my $count = 0;

while (my $seq = $ei->next_seq)
{
	if ($seq->has_features)
    {
        foreach my $f ($seq->features)
        {
            $count = ("0" x (6 - length $count)) . $count;
            $f->qadd("$qual", "$base.$count");
            $count++;
        }
    }

    $eo->write_seq($seq);
}
