#!/usr/local/bin/perl -wT

use strict;

# $Header: /nfs/pathsoft/repository/psu/genexec/perl/src/fasta_search.pl,v 1.3 1999/10/20 13:31:03 kmr Exp $

# purpose: This script will grep a fasta database file for a regular expression
# purpose: given on the command line.  If a match is found the whole record
# purpose: is printed.  Each record is treated as a contiguous sequence so
# purpose: matches can span lines.

sub usage
{
  die <<EOF;
usage: $0 -help (for help)
   or: $0 [-i] [-h] [-s] [-H] <search_regular_expression> <fasta_file> ...
Search a fasta database and display those records that match the given regular
expression.  

Flags: -i will cause the case to be ignored when searching.
       -h search the header but not the sequence for the regular expression
       -s search the sequence but not the header (this is the default)
       -H write only the header of the matching record
EOF
}

my $ignore_case = 0;
my $write_header = 1;
my $write_sequence = 1;
my $search_header = 0;
my $search_sequence = 1;

while (@ARGV && $ARGV[0] =~ /^-(.*)$/) {
  shift;
  $ignore_case = 1,                         next if ($1 eq 'i');
  $search_header = 1, $search_sequence = 0, next if ($1 eq 'h');
  $search_header = 0, $search_sequence = 1, next if ($1 eq 's');
  $write_header = 1, $write_sequence = 0,   next if ($1 eq 'H');

  usage;
}

if (!@ARGV) {
  usage;
}

my $search_re = $ARGV[0];


eval {
  "foo" =~ /$search_re/;
};

if (defined $@ && length $@) {
  my $message = $@;

  $message =~ s/ at.*//;

  die "illegal search pattern: $message\n";
}

shift;

my $header;

my $body;
my $body_no_linebreak;

while (<>) {
  if (/^>/) {
    check_record ($header, $body, $body_no_linebreak);

    undef $body;
    undef $body_no_linebreak;

    $header = $_;
  } else {
    if (defined $header) {
      $body .= $_;
      chomp $_;
      $body_no_linebreak .= $_;
    } else {
#      warn "input file seems to contain a record without a header at line $.\n";
    }
  }
}

check_record ($header, $body, $body_no_linebreak);

sub check_record
{
  my ($header, $body, $body_no_linebreak) = @_;

  if (defined $body) {
    if ($search_sequence &&
        ($ignore_case && $body_no_linebreak =~ m"$search_re"i ||
         ! $ignore_case && $body_no_linebreak =~ m"$search_re") ||
        $search_header &&
        ($ignore_case && $header =~ m"$search_re"i ||
         ! $ignore_case && $header =~ m"$search_re")) {
      print $header if ($write_header);
      print $body if ($write_sequence);
    }
  } else {
    if (defined $header) {
      warn "input file seems to contain an empty record: $header\n";
    }
  }
}
