#!/usr/local/bin/perl5.6.1 -w

# External Modules

# Constraint variables declaration before using them
use strict;
##########################################

use Data::Dumper;

# Benchmark Module
use Benchmark;
##################################################################

###################
# PSU Bio Modules
##
use Bio::PSU::Feature;
use Bio::PSU::SeqFactory;
use Bio::PSU::Range;
use Bio::PSU::Utils::FeatureIdHelper::BioPSUHelper;

eval "require Bio::PSU::Utils::GeneAssociationParser";
# use Bio::PSU::Utils::GeneAssociationParser;

##########################################
=pod

=item * Configuration loading from the command line

=cut
##########################################

my $t1 = Benchmark->new ();

############################
# Debug boolean
##

my $_debug = 0;

my $_help   = "-help";
my $_tab    = "-tab";
my $_ga     = "-ga";
my $_go_numbers = "-n";
my $_regex  = "-r";
my $_remove = "-rm";
# no gene association and annotation files per default
my $tab = "-1";
my $gene_association = "-1";
my $go_numbers_file  = "/nfs/pathsoft/databases/go_cvs/numbers/go_numbers";
my $regex;
my $remove = 0;

my $i = 0;
foreach my $element (@ARGV) {
  if ($element eq $_help) {
    display_help ();
    exit 1;
  }
  elsif ($element eq $_tab) {
    $tab = $ARGV[$i+1];
  }
  elsif ($element eq $_ga) {
    $gene_association = $ARGV[$i+1];
  }
  elsif ($element eq $_go_numbers) {
    $go_numbers_file = $ARGV[$i+1];
  }
  elsif ($element eq $_regex) {
    $regex = $ARGV[$i+1];
  }
  elsif ($element eq $_remove) {
    $remove++;
  }
  # if at least one of the options is not recognized, display the help message and exit
  elsif ($element =~ /^-/) {
    print STDERR "option not recognized: $element\n";
    display_help ();
    exit 1;
  }
  $i++;
}

if (($tab eq "-1") || ($gene_association eq "-1")) {
  print STDERR "option(s) not specified!!\n";
  display_help ();
  exit 1;
}

if ($remove) {
  if ($_debug) {
    print STDERR "GO qualifiers removal option on\n";
  }
}

##############################
# Parse the go_numbers file to get the mapping between GO ids and GO terms
##

# my %GOs = parseGONumbersFile ($go_numbers_file);

##############################
# Parse gene association file
##

my $ga_parser = Bio::PSU::Utils::GeneAssociationParser->new (
							     geneAssociationFile => $gene_association,
							     debug => $_debug,
							    );

if ($_debug) {
  # print STDERR "parsing object: " . Dumper ($ga_parser) . "\n";
}

if ($_debug) {
  print STDERR "parsing gene association file...\n";
}
my %ga_entries = $ga_parser->parse;

if ($_debug) {
  # print STDERR "#############\ngene association entries: " . Dumper (%ga_entries) . "\n#############\n";
}

############################
# open the annotation file
##

my $id_helper =
  Bio::PSU::Utils::FeatureIdHelper::BioPSUHelper->new(-selection_regex => $regex,
						      -verbose => 0);

my $tab_obj = Bio::PSU::SeqFactory->make (-file   => "<$tab",
					  -format => 'embl');  
my $new_tab_obj = Bio::PSU::SeqFactory->make (-fh   => \*STDOUT,
					      -format => 'embl');
while (my $seq_obj = $tab_obj->next_seq) {
  my @features = $seq_obj->features;
  my $new_seq_obj = Bio::PSU::Seq->new (-id   => $seq_obj->id,
					-desc => $seq_obj->desc,
					-type => $seq_obj->type,
					-str  => $seq_obj->str,
				       );
  my @new_features = ();

  ############################
  # add GO terms within the annotation file
  ##
  
  foreach my $feature (@features) {
    my $new_feature = $feature->clone;
    if (($feature->key eq 'CDS') || ($feature->key =~ /RNA/)) {
      
      my $systematic_id = undef;
      my $primary_name  = undef;
      
      eval {
	$systematic_id = $id_helper->get_systematic_id($feature);
      };
      
      if ($@) {
	my $loc = join ",", map {
	  $_->{start} . ".." . $_->{end}
	} $feature->ranges ();
	
	print STDERR "Error in CDS at: $loc\n";
	print STDERR "A Systematic Id is missing !!\n";
	
	# Remove former GO annotations if any and if removal option is on
      
	if ($remove) {
	  if ($new_feature->qexists ('GO')) {
	    $new_feature->qremove ('GO');
	  }
	  if ($new_feature->qexists ('GO_process')) {
	    $new_feature->qremove ('GO_process');
	  }
	  if ($new_feature->qexists ('GO_component')) {
	    $new_feature->qremove ('GO_component');
	  }
	  if ($new_feature->qexists ('GO_function')) {
	    $new_feature->qremove ('GO_function');
	  }
	}
	push (@new_features, $new_feature);
	next;
      }
      
      eval {
	$primary_name = $id_helper->get_primary_name($feature);
      };
      
      # Remove former GO annotations if any and if removal option is on
      
      if ($remove) {
	if ($new_feature->qexists ('GO')) {
	  $new_feature->qremove ('GO');
	}
	if ($new_feature->qexists ('GO_process')) {
	  $new_feature->qremove ('GO_process');
	}
	if ($new_feature->qexists ('GO_component')) {
	  $new_feature->qremove ('GO_component');
	}
	if ($new_feature->qexists ('GO_function')) {
	  $new_feature->qremove ('GO_function');
	}
      }
      
      # incorporate the GO terms if any
      
      my @ga_entries = ();
      
      if (exists $ga_entries{$systematic_id}) {
	@ga_entries = @{$ga_entries{$systematic_id}};
      }
      elsif (defined $primary_name && exists $ga_entries{$primary_name}) {
	@ga_entries = @{$ga_entries{$primary_name}};
      }
      else {
	if ($_debug) {
	  print STDERR "No GO classification for systematic id, $systematic_id!\n";
	}
	push (@new_features, $new_feature);
	next;
      }
      
      foreach my $ga_entry (@ga_entries) {
	my $qualifier_key = "GO";
	my $qualifier_value = "";	  
	
	my $ontology = $ga_entry->getAspect;
	if ($ontology =~ /p/i) {
	  $ontology = "process";
	}
	elsif ($ontology =~ /c/i) {
	  $ontology = "component";
	}
	elsif ($ontology =~ /f/i) {
	  $ontology = "function";
	}
	
	# get the GO term using the go_numbers file
	
	my $go_acc  = $ga_entry->getGOid;
	my $go_term = undef;
	#if (defined $GOs{$go_acc}) {
	#  $go_term = $GOs{$go_acc};
	#}
	
	my $ontology_path = "/nfs/pathsoft/databases/go_cvs/ontology";
	my $ontology_file = "$ontology_path/$ontology.ontology";
	if (-f $ontology_file) {
	  my $line = qx/cat $ontology_file | grep "$go_acc"/;
	  if (defined $line) {
	    
	    $line =~ /\W+([^;]+);.+/;
	    $go_term = $1;
	    
	    if (not defined $go_term) {
	      print STDERR "ontology: $ontology\n";
	      print STDERR "go acc: $go_acc\n";
	      print STDERR "line: $line.\n";
	    }
	    else {
	      # Get rid of the space at the end
	      chop $go_term;
	    }
	  }
	  else {
	    print STDERR "WARN, can't find GO term, $go_acc, for mapping GO Accession/Name!!\n";
	  }
	}
	else {
	  print STDERR "WARN, can't open file, $ontology_file!!\n";
	}
	
	# if there is any semi-colon in the term definition, don't add it to the GO qualifier as the semi-colon is a field delimitor !!
	if (defined $go_term && $go_term =~ /;/) {
	  $go_term = undef;
	}
	
	my $db_ref   = $ga_entry->getDbReference;
	my $evidence = $ga_entry->getEvidence;
	my $with     = $ga_entry->getWithOrFrom;
	my $date     = $ga_entry->getDate;
	
	if ($_debug) {
	  print STDERR "\t* evidence: " . $evidence . "\n";
	  print STDERR "\t* db reference: $db_ref\n";
	  print STDERR "\t* with: $with\n";
	  print STDERR "\t* date: $date\n";
	}
	
	$qualifier_value = "aspect=$ontology; GOid=$go_acc";
	
	if (defined $go_term) {
	  $qualifier_value .= "; term=$go_term";
	}
	$qualifier_value .= "; evidence=$evidence; db_xref=$db_ref";
	if (defined $with && (length ($with) > 1)) {
	  $qualifier_value .= "; with=$with";
	}
	$qualifier_value .= "; date=$date";
	
	if ($_debug) {
	  print STDERR "qualifier value: $qualifier_value\n";
	}
	
	$new_feature->qadd ($qualifier_key, $qualifier_value);
      }
      push (@new_features, $new_feature);
    } # end if CDS or RNA feature
    else {
      # it's not a CDS or RNA feature
      push (@new_features, $feature);
    }
  } # end iteration over features

  if ($_debug) {
    print STDERR "Got " . @features . " features\n";
    print STDERR "Got now " . @new_features . " features\n";
  }
  
  $new_seq_obj->features (@new_features);
  $new_tab_obj->write_seq ($new_seq_obj);
}

if ($_debug) {
  print STDERR "Inclusion done!\n";
  my $t2 = Benchmark->new ();
  print STDERR "\nTotal : ", timestr (timediff ($t2, $t1)), "\n";
}

####################
# END
##

#############################
# display the help message
##

sub display_help {
  print "geneAssociation2embl.pl $_help $_tab [Annotation File] $_ga [Gene Association File] $_go_numbers [go_numbers File] -r [regex expression] -rm [remove former GO annotations]\n\n";
  print "\t$_help a help message\n";
  print "\t$_tab the annotation file (tab or embl file)\n";
  print "\t$_ga the gene association file\n";
  print "\t$_go_numbers the mapping file between GO IDs and GO terms (the file in the GO CVS repository per default)\n";
  print "\t$_regex regex expression to resolve the systematic id\n";
  print "\t$_remove remove former GO annotations - off per default\n";
}

sub parseGONumbersFile {
  my ($go_numbers_file) = @_;
  my %GOs;
 
  open GO, "<$go_numbers_file" or die "can't open go_numbers file, $go_numbers_file!\n";
 
  while (<GO>) {
    my $line = $_;

    if ($line =~ /some\stext/) { next; }
    if ($line =~ /^(GO:\d+)\t\S+\t(\S+.+)/) {
      
      if ($_debug) {
	# print STDERR "go_id: " . $1 . ", go_term: " . $2 . "\n";
      }
      my $go_term = $2;
      chomp ($go_term);

      $GOs{$1} = $go_term;
    }
  }
  close GO;

  if ($_debug) {
    # print STDERR "GOs hash dump: " . Dumper (%GOs) . "\n";
  }

  return %GOs;
}
