#!/usr/local/bin/perl5.6.1 -w

use strict;
use Data::Dumper;

use Bio::PSU::SeqFactory;
use Bio::PSU::Seq;

my $_debug = 1;

my @input_files = ();
while (my $file = shift @ARGV) {
  if ($_debug) {
    print STDERR "input file: $file\n";
  }
  push (@input_files, $file);
}

my $sout = Bio::PSU::SeqFactory->make(
				      -fh   => \*STDOUT,
				      -format => 'embl',
				     );

my @features_per_file = ();
my %seqs;

# Parsing input files

if ($_debug) {
  print STDERR "Parsing input files...\n";
}

my $index = 0;
foreach my $input_file (@input_files) {
  my %features;
  my $sin = Bio::PSU::SeqFactory->make(
				       -file   => "$input_file",
				      );
  while (my $seqobj = $sin->next_seq()) {
    my $seqId = $seqobj->id;
    if ($index == 0) {
      my $seqStr = $seqobj->str;
      $seqs{$seqId} = $seqStr;
    }
    my @features = $seqobj->features;
    $features{$seqId} = \@features;
  }
  $features_per_file[$index] = \%features;

  $index++;
}

# Parsing done

if ($_debug) {
  print STDERR "Parsing done!\n";
}

# Generating the new tab file

if ($_debug) {
  print STDERR "Generating the new tab file\n";
}

my @seqIds = keys (%seqs);
foreach my $seqId (@seqIds) {

  my $seqStr = $seqs{$seqId};
  my @features = ();
  my $index = 0;
  foreach my $featuresSet (@features_per_file) {
    my $features_for_this_file = $featuresSet->{$seqId};
    if (defined $featuresSet->{$seqId}) {
      push (@features, @$features_for_this_file);
    }
    $index++;
  }

  my $newseqobj = Bio::PSU::Seq->new (
				      -id => $seqId,
				      -str => $seqStr,
				      -features => [@features],
				     );
  $sout->write_seq ($newseqobj);
}

# Done !
