#!/usr/local/bin/perl5.6.0 -w

# purpose: This script converts the output of GlimmerMExon
# purpose: to an EMBL feature table
# purpose: Similar to parsing GlimmerF (Asp fumigatus glimmer version)
# purpose: Just an extra attribute, the frame

use strict;

if (@ARGV && $ARGV[0] =~ /^-/) {
  die <<EOF;
usage:
  $0 glimmerMExon_file > glimmerMExon.tab
EOF
}

my $first_line = <>;

if (!defined $first_line || $first_line !~ /^GlimmerMExon/) {
  die <<EOF;
$0: the input file doesn't look like GlimmerMExon output - exiting
EOF
}

my $second_line = <>;
my $seq_name = "glimmermExon";
if ($second_line =~ /^Sequence name/) {
  $second_line =~ /^Sequence name: (.+)/;
  $seq_name = $1;
  print STDERR "seq name: $seq_name\n";
}
else {
  warn "can't find any sequence name !!! \"glimmermExon\" will be assign per default\n";
}

my %genes = ();

while (my $line = <>) {
  if ($line =~ /(\d+)\s+(\d+)\s+([\+\-])\s+(\S+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)/) {

    # print STDERR "matching line: $line\n";

    my $gene_num = $1;
    my $strand = $3;
    my $exon_start = $5;
    my $exon_end = $6;

    my $exon_num = $2;
    my $exon_type = $4;
    my $frame = $8;

    if ($exon_num == 1) {
      $genes{$gene_num}{codon_start} = $frame;
      if ($exon_type =~ /internal|terminal/i) {
        $genes{$gene_num}{fuzzy_start} = 1;
      }
      else {
        $genes{$gene_num}{fuzzy_start} = 0;
      }
    }

    if (exists $genes{$gene_num}{strand}) {
      if ($genes{$gene_num}{strand} ne $strand) {
        warn "warning: strands don't match in gene #$gene_num\n";
      }
    } else {
      $genes{$gene_num}{strand} = $strand;
    }

    if ($strand eq "+") {
      push @{$genes{$gene_num}{exons}}, {
                                         start => $exon_start,
                                         end => $exon_end,
					 exon_type => $exon_type
                                        }
    } else {
      # unshift
      push @{$genes{$gene_num}{exons}}, {
					 start => $exon_start,
					 end => $exon_end,
					 exon_type => $exon_type
					}
    }
  }
}

for my $gene_num (keys %genes) {
  my $location = "";

  my @exons = @{$genes{$gene_num}{exons}};
  my $codon_start = "";
  if (exists $genes{$gene_num}{codon_start}) {
    $codon_start = $genes{$gene_num}{codon_start};
  }

  my $fuzzy_end = 0;

  my $exon_tmp = $exons[0];
  my $first_exon_type = $exon_tmp->{exon_type};
  $exon_tmp = $exons[@exons-1];
  my $last_exon_type = $exon_tmp->{exon_type};

  if ((@exons == 1 && $exons[0]->{exon_type} eq "Initial")
      ||
      (@exons > 1 
       && ($genes{$gene_num}{strand} eq "+" && $exons[@exons-1]->{exon_type} ne "Terminal")
       && ($genes{$gene_num}{strand} eq "-" && $exons[0]->{exon_type} ne "Terminal")
     )) {
    $fuzzy_end = 1;
  }

  if (($genes{$gene_num}{strand} eq "+" && $genes{$gene_num}{fuzzy_start})
    ||
      ($genes{$gene_num}{strand} eq "-" && $fuzzy_end)
     ) {
    $location = "<";
  }
  # $location .= join ",", map { $_->{start} . ".." . $_->{end}} @exons;
  if (($genes{$gene_num}{strand} eq "-" && $genes{$gene_num}{fuzzy_start})
      ||
      ($genes{$gene_num}{strand} eq "+" && $fuzzy_end)
     ) {
    my $end = $exons[@exons-1]->{end};
    $exons[@exons-1]->{end} = ">" . $end;
  }
  $location .= join ",", map { $_->{start} . ".." . $_->{end}} @exons;

  if (@exons > 1) {
    $location = "join($location)";
  }

  if ($genes{$gene_num}{strand} eq "-") {
    $location = "complement($location)";
  }

  print <<EOF;
FT   CDS             $location
FT                   /temporary_systematic_id="$seq_name\_$gene_num"
FT                   /codon_start=$codon_start
EOF
}
