#!/usr/local/bin/perl5.6.0 -w

# $Header: /nfs/pathsoft/repository/psu/genexec/perl/src/glimmerf2tab.pl,v 1.2 2003/05/19 13:05:55 kdj Exp $

# purpose: This script converts the output of GlimmerF (the Aspergillus version
# purpose: of Glimmer to an EMBL feature table


use strict;

if (@ARGV && $ARGV[0] =~ /^-/) {
  die <<EOF;
usage:
  $0 glimmerf_file > glimmerf.tab
EOF
}

my $first_line = <>;

if (!defined $first_line || $first_line !~ /^GlimmerF/) {
  die <<EOF;
$0: the input file doesn't look like GlimmerF output - exiting
EOF
}

my %genes = ();

while (my $line = <>) {
  if ($line =~ /(\d+)\s+(\d+)\s+([\+\-])\s+(\S+)\s+(\d+)\s+(\d+)/) {
    my $gene_num = $1;
    my $strand = $3;
    my $exon_start = $5;
    my $exon_end = $6;

    if (exists $genes{$gene_num}) {
      if ($genes{$gene_num}{strand} ne $strand) {
        warn "warning: strands don't match in gene #$gene_num\n";
      }
    } else {
      $genes{$gene_num}{strand} = $strand;
    }

    if ($strand eq "+") {
      push @{$genes{$gene_num}{exons}}, {
                                         start => $exon_start,
                                         end => $exon_end
                                        }
    } else {
      unshift @{$genes{$gene_num}{exons}}, {
                                            start => $exon_start,
                                            end => $exon_end
                                           }
    }
  }
}

for my $gene_num (keys %genes) {
  my $location;

  my @exons = @{$genes{$gene_num}{exons}};

  $location = join ",", map { $_->{start} . ".." . $_->{end}} @exons;

  if (@exons > 1) {
    $location = "join($location)";
  }

  if ($genes{$gene_num}{strand} eq "-") {
    $location = "complement($location)";
  }

  print <<EOF;
FT   CDS             $location
FT                   /temporary_systematic_id="glimmerf_$gene_num"
EOF
}
