#!/usr/local/bin/perl 

=head1 NAME

glimmerm2tab.perl - conversion script - glimmerm output to a tab file

Copy of the version 1 => glimmerm2tab_v1.perl

=head1 SYNOPSIS

  perl glimmerm2tab.perl [input_file] > output_file.tab

=head1 DESCRIPTION

  This script to convert the output of glimmerm, a Gene Prediction Tool version for Malaria into a tab file
  
  This tab file doesn't contain the all set of predicted CDS. When some overlap each other, only the best hits are presented.

   Input:   a glimmerm output file
   Ouptput: a tab file

=cut

use strict;

$/ = "";
$* = 1;

my $in_file = shift @ARGV;

my @gene_score_list         = ();
my @splice_site_scores_list = ();

open(FEATURES, "$in_file");

while (<FEATURES>) {

  # parsing the gene scoring for each frame

  # print STDERR "line: $_";
  if (/GeneNo/) {
    my @a_tmp = split (/\n/, $_);
    # print STDERR "a_tmp: @a_tmp\n\n";
    foreach my $elem (@a_tmp) {
      next if ($elem =~ /GeneNo/);
      next if ($elem =~ /_+/);
      my @a_tmp_2 = split (/ +/, $elem);
      # print STDERR "a_tmp_2: @a_tmp_2\n";
      # for each frame, parsing the gene scoring - stored in @gene_score
      my @gene_score = ();
      for (my $i=2; $i<8; $i++) {
	next if ($a_tmp_2[$i] =~ /-/);
	# print STDERR "scoring for frame $i: $a_tmp_2[$i]\n";
	push (@gene_score, $a_tmp_2[$i]);
      }
      push (@gene_score_list, \@gene_score);

      # parsing splice site scoring
      my @splice_site_scores = ();
      my $i = 10;
      while ($a_tmp_2[$i] =~ /\d/) {
	push (@splice_site_scores, $a_tmp_2[$i]);
	$i++;
      }
      # print STDERR ("splice_scores: @splice_site_scores\n");
      push (@splice_site_scores_list, \@splice_site_scores);
    }
  }

  my $i = 0;

  s/(Putative Genes:\n)/hello$1\n/g;
  for (split (/hello/)) {
    next unless /Putative Genes/ ;
    for (split (/\n/)) {
      # filter the first lines 
      next if /Putative Genes/;
      # filter unactivated
      next if /Bad overlap/ ;
      my @a_tmp = split (/ +/, $_);
      # print STDERR "a_tmp: @a_tmp\n";
      my $gene = $a_tmp[1];
      # this filter because the first line is empty
      # if not empty then $gene will store the gene number
      next if (! $gene =~ /\d/);
      /\s+\d+((\S|\s)*)AT:(((\S|\s)*))/ && (&parseCoords($1)) ;
      my $note_content = $3;
      $note_content =~ s/\n//g;
      my $gene_score_ref = $gene_score_list[$i];
      my @gene_score = @$gene_score_ref;
      my $splice_site_scores_ref = $splice_site_scores_list[$i];
      my @splice_site_scores = @$splice_site_scores_ref;
      print  "FT                   /temporary_systematic_id=$gene\n" unless ($gene eq "");
      print  "FT                   /note=\"%AT: $note_content" unless ($note_content eq "") ;
      if (@splice_site_scores != ()) {
	print  " / splice site scores = @splice_site_scores";
      }
      print "\"\n";
      
      print  "FT                   /note=\"gene score = @gene_score\"\n" unless (@gene_score == ()) ;
      print  "FT                   /method=\"GLIMMERM\"\n";
      $i++;
    }
    
  } # split on Putative Genes
} # end of while input file



sub parseCoords {

  my $exons =  shift(@_) ;
  my @splicing_coords = split(/\s+/, $exons) ;

  my $start = $splicing_coords[1] ;
  my $stop  = $splicing_coords[$#splicing_coords] ;
  
  print "FT   CDS             " ; 
  if ($start > $stop) {
    print "complement(" ;
    if ($#splicing_coords > 2) {
      print "join(" ;
      foreach my $splicing_coord (1 .. $#splicing_coords) {
	print $splicing_coords[$splicing_coord], ;
	if ($splicing_coord <  $#splicing_coords) {
	  ((($splicing_coord%2) == 0)&&($splicing_coord != 1))?{print ","}:{print ".."
									  } ;
	}
      }
      print ")" ;
    }
    else {print $start, "..", $stop ;}
    print ")\n" ;
  }
  
  
  else {
    
    if ($#splicing_coords > 2) {
      print "join(" ;
      foreach my $splicing_coord (1 .. $#splicing_coords) {
	print $splicing_coords[$splicing_coord], ;
	if ($splicing_coord <  $#splicing_coords) {
	  ((($splicing_coord%2) == 0)&&($splicing_coord != 1))?{print ","}:{print ".."
									  } ;
	}
      }
      print ")\n" ;
    }
    else {print $start, "..", $stop, "\n" ;}
    
  }
  print "FT                   /colour=2\n";
}
