#!/usr/local/bin/perl5.6.1 -w

use strict;
use Data::Dumper;

################################
#
## It is very slow the way it is done because of entry2method mapping done by diong a cat ...
## better to put the file into a hash first !!
#
################################

################################
################################
# This script takes an interpro raw file and adds to it mappping to interpro and GO
# Create a new file whith the ".GOmapped" suffix
################################
################################

############################
# Debug boolean
##

my $debug = 0;

my $_help                = "-help";
my $_interpro            = "-ipro";

# no interpro ouput per default
my $interpro            = "-1";

my $domain2ipro_mapping = "/nfs/pathsoft/databases/interpro/entry2method";
my $ipro2GO_mapping     = "/nfs/pathsoft/databases/go_cvs/external2go/interpro2go";
my $interproId2Name     = "/nfs/pathsoft/databases/interpro/names.dat";
my $GO_id2Name          = "/nfs/pathsoft/databases/go_cvs/numbers/go_numbers";

my $go_process_file   = "/nfs/pathsoft/databases/go_cvs/ontology/process.ontology";
my $go_function_file  = "/nfs/pathsoft/databases/go_cvs/ontology/function.ontology";
my $go_component_file = "/nfs/pathsoft/databases/go_cvs/ontology/component.ontology";
my @ontologies = ($go_process_file, $go_component_file, $go_function_file);

my $i = 0;
foreach my $element (@ARGV) {
  if ($element eq $_help) {
    display_help ();
    exit 1;
  }
  elsif ($element eq $_interpro) {
    $interpro = $ARGV[$i+1];
  }
  # if at least one of the options is not recognized, display the help message
  elsif ($element =~ /^-/) {
    print STDERR "option not recognized: $element\n";
    display_help ();
    exit 1;
  }
  $i++;
}

if ($interpro eq "-1") {
  print STDERR "option(s) not specified!!\n";
  display_help ();
  exit 1;
}

##############################
# Parse the go_numbers file to get the mapping between GO ids and GO terms
##

my %GOs = parseGONumbersFile ($GO_id2Name);

my $new_output = "$interpro.GOmapped";

# set the list of database domain ids
my @domain_ids = ();
my @lines = ();
# hash with domains ids as keys and ?
my %ids;

print STDERR "Parsing interpro raw file, $interpro...\n";

open INTERPRO, "<$interpro" or die "can't open interpro file, $interpro!\n";

while (<INTERPRO>) {
  my $line = $_;
  push (@lines, $line);

  if ($debug) {
    print STDERR "\tparsing line, $line...\n";
  }

  my @split_line = split ('\t', $line);
  my $domain_id = $split_line[4];

  if (not defined $domain_id) {
    print STDERR "ERROR: can't get domain id from line, $line in file, $interpro!!\n";
  }
  else {
    push (@domain_ids, $domain_id);
  }
}

close INTERPRO;

print STDERR "parsing done!\n\n";

# Mapping 2 interpro

print STDERR "\nmapping domain ids to interpro...\n";

%ids = domain2interproMapping ($domain2ipro_mapping, $interproId2Name, @domain_ids);

print STDERR "interpro mapping done!\n\n";

# Mapping 2 GO

print STDERR "\nmappping interpro ids to GO...\n";

%ids = ipro2GOMapping ($ipro2GO_mapping, $GO_id2Name, \@ontologies, \%GOs, \%ids);

print STDERR "GO mapping done!\n\n";

if ($debug) {
  # print STDERR "Dumping ids hash: " . Dumper (%ids) . "\n";
}

# Generate a new interpro file including the mapping

print STDERR "Generating new file with mapping data...\n";

open OUTPUT, ">$new_output" or die "can't open new file, $new_output!\n";

$i = 0;

if ($debug) {
  print STDERR "nb lines, nb ids: " . @lines . ", " . @domain_ids . "\n";
}

while ($i<@lines) {
  my $line = $lines[$i];
  chomp ($line);

  print STDERR "adding line, $line...\n";

  my $id = $domain_ids[$i];

  my $new_line = $line;

  # add mapping stuff

  if (exists ($ids{$id})) {

    print STDERR "adding $id mapping data\n";

    $new_line = $line . "\t" . $ids{$id};
  }

  print OUTPUT "$new_line\n";
  $i++;
}

close OUTPUT;

print STDERR "new file done!\n";

#####################
# End
##

# TO DO !!!!

sub display_help {
  print "interpro2go_mapping.pl -help $_interpro [interpro raw file]\n\n";
  print "\t$_help a help message\n";
  print "\t$_interpro the interpro raw file output\n\n";
}


sub domain2interproMapping {
  my ($domain2interpro_mapping, $interproId2Name, @ids) = @_;
  my %ids;

  if ($debug) {
    print STDERR "Domain ids to map, @ids...\n";
  }
  
  foreach my $id (@ids) {
    my $ipro_id_line = qx/cat $domain2interpro_mapping | grep -v I$id | grep $id/;
    if (length ($ipro_id_line) > 0) {
      my @split_line = split (' ', $ipro_id_line);
      my $ipro_id = $split_line[0];
      
      if ($debug) {
	print STDERR "domain id: $id\n";
	print STDERR "ipro mapping line: $ipro_id_line";
	print STDERR "found ipro id, $ipro_id\n\n";
      }
      
      # mapping ipro id 2 ipro name 
      
      my $ipro_name_line = qx/cat $interproId2Name | grep $ipro_id/;
      $ipro_name_line =~ /IPR\d+.(.+)/;
      my $ipro_name = $1;
      
      if ($debug) {
	print STDERR "ipro_name_line: $ipro_name_line\n";
	print STDERR "found ipro_name: $ipro_name\n";
      }
      
      # my $ipro_mapping = $ipro_id;
      my $ipro_mapping = "$ipro_id\t$ipro_name";
      
      if (length ($ipro_mapping) > 0) {
	$ids{$id} = $ipro_mapping;
      }
    }
    else {
      print STDERR "Can\'t map domain id, $id to interpro !!!\n";
    }
  } 
  
  return %ids;
}

sub ipro2GOMapping {
  my ($ipro2GO_mapping, $GO_id2Name, $ontologies_ref, $GOs_ref, $ids_ref) = @_;
  my @ontologies = @$ontologies_ref;
  my %GOs = %$GOs_ref;
  my %ids = %$ids_ref;
  
  my %new_ids;
  my @ids = keys (%ids);

  if ($debug) {
    print STDERR @ids . " to be mapped...\n";
    print STDERR "ids hash " . Dumper (%ids) . "\n";
  }

  foreach my $id (@ids) {
    my $ipro_mapping = $ids{$id};
    my $ipro_id;
    if ($ipro_mapping =~ /\t/) {
      $ipro_mapping =~ /^(IPR\d+).+/;
      $ipro_id  = $1;
    }
    else {
      $ipro_id = $ipro_mapping;
    }

    my $complete_mapping = $ipro_mapping;

    if ($debug) {
      print STDERR "ipro mapping line: $ipro_mapping\n";
      print STDERR "found ipro id, $ipro_id\n\n";
    }

    my @go_id_lines = qx/cat $ipro2GO_mapping | grep $ipro_id/;

    if (@go_id_lines == 0) {
      print STDERR "can\'t map interpro id, $ipro_id, to GO!!\n";
    }

    my $i = 0;
    foreach my $go_id_line (@go_id_lines) {
      my @split_line = split (' ; ', $go_id_line);
      my $go_term = $split_line[0];
      $go_term =~ s/.+>\sGO://;
      my $go_id = $split_line[1];
      chomp ($go_id);

      if ($debug) {
	print STDERR "ipro2GO mapping line: $go_id_line\n";
	print STDERR "found GO id, $go_id\n";
	print STDERR "found GO term, $go_term\n";
      }
      
      # GO Id 2 name mapping
      
      # my $go_term = $GOs{$go_id};

      # GO id 2 ontology class mapping
      
      my $ontology_name = getOntology ($go_id, @ontologies);
      
      if ($i == 0) {
	$complete_mapping = "$complete_mapping\t$ontology_name: $go_term ($go_id)";
      }
      else {
	$complete_mapping = "$complete_mapping, $ontology_name: $go_term ($go_id)";
      }
	
      $i++;
    }

    if ($debug) {
      print STDERR "complete mapping found for $id: $complete_mapping\n";
    }

    $new_ids{$id} = $complete_mapping;
  }

  return %new_ids;
}


##########################
# check whether it's a GO process, component or function key
##

sub getOntology {
  my ($go_acc, @ontologies) = @_;

  my $debug = 0;

  foreach my $file (@ontologies) {
    my $is_in = qx/cat $file | grep $go_acc/;
    
    if ($debug) {
      print STDERR "is_in: $is_in\n";
    }
    
    if ($is_in ne "") {
      $file =~ /.+\/(\w+)\.ontology/;
      my $ontology_name = $1;
      return $ontology_name;
    }
  }
}

sub parseGONumbersFile {
  my ($go_numbers_file) = @_;
  my %GOs;

  my $debug = 0;
 
  open GO, "<$go_numbers_file" or die "can't open go_numbers file, $go_numbers_file!\n";
 
  while (<GO>) {
    my $line = $_;

    if ($line =~ /some\stext/) { next; }
    if ($line =~ /^(GO:\d+)\t\S+\t(\S+.+)/) {
      
      if ($debug) {
	print STDERR "go_id: " . $1 . ", go_term: " . $2 . "\n";
      }
      my $go_term = $2;
      chomp ($go_term);

      $GOs{$1} = $go_term;
    }
  }
  close GO;

  if ($debug) {
    print STDERR "GOs hash dump: " . Dumper (%GOs) . "\n";
  }

  return %GOs;
}
