#!/usr/local/bin/perl5.6.0 -w

# $Header: /nfs/pathsoft/repository/psu/genexec/perl/src/make_embl_qualifiers.pl,v 1.3 2002/06/19 13:25:08 kmr Exp $

# purpose: This script removes /similarity and others non-embl qualifiers
# purpose: and replaces them with EMBL /note qualifiers that contain the
# purpose: contain the same information in human readable form.  The script
# purpose: uses the same code as qualifier_editor so the /note should be the
# purpose: same as those produced by the MESS.

BEGIN
{
  unshift (@INC, "/nfs/disk222/yeastpub/lib/site_perl/5.6.0");
  unshift (@INC, "/nfs/disk222/yeastpub/lib/site_perl/5.6.0/alpha-dec_osf");
  unshift (@INC, "/nfs/disk222/yeastpub/bio-soft/lib/site_perl/5.6.0");
  unshift (@INC, "/nfs/pathsoft/lib/perl5/site_perl/5.6.0");
}

use strict;

use QualifierEditor::HitInfo;
use QualifierEditor::HitInfoCollection;
use QualifierEditor::App qw (create_app make_prok_note make_euk_similarity);

use Bio::PSU::SeqFactory;
use Bio::PSU::IO::BufferFH;

sub usage
{
  die "$0: needs exactly one argument - an EMBL feature table file\n";
}

if (@ARGV != 1) {
  usage ();
}

# try to read it as a tab file
my $seqfactory =
  Bio::PSU::SeqFactory->make(-file   => $ARGV[0],
                             -format => "EMBL");

my $seq = $seqfactory->next_seq;

sub make_note
{
  my $similarity = shift;

  if ($similarity =~ /^\s*(.*)\s*;
                       \s*(.*)\s*;
                       \s*(.*)\s*;
                       \s*(.*)\s*;
                       \s*(.*)\s*;
                       \s*(?:length\s+(\d+)\s+aa)?\s*;
                       \s*id=(.*)\%\s*;
                       \s*(?:ungapped\s+id=(.*)\%)?\s*;
                       \s*E\(\)=(.*)\s*;
                       \s*(?:score=(.*))?\s*;
                       \s*(?:(\d+)\s+aa\s+overlap)?\s*;
                       \s*(?:query\s+(\d+)\s*-\s*(\d+)\s+aa)?\s*;
                       \s*(?:subject\s+(\d+)\s*-\s*(\d+)\s+aa)?\s*/x) {

    if (!defined $1) {
      warn "incomplete /similarity search type missing from: $similarity\n";
      return undef;
    }

    if (!length $2) {
      warn "incomplete /similarity id missing from: $similarity\n";
      return undef;
    }

    if (!length $3) {
      warn "incomplete /similarity organism missing from: $similarity\n";
      return undef;
    }

    if (!length $4) {
      warn "incomplete /similarity description missing from: $similarity\n";
      return undef;
    }

#      if (!length $5) {
#        warn "incomplete /similarity gene name missing from: $similarity\n";
#        return undef;
#      }

    my ($result_type,
        $id_or_acc,
        $org,
        $description,
        $gene_name,
        $hit_length,
        $id,
        $ungapped_id,
        $e_value,
        $score,
        $overlap,
        $query_start_pos,
        $query_end_pos,
        $subject_start_pos,
        $subject_end_pos) = ($1,
                             $2,
                             $3,
                             $4,
                             $5,
                             $6,
                             $7,
                             $8,
                             $9,
                             $10,
                             $11,
                             $12,
                             $13,
                             $14,
                             $15);

#      print STDERR "$similarity\n";
#      print STDERR "$result_type $id_or_acc\n";

    $id_or_acc =~ s/(?:SWALL|TR|SW):(\S+).*/$1/;

    my %args = (id                => $id_or_acc,
                desc                => $description,
                opt                 => 0,
                z_score             => 0,
                e_value             => $e_value,
                score               => $score,
                sw_score            => 0,
                percent_id          => $id,
                ungapped_percent_id => $ungapped_id,
                overlap             => $overlap,
                query_start_pos     => $query_start_pos,
                query_end_pos       => $query_end_pos,
                subject_start_pos   => $subject_start_pos,
                subject_end_pos     => $subject_end_pos,
                hit_length          => $hit_length,
               );

    my $info_ref = new QualifierEditor::HitInfo (%args);

    my %params = (results_type=>$result_type);
    return make_prok_note ($info_ref, \%params);
  } else {
    warn "no match: $similarity\n";
    return undef;
  }
}


if (defined $seq) {
  my @features = ();

 FEATURE:
  for my $feature ($seq->features) {
    next unless $feature->key () eq "CDS";

    my @similarities = $feature->qvalues ("similarity");

    if (@similarities) {
      my $note_value = undef;

      while (@similarities > 0) {
        my $info_ref = shift @similarities;

        my $new_note = make_note ($info_ref);

        if (defined $new_note) {
          $note_value = "Similar to " . $new_note;

          last;
        }
      }

      if (!defined $note_value) {
        next FEATURE;
      }

      for my $info_ref (@similarities) {
        my $new_note = make_note ($info_ref);

        if (defined $new_note) {
          $note_value .= ", and to " . $new_note;
        }
      }

#      print STDERR "/similarity=$note_value\n";

      $feature->qremove ("similarity");
      $feature->qadd ("note", $note_value);
    }
  }
}

my $out_seq = Bio::PSU::SeqFactory->make(-fh     => \*STDOUT,
                                         -format => 'embl');

$out_seq->write_seq ($seq);
