#!/usr/local/bin/perl5.6.0 -w

=head1 NAME

map_simfeat - Maps a protein domain onto an EMBL feature

=head1 SYNOPSIS

 map_simfeat -d <domain_file> -q <xref_qualifier> < <embl_file>

=head1 DESCRIPTION

It's best not to use this script. This is a nasty hack. Find a better
way.

This script requires two file in EMBL format, one containing
similarity features with peptide coordinates and one containing the
CDS features from which the peptides were derived. The features will
be mapped to their CDS using a string in a specified qualifier as a
cross-reference.

=head1 METHODS

None

=head1 AUTHOR

Keith James (kdj@sanger.ac.uk)

=head1 COPYRIGHT

Copyright (C) 2002 Genome Research Limited. All Rights Reserved.

=head1 DISCLAIMER

This software is provided "as is" without warranty of any kind. It may
be used, redistributed and/or modified under the same conditions as
Perl itself.

=cut

use strict;
use Getopt::Std;
use Bio::PSU::SeqFactory;
use Bio::PSU::FeatureFactory;

my $usage = "Usage: map_simfeat -m <sim_file> -q <xref_qualifier> -s <source> < <embl_file>

\t-m Similarity file to use
\t-q Qualifier to use as cross-reference
\t-s Source of domain (Pfam, Prosite etc)
";

my (%opts, $sim_file, $xref, $source);
getopts('hm:q:s:', \%opts);

defined $opts{h} and die $usage;
defined $opts{m} and $sim_file = $opts{m};
defined $opts{q} and $xref     = $opts{q};
defined $opts{s} and $source   = $opts{s};

unless (defined $sim_file &&
        defined $xref &&
        defined $source) { die $usage}

my $ei = Bio::PSU::SeqFactory->make(-fh     => \*STDIN,
                                    -format => 'embl');
my $eo = Bio::PSU::SeqFactory->make(-fh     => \*STDOUT,
                                    -format => 'embl');

my $si = Bio::PSU::SeqFactory->make(-file   => "<$sim_file",
                                    -format => 'embl');
my $sim_seq = $si->next_seq;
my %sim_id_map;

foreach my $sim ($sim_seq->features)
{
    if ($sim->qexists("$xref"))
    {
        my @ids = $sim->qvalues("$xref");
        my $id = shift @ids;

        if (exists $sim_id_map{$id})
        {
            push(@{$sim_id_map{$id}}, $sim);
        }
        else
        {
            $sim_id_map{$id} = [$sim];
        }
    }
}

while (my $seq = $ei->next_seq)
{
    foreach my $f ($seq->features)
    {
        if ($f->qexists("$xref"))
        {
            my @ids = $f->qvalues("$xref");
            my $id = shift @ids;

            # This really, really sucks
            if (exists $sim_id_map{$id})
            {
                my @sims = @{$sim_id_map{$id}};

                foreach my $s (@sims)
                {
                    my $key = $s->key;

                    if ($key =~ /^CDS_motif/)
                    {
                        my $start = $s->start;
                        my $end   = $s->end;
                        my $acc   = $s->acc;
                        my $score = $s->bits;

                        $f->qadd('domain',
                                 "$source:$acc;;$score;codon $start-$end");
                    }
                    else
                    {
                        my @types = $s->qvalues('type');
                        my $type = shift @types;
                        my $acc = $s->acc;
                        my $score = $s->score;
                        my $desc = $s->desc;
                        my $identity = $s->identity;
                        my $evalue = $s->evalue;
                        my $query_start = $s->start;
                        my $query_end = $s->end;
                        my $subject_start = $s->subject_start;
                        my $subject_end = $s->subject_end;

                        $f->qadd('similarity',
                                 sprintf("%s;%s:%s;;%s;;;id %s;;E()=%s;score=%s;;query %d-%d;subject %d-%d",
                                         $type, $source, $acc, $desc,
                                         $identity, $evalue, $score,
                                         $query_start, $query_end,
                                         $subject_start, $subject_end));
                    }
                }
            }
        }
    }

    $eo->write_seq($seq);
}
