#!/usr/local/bin/perl5.6.0 -w

=head1 NAME

merge_embl - Simple EMBL sequence and feature merger

=head1 SYNOPSIS

 merge2embl ftable_one ftable_two < sequence > out.embl

=head1 DESCRIPTION

This script takes sequences in Fasta or EMBL format on STDIN and a
list of arguments which must be the filenames of EMBL feature
tables. All of the feature tables are added to each sequence, which is
then written to STDOUT. The script also works with complete EMBL files
instead of feature tables, in which case it simply ignores their
sequence.

=head1 METHODS

None

=head1 AUTHOR

Keith James (kdj@sanger.ac.uk)

=head1 COPYRIGHT

Copyright (C) 2002 Genome Research Limited. All Rights Reserved.

=head1 DISCLAIMER

This software is provided "as is" without warranty of any kind. It may
be used, redistributed and/or modified under the same conditions as
Perl itself.

=cut

use strict;
use Getopt::Std;
use Bio::PSU::SeqFactory;

my %opts;
my $usage = "Usage: merge2embl <ftable_1> [.. <ftable_n>] < sequence
";

getopts('h', \%opts);

defined $opts{h} and die $usage;

my $ei = Bio::PSU::SeqFactory->make(-fh     => \*STDIN);
my $eo = Bio::PSU::SeqFactory->make(-fh     => \*STDOUT,
                                    -format => 'embl');

while (my $seq = $ei->next_seq)
{
	foreach my $ftable_name (@ARGV)
	{
		my $fi = Bio::PSU::SeqFactory->make(-file   => "<$ftable_name",
											-format => 'embl');

		while (my $ftable = $fi->next_seq)
		{
			if ($ftable->has_features)
			{
				$seq->features($ftable->features);
			}
		}
	}

	$eo->write_seq($seq);
}
