#!/usr/local/bin/perl5.6.0 -w

=head1 NAME

pfamscan2tab

=head1 SYNOPSIS

 pfamscan2tab < filename

=head1 DESCRIPTION

A parser for the output of pfam_scan which creates an EMBL feature
table.

=head1 AUTHOR

Keith James (kdj@sanger.ac.uk)

=cut

use strict;
use Getopt::Std;
use Bio::PSU::Feature;
use Bio::PSU::IO::FTHandler;

my %opts;
my $usage = "Usage: pfamscan2tab < filename
";

getopts('h', \%opts);

defined $opts{h} and die $usage;

my $fth = Bio::PSU::IO::FTHandler->new;

while (<>)
{
    my ($id, $sstart, $send, $acc, $hstart, $hend, $bits, $evalue, $name)
        = split;

    my $f = Bio::PSU::Feature->new(-key    => "CDS_motif",
                                   -start  => $sstart,
                                   -end    => $send,
                                   -strand => 0);
    $f->qadd('temporary_systematic_id', $id);
    $f->qadd('note', "Pfam domain hit to $acc ($name) at HMM postions $hstart to $hend, bit score: $bits, e-value: $evalue");
    $f->qadd('acc', $acc);
    $f->qadd('bits', $bits);
    $f->qadd('evalue', $evalue);

    print $fth->make_feature_block($f);
}
