#!/usr/local/bin/perl5.6.0 -w

use strict;

# $Header: /nfs/pathsoft/repository/psu/genexec/perl/src/psu_splitter.pl,v 1.6 2004/03/15 16:07:57 es2 Exp $

# purpose: This script will split a file based on the source features by
# purpose: using the PSU version of the EMBOSS splitter program.  It can
# purpose: with files in any format that EMBOSS supports.

use strict;
use Bio::PSU::SeqFactory;

my $emboss_options = "";

sub usage
{
  die <<EOF;
usage: $0 [emboss options] file_to_split ...
       (makes a single multi-entry file)
   or: $0 -ossingle [emboss options] file
       (makes a separate file for each entry in a new sub-directory)
EOF
}

while (@ARGV && $ARGV[0] =~ /^-(.+)/) {
  shift;
  if ($1 eq 'h' or $1 =~ 'help') {
    usage;
  }
  $emboss_options .= " -$1";
}

my $input_file = $ARGV[0];

if (!defined $input_file) {
  $input_file = "";
  print STDERR "(reading from standard input)\n";
}

if ($emboss_options =~ /-ossingle/){
    system "mkdir SplitSeqs";
    system "$ENV{PSU_EMBOSS}/bin/splitter -filter -auto -feature -source -osf embl -osdirectory2 SplitSeqs/ $emboss_options $input_file";
    redo_embl_id_lines();
}
else{
    system "$ENV{PSU_EMBOSS}/bin/splitter -filter -auto -feature -source -osf embl $emboss_options $input_file";
}



sub redo_embl_id_lines{
    my $folder = "SplitSeqs/";
    my $embl_in;
    my @files;

    system "mkdir SplitSeqs/old/";

    opendir DH, $folder or die "$! - Cannot open $folder\n";
    while (my $name = readdir DH){
	next if $name eq "." || $name eq "..";
	$name = $folder.$name;
	next if -d $name;
	push @files, $name;
    }
    
    foreach (@files){
	print "$_\n";
	
	$embl_in = Bio::PSU::SeqFactory->make(-format => 'EMBL',
					      -file => $_);
	
	my $seq = $embl_in->next_seq();
	my @source_id;
	my $source;
	my $file_id = "";
	if (/SplitSeqs\/(.*)\./){
	    $file_id = $1;
	}

	my @all_ids;
	
	foreach my $feat ($seq->features){
	    next unless ($feat->key eq "source");
	    next unless ($feat->qexists('origid'));
	    #next unless (!defined $source);	    
	    @source_id = $feat->qvalues('origid');
	    $source = $source_id[0];
	    push @all_ids, $source;
	}

	foreach (@all_ids){
	    if ($_ =~ /$file_id/i){
		$source = $_;
	    }
	}
	$seq->id($source);
	my $file = "$folder$source";
	unless ($file =~ /.artemis/){
	    $file .= ".embl";
	}
	my $embl_out = Bio::PSU::SeqFactory->make(-format => 'EMBL',
						  -file => ">$file");
	$embl_out->write_seq($seq);

	system "mv $_ SplitSeqs/old/";
    }   
}
