#!/usr/local/bin/perl5.6.1 -w

# $Header: /nfs/pathsoft/repository/psu/genexec/perl/src/psu_to_go.pl,v 1.12 2003/11/27 14:31:50 axk Exp $

# purpose: This script reads the /GO_component, /GO_process and /GO_function
# purpose: qualifiers from the file passed on the command line and writes a
# purpose: Gene Ontology consortium format gene association file to STDOUT


use strict;
use Data::Dumper;
use Getopt::Long;

# BioPSU

use Bio::PSU::SeqFactory;
use Bio::PSU::IO::BufferFH;

# Parsing Module
# Specific of a given organism

# eval "require Bio::PSU::Utils::GenericPSU_to_GO";
use Bio::PSU::Utils::GenericPSU_to_GO;
# eval "require Bio::PSU::Utils::MalariaPSU_to_GO";
use Bio::PSU::Utils::MalariaPSU_to_GO;

my $debug = 0;

my %option;
$option{i}  = 0;
$option{d}  = 0;
$option{t}  = 0;
$option{a}  = 0;
$option{x}  = 0;

my $usage = "Usage: $0 -i tab_file     -d datasource -t taxonID -a assigned_by -r regex
    or $0 -i tigr_file -x -d datasource -t taxonID -a assigned_by -r regex

This script reads the /GO_component, /GO_process and /GO_function
qualifiers from the file passed on the command line and writes a
Gene Ontology consortium format gene association file to STDOUT

The -x flag tells the script to expect qualifiers formatting for easy
exchange with TIGR
";

if (!(&GetOptions(\%option,"i=s","d=s","t=s","a=s","r=s","x"))) {
  die $usage;
}

my $in_file     = $option{i} or die $usage;
my $db_name     = $option{d} or die $usage;
my $taxon_id    = $option{t} or die $usage;
my $assigned_by = $option{a} or die $usage;
my $regex       = $option{r};
my $tigr_format = 0;
$tigr_format    = 1 if defined ($option{x});

#  print STDERR "Enter a taxonomy identifier (e.g. Tbrucei TREU927: 185431)\n";

if ($debug) {
  print STDERR "Taxon id: $taxon_id\n";
  print STDERR "Database name: $db_name\n";
  print STDERR "Assigned by: $assigned_by\n";
}

if ($debug) {
  print STDERR "Reading the annotations file, $in_file\n";
}

# try to read it as a tab file
my $seqfactory =
  Bio::PSU::SeqFactory->make(-file   => $in_file,
                             -format => "EMBL"
			    );
my $seq = $seqfactory->next_seq;

my $parser = undef;

if ($db_name =~ /genedb_pfalciparum/i) {
  $parser = Bio::PSU::Utils::MalariaPSU_to_GO->new;
}
else {
  $parser = Bio::PSU::Utils::GenericPSU_to_GO->new;
}

if ($debug) {
  print STDERR "Processing the GO annotations...\n";
}

$parser->embl_parsing ($seq, $tigr_format, $db_name, $taxon_id, $assigned_by, $regex);

###########
# END
##
