#!/usr/local/bin/perl5.6.0 -w

$Header: /nfs/pathsoft/repository/psu/genexec/perl/src/qmove.pl,v 1.8 2003/10/02 12:51:49 es2 Exp $

=head1 NAME

qmove - Simple EMBL qualifier rearrangement tool

=head1 SYNOPSIS

qmove -q gene -r 'NMA\d+' -i < NM.tab

 efetch | qmove -q note -r 'Pfam' > out.embl

=head1 DESCRIPTION

Where there are multiple qualifiers of a type (e.g. multiple gene
names) qmove will take those matching the regular expression and move
them to the top (or bottom, if the -i flag is on). This is useful
where a software package uses the first qualifier that it comes
across. e.g. Artemis displays the first gene name in the feature, so
it is useful for that to be the functional gene name (trpE), rather
than a systematic one (GN12345).

=head1 METHODS

None

=head1 AUTHOR

Keith James (kdj@sanger.ac.uk)

=head1 COPYRIGHT

Copyright (C) 2000 Genome Research Limited. All Rights Reserved.

=head1 DISCLAIMER

This software is provided "as is" without warranty of any kind. It may
be used, redistributed and/or modified under the same conditions as
Perl itself.

=cut

use strict;
use Getopt::Std;
use Bio::PSU::SeqFactory;

my (%opts, $qual, $regex, $invert);
my $usage = "Usage: qmove -q <qualifier> -r <regex> [-i <boolean>] < embl_file

\t-q qualifier to move
\t-r regex matching the content of the qualifier(s) to
\t   bring to the top
\t-i invert the move i.e. the qualifier(s) with matching
\t   content are sent to the bottom
";

getopts('q:r:ih', \%opts);

defined $opts{q} and $qual  = $opts{q};
defined $opts{r} and $regex = $opts{r};
defined $opts{i} and $invert++;
defined $opts{h} and die $usage;

unless (defined $qual and defined $regex) { die $usage }

my $ei = Bio::PSU::SeqFactory->make(-fh     => \*STDIN,
                                    -format => 'embl');
my $eo = Bio::PSU::SeqFactory->make(-fh     => \*STDOUT,
                                    -format => 'embl');

while (my $seq = $ei->next_seq)
{
    foreach my $feature ($seq->features)
    {
        next unless $feature->qexists($qual);

        my @qual = $feature->$qual;

        my $compiled = eval { qr/$regex/ } or
            die "Failed to compile regex: $regex\n";

        my (@move, @leave);
        @move  = grep  /$compiled/, @qual;
        @leave = grep !/$compiled/, @qual;

        if ($invert)
        {
            push(@leave, @move) if @move
        }
        else
        {
            unshift(@leave, @move) if @move
        }

        $feature->qremove($qual);
        $feature->qadd($qual, @leave);
    }

    $eo->write_seq($seq);
}



