#!/usr/local/bin/perl5.6.1 -w

use strict;
use Getopt::Std;

my $_verbose = 0;

# Configuration of SignalP variables
my $bindir     = "/usr/local/pubseq/scripts";
my $rp_bin   = "$bindir/RepeatMasker";
my $rp_args  = "";

my (%opts, $input, $lib, $_dust, $_softmasking);
my ($chunk, $bqueue) = (1, "pcs3q2");

my $usage = "Usage: repeatmasker.pl [-d <dust>] -i <fasta input file> [-l <repeat database>] [-c <chunk number>] [-q <bqueue>] [-s <sofmasking>]

\t-i fasta input file
\t-d run dust for low complexity region masking - not by default
\t-l give a specific repeat database
\t-c split the database in a given number of chunk to speedup the masking - no chunk by default
\t-q bqueue - Default is pcs3q2
\t-s softmasking - no softmasking by default
";

# Process options
getopts('hdsi:l:q:c:', \%opts);

defined $opts{h} and die $usage;
defined $opts{d} and $_dust++;
defined $opts{l} and $lib    = $opts{l};
defined $opts{i} and $input  = $opts{i};
defined $opts{c} and $chunk  = $opts{c};
defined $opts{q} and $bqueue = $opts{q};
defined $opts{s} and $_softmasking++;

if (not defined $input) {
   print STDERR "no input file specified!!\n";
   die "$usage";
}

if ($_verbose) {
   print STDERR "jobs will be submitted to queue $bqueue\n";
}

if ($lib) {
   $rp_args = "-lib $lib"; 
}

my @files = ($input);
if (defined $chunk && $chunk > 1) {
   qx/fastasplit $input . $chunk/;
   my $pwd = qx/pwd/;
   chomp ($pwd);
   opendir DIR, $pwd or die "can't open current dir, $pwd!!!\n";
   @files = grep {$_ =~ /.+_chunk_\d+/} readdir DIR;
   closedir DIR;   
}

if ($_verbose) {
   print STDERR "files to mask: " . join (", ", @files) . "\n";
}

if (! -d "logs") {
  if ($_verbose) {
    print STDERR "Creating logs directory...\n";
  }
  qx/mkdir logs/;
}
else {
  if ($_verbose) {
    print STDERR "logs directory already in there!!\n";
  }
}

foreach my $file (@files) {

   if ($_verbose) {
     print STDERR "processing file, $file...\n";
   }

   my $cmd = "$rp_bin $rp_args $file";

   if ($_dust) {
      $cmd .= "; dust $file.masked > $file.masked.dust; mv -f $file.masked.dust $file.masked";
   }

   if ($_softmasking) {
      $cmd .= "; fastasoftmask $file $file.masked > $file.softmasked";
   }

   if ($_verbose) {
     print STDERR "Running repeat masking command, $cmd...\n";
   }

   qx/bsub -q $bqueue -o logs -e logs "$cmd"/;
}

print STDERR "The masking process is being processed, " . @files . " jobs have been submitted to $bqueue...\n";

####
# End
##
