#!/usr/local/bin/perl5.6.1 -w

use strict;

# purpose: this script will search an entry for regions rich in a particular
# purpose: set of bases and will generate a feature for each repeat

# $Header: /nfs/pathsoft/repository/psu/genexec/perl/src/rich_region_finder.pl,v 1.6 2003/04/11 10:09:18 kmr Exp $


use Bio::SeqIO;

sub usage
{
  die <<EOF;
usage $0 -h (for help) 
   or $0 [-both] [-note <some text>] bases min_length embl_file

The -both flag will look for matches on both strands.
The -note flag sets the value of the /note field

This script will search an EMBL entry for regions rich in a particular
set of bases and will generate a feature for each repeat

example:
  $0 ct 15 foo.embl
will look for regions at least 15 base pairs long that consist only of C and T
EOF
}

if (@ARGV == 0 || $ARGV[0] eq "-h" || @ARGV < 3) {
  usage ();
}

my $both = 0;
my $note = "";

while (@ARGV && $ARGV[0] =~ /^-(.*)/) {
  if ($1 eq "both") {
    shift;
    $both = 1;
  } else {
    if ($1 eq "note") {
      shift;

      if (@ARGV) {
        $note = shift;
      } else {
        warn "$0: -note needs an argument\n";
        usage;
      }
    } else {

      warn "$0: unknown option: -$1\n";
      usage;
    }
  }
}

if (@ARGV < 3) {
  usage;
}

open IN_FILE, "$ARGV[2]" or die "$0: can't open $ARGV[2]\n";

my $seq_fact = Bio::SeqIO->new(-fh => \*IN_FILE, -format => "EMBL");

my $seq = $seq_fact->next_seq;

my $prim_seq = $seq->primary_seq ();

my $sequence_string = lc $prim_seq->seq ();

my $sequence_length = length $sequence_string;

if ($ARGV[0] !~ /^[actg]+$/i) {
  print STDERR "the base pattern list must contain only A,T,G or C\n";
  usage ();
}

my $base_pattern = "[$ARGV[0]]";

my $min_length = $ARGV[1];

while ($sequence_string =~ /((?:$base_pattern){$min_length,})/g) {
  my $whole_match = $&;
  my $match;
  $match = $1 if (defined $1);
  $match = $2 if (defined $2);
  $match = $3 if (defined $3);
  my $start = (pos $sequence_string) - (length $whole_match) + 1;
  my $end = $start + (length $whole_match) - 1;

  my $my_note = $note;

  if ($my_note eq "") {
    $my_note = $match;
  }
  
  print <<EOF;
FT   repeat_region   $start..$end
FT                   /note="$my_note"
FT                   /label=$match
EOF
}

$sequence_string =~ tr/atgc/tacg/;

if ($both) {
  while ($sequence_string =~ /((?:$base_pattern){$min_length,})/g) {
    my $whole_match = $&;
    my $match;
    $match = $1 if (defined $1);
    $match = $2 if (defined $2);
    $match = $3 if (defined $3);
    my $start = (pos $sequence_string) - (length $whole_match) + 1;
    my $end = $start + (length $whole_match) - 1;

    my $my_note = $note;

    if ($my_note eq "") {
      $my_note = $match;
    }
  
    $match = reverse $match;
  print <<EOF;
FT   repeat_region   complement($start..$end)
FT                   /note="$my_note"
FT                   /label=$match
EOF
  }

}
