#!/usr/local/bin/perl5.6.1 -w

# Run clustalw on a directory with fasta files - if clustalw has already run on a file, it doesn't run again on it.

=head1 NAME

run_clustalw.pl - a clustalw alignment process wrapper script

=head1 SYNOPSIS

  run_clustalw -t [output file format (ALN, GCG)]

=head1 DESCRIPTION

  This script wraps an alignment process on a set of fasta files. These files are in the current directory. Each fasta file is a set of sequences on which the alignment is done with clustalw.
  If the aln file has been already generated, the script doesn't run clustalw again on the corresponding fasta file.

  @author axk

=cut

=head1 ALGORITHM

=cut

##############################################
#
# NB: Files must be all in lowercase otherwise not recognized by clustalw !!!!!
#
##############################################

####################
# import packages
####################

use strict;
use Getopt::Std;

my $_verbose = 0;

my %opts;
my $type = "GCG";
my $machine = "pcs3";

my $usage = "Usage: run_clustalw.pl [-t <output format type> -m <machine]

\t-t output format type: GCG (ie MSF), GDE, PHYLIP or PIR - Default is GCG
\t-m machine is will running on - Default is pcs3
";

# Process options
getopts('ht:m:', \%opts);

defined $opts{h} and die $usage;
defined $opts{t} and $type = $opts{t};
defined $opts{m} and $machine = $opts{m};

my $cur_dir = qx/pwd/;
chomp ($cur_dir);
$cur_dir =~ s/\/tmp_mnt//g;
if ($_verbose) {
   print STDERR "current dir : $cur_dir\n";
}

run_clustalw ($cur_dir, $type, $machine);


##############
# END
##############

sub run_clustalw {
  
  my ($cur_dir, $type, $machine) = @_;
    
  print "running clustalw on each contig to generate the alignment file...\n";
  
  # output directory files
  opendir (THISDIR, $cur_dir) or die "can not read this directory $cur_dir ...";
  my @files_tmp = grep {$_ =~ /.+cluster.+fasta/} readdir THISDIR;
  closedir THISDIR;

  if ($_verbose) {
     print STDOUT "files nb: \n" . (@files_tmp+0) . "\n";
  }

  # filter based on the already existing aln files
  # if already an aln file, don't run clustalw on this cluster

  my @files = ();
  foreach my $file_name (@files_tmp) {
    my @tmp = split (/\./, $file_name);
    my $cluster_name = $tmp[0];
    my $aln_file_name = $cluster_name . ".$type";
    # print "$type file : $aln_file_name\n";
    my $aln_file_name_path = $cur_dir . "/" . $aln_file_name;
    if (not -f $aln_file_name_path) {
      push (@files, $file_name);
    }
  }

  if ($_verbose) {
     print STDOUT "files nb: \n" . (@files+0) . "\n";
  }

  my @files_t = ();
  foreach my $file_name (@files) {

    # Set the output file name

    my @tmp = split (/\./, $file_name);
    my $cluster_name = $tmp[0];
    # print "cl name: $cluster_name\n";
    # my $cluster_number = substr ($cluster_name, 7, length ($cluster_name)) + 0;
    # print STDOUT "cluster_number: $cluster_number\n";
    my $aln_file_name = $cluster_name . ".$type";

    if ($_verbose) {
       print STDERR "command: lsrun -m $machine -v clustalw -infile=" . $file_name . " -outfile=" . $aln_file_name . " -output=" . $type . " -outorder=INPUT -type=DNA\n";
    }

    my @args = ("lsrun -m $machine -v clustalw -infile=" . $file_name . " -outfile=" . $aln_file_name . " -output=" . $type . " -outorder=INPUT -type=DNA");
    system (@args);
  }

  print "alignment files done\n these files (*.$type) can be found in the $cur_dir directory\n";
}
