#!/usr/local/bin/perl -w

=head1 NAME

run_genefinder

=head1 SYNOPSIS

 run_genefinder <work directory> <genefinder args> <file pattern>

=head1 DESCRIPTION

This script serialises running of genefinder on a set of
files. Genefinder uses the same hard-coded temporary files for every
run, so having >1 concurrent instance in a directory risks data
corruption. The last argument on the line is taken to be a regular
expression for matching target sequence files and must therefore be
protected from expansion by the shell. All other arguments are passes
directly to genefinder. The output filenames are are based on the
sequence filename: any "." suffix is removed and replaced with
".genefinder.out". So, sequence.nt produces a file named
"sequence.genefinder.out".

=head1 METHODS

None

=head1 AUTHOR

Keith James (kdj@sanger.ac.uk)

=head1 COPYRIGHT

Copyright (C) 2002 Genome Research Limited. All Rights Reserved.

=head1 DISCLAIMER

This software is provided "as is" without warranty of any kind. It may
be used, redistributed and/or modified under the same conditions as
Perl itself.

=cut

use strict;

my $exec = "/nfs/disk222/yeastpub/databases/acedbs/genefinder/bin/genefinder";
my $suffix = ".genefinder.out";

my @args = @ARGV;

my $dir = shift(@args);

if (! chdir $dir)
{
    die "Failed to change directory to $dir\n";
}

my $pattern = pop(@args);
my @files = glob("$pattern");

foreach my $file (@files)
{
    my $outfile;

    if ($file =~ /(.+)\.\S+$/)
    {
        $outfile = $1 . $suffix;
    }
    else
    {
        $outfile = $file . $suffix;
    }

    my $command = join(" ", $exec, @args);

    my $return = 0xffff & system "$command -seqfile $file > $outfile";

    print "Got $return\n";
    exit($return) if $return;
}
