#!/usr/local/bin/perl5.6.1 -w

use strict;
use Getopt::Long;
# Bioperl Sequence Factory
use Bio::SeqIO;

# BioPSU Sequence Factory
use Bio::PSU::SeqFactory;

my $debug = 0;

my %option;
$option{i}  = undef;
$option{if} = undef;
$option{o}  = undef;
$option{of} = undef;
$option{s}  = undef;
$option{bp} = undef;

if (!(&GetOptions(\%option,"i=s","if=s","o=s","of=s", "s", "bp", "id"))) {
  usage ();
  die("Error: problem processing command-line options\n");
}

my $in_file    = $option{i};
my $in_format = undef;
defined $option{if} and $in_format = $option{if};
my $out_file   = $option{o};
my $out_format = $option{of};
my $split = 0;
defined $option{s} and $split++;;
my $bioperl = 0;
defined $option{bp} and $bioperl++;
my $addSeqId = 0;
defined $option{id} and $addSeqId++;

if (($in_file eq "-1") || ($out_format eq "-1")) {
  usage ();
  die("Error: problem processing command-line options\n");
}

if ((!$split) && ($out_file eq "-1")) {
  print STDERR "Specify an output file!\n";
  usage ();
  die("Error: problem processing command-line options\n");
}

if ($debug) {
  if ($split) {
    print STDERR "split option is on\n";
  }
}

my $in;
if ($bioperl) {
  # Bioperl
  if (defined $in_format) {
    $in  = Bio::SeqIO->new (
  			    -file   => "$in_file",
                            -format => "$in_format",
			   );
  }
  else {
    $in  = Bio::SeqIO->new (
  			    -file   => "$in_file",
			   );
  }
}
else {
  # BioPSU
  if (defined $in_format) {
    $in  = Bio::PSU::SeqFactory->make (
  			              -file   => "$in_file",
                                      -format => "$in_format",
			             );
  }
  else {
    $in  = Bio::PSU::SeqFactory->make (
  				       -file   => "$in_file",
				      );
  }
}

my $out = undef;
if (! $split) {

  if ($bioperl) {
    # Bioperl

    $out = Bio::SeqIO->new (
			    -file   => ">$out_file",
			    -format => "$out_format"
			   );
  }
  else {
    # BioPSU
    
    $out = Bio::PSU::SeqFactory->make (
				       -file   => ">$out_file",
				       -format => "$out_format"
				      );
  }
  
}

while ( my $seq = $in->next_seq() ) {

  if ($debug) {
     print STDERR "next sequence...\n";
  }

  if ($split) {
    my $seq_id = $seq->id;

    if ($debug) {
      print STDERR "seq id: $seq_id\n";
    }

    $out_file = "$seq_id" . ".$out_format";

    if ($bioperl) {
      
      # Bioperl

      $out = Bio::SeqIO->new (
			      -file   => ">$out_file",
			      -format => "$out_format"
			     );
    }
    else {
      # BioPSU
      
      $out = Bio::PSU::SeqFactory->make (
					 -file   => ">$out_file",
					 -format => "$out_format"
					);
    }
    
  }

  if ($addSeqId) {

    # add ID line if missing

    if ($debug) {
       print STDERR "seq id: " . $seq->id . "\n";
    }

    # if (not (defined $seq->id) || $seq->id eq "") {
    if (1) {
      my @tmps = split (/\./, $in_file);
      my $seqId = $tmps[0];

      if ($debug) {
        print STDERR "adding seqId, $seqId\n";
      }

      if (not $bioperl) {
	my $newseq = $seq->clone (-id => $seqId);
	$out->write_seq($newseq);
      }
      else {
        $seq->id ($seqId);
	$out->write_seq($seq);
      }
    } 

  }
  else {
    $out->write_seq($seq);
  }
}
    
sub usage {
  print "sequenceformat.converter.pl -i [input sequence file] -o [output sequence file] -of [output sequence format] -s -bp -id\n";
  print "-i  => input file\n";
  print "-o  => output file\n";
  print "-of => output file format\n";
  print "-s  => split the sequence database into a set of files (optional) - no need to specify the output file then\n";
  print "-bp  => Bioperl instead of BioPSU\n";
  print "-id  => Add the sequence Identifier line if missing\n\n";
}
