#!/usr/local/bin/perl -w

=head1 NAME

 - split_tab_file.pl [input_tab_file] [number]

=head1 SYNOPSIS


=head1 DESCRIPTION

  split a tab file into a given number of smaller tab files.
  each new tab is created in a new directory

  can be used cat_splited_tab_files which generates a unique tab file from a set of tab files

  @author axk

=cut

use strict;
##########################################

use Cwd;
##########################################

# Bio modules
# PSU
use Bio::PSU::SeqFactory;
use Bio::PSU::Seq;
##########################################


my $tab_file     = $ARGV[0];
my $nb_new_files = $ARGV[1];
my $curdir = cwd;
$curdir =~ s/\/tmp_mnt//g;

# create new split directory
# each directory will have a split tab file

my @new_dir = ();

for (my $i=0; $i<$nb_new_files; $i++) {
  my $new_dir = $curdir . "/split$i";
  # add into @new_dir
  push (@new_dir, $new_dir);
  my @args = ("mkdir $new_dir");
  system (@args);
  @args = ("mkdir $new_dir/blast_search");
  system (@args);
}

my $tab_file_path = $curdir . "/" . $tab_file;

my $tab_obj = Bio::PSU::SeqFactory->make(-file   => "<$tab_file_path",
					 -format => 'embl');
my $tab_seq_obj = $tab_obj->next_seq;
my $has_sequence = "false";
if (defined ($tab_seq_obj->str)) {
  $has_sequence = "true";
}

# initialisation of an array of (array of features) !!
my @new_features_sets = init_aoa ($nb_new_files);

if ((my $fnum = $tab_seq_obj->has_features) > 0) {
  print STDERR "found $fnum features\n";
  my $nb_features_per_new_tabfile = int ($fnum / $nb_new_files);
  my $index = 0;
  my $j = 0;
  foreach my $feature ($tab_seq_obj->features) {
    push (@{$new_features_sets[$index]}, $feature);
    if ($j == ($nb_features_per_new_tabfile-1)) {
      $index++;
      $j=0;
    }
    else {
      $j++;
    }
  }
}

for (my $i=0; $i<$nb_new_files; $i++) {
  my $new_tab_file_path = $new_dir[$i] . "/" . $tab_file . ".split$i";
  my $new_tab_obj = Bio::PSU::SeqFactory->make(-file   => ">$new_tab_file_path",
					       -format => 'embl');
  my $new_tab_seq_obj= Bio::PSU::Seq->new (-id   => $tab_seq_obj->id,
					   -desc => $tab_seq_obj->desc,
					   -type => $tab_seq_obj->type);
  $new_tab_seq_obj->features (@{$new_features_sets[$i]});

  # copy the sequence in the last tab/embl file

  ########################################
  # changed $tab_obj for $tab_seq_obj
  ########################################

  if (lc ($has_sequence) eq "true" && ($i == ($nb_new_files-1))) {
    $new_tab_seq_obj->str ($tab_seq_obj->str);
  }

  $new_tab_obj->write_seq($new_tab_seq_obj);

}

##################################
# THE END
##################################

sub init_aoa {
  my ($nb_new_files) = @_;

  my @new_features_sets = ();
  for (my $i=0; $i<$nb_new_files; $i++) {
    my @tmp_array = ();
    $new_features_sets[$i] = [ @tmp_array ];
  }
  
  return @new_features_sets;
}
