#!/usr/local/bin/perl5.6.1 -w

#################################
# This script takes a DNA sequence, in fasta or EMBL format and replaces a letter that you need to specify (prompt) by another one that you specify as well (prompt)
# 
# Syntax: substitute_DNA_X_N.pl [file name]
#################################

# Constraint variables declaration before using them
use strict;
##########################################

# Bio modules
# PSU
use Bio::PSU::SeqFactory;
use Bio::PSU::IO::BufferFH;

use IO::File;
use Carp;

my $debug = 0;

my $letter_to_substitute = "";
while (not $letter_to_substitute =~ /\w/) {
  print STDERR "Enter the letter you want to substitute (e.g. x)\n";
  $letter_to_substitute = <STDIN>;
  chomp ($letter_to_substitute);
}

my $letter_to_add = "";
while (not $letter_to_add =~ /\w/) {
  print STDERR "Enter the letter you want to add (e.g. n)\n";
  $letter_to_add = <STDIN>;
  chomp ($letter_to_add);
}

my $string_to_add = "";
my $i = 0;
while ($i < 100) {
  $string_to_add .= "$letter_to_add";
  $i++;
}

my $input_file = $ARGV[0];
my $new_file = "$input_file.new";

my $fh = IO::File->new("$input_file");
confess "Unable to open $input_file" unless defined $fh;
my $bfh = undef;
$bfh = Bio::PSU::IO::BufferFH->new(-fh => $fh) unless defined $bfh;
my $format = _test_format($bfh);

print STDERR "file format: $format\n";

my $seq1_factory = Bio::PSU::SeqFactory->make(
					      -bfh    => $bfh,
					      -format => "$format"
					     );
my $seq1_obj     = $seq1_factory->next_seq;

my $seq2_factory = Bio::PSU::SeqFactory->make(
					      -file   => ">$new_file",
					      -format   => "$format");

my $seq_str = $seq1_obj->str;

if (not $seq_str =~ /$letter_to_substitute{100,}/) {
  print STDERR "nothing to substitute !!!\n";
}

$seq_str =~ s/$letter_to_substitute{100,}/$string_to_add/g;

my $new_seq_obj = $seq1_obj->clone (-str => "$seq_str");
$seq2_factory->write_seq ($new_seq_obj);

##
# END
###############

sub _test_format  {

    my ($bfh) = @_;


    my $format = 'auto_detect';


    while (my $line = $bfh->unbuffered_read)
    {
	if ($line !~ /^\s*$/)
	{
	    $line =~ /^>/          and $format = 'fasta';
	    $line =~ /^(ID|FH|FT)/ and $format = 'embl';
	    $bfh->buffer_line($line);
	    last;
	}
    }
    return $format;
}
