#!/usr/local/bin/perl5.6.1 -w

=head1 NAME

 tribe2fasta

=head1 SYNOPSIS

 tribe2fasta -c <cluster file> -b <basename> <fasta file>

=head1 DESCRIPTION

This script will split a fasta file of peptide sequences into many
individual fasta files, each containing the members of a cluster
identified by TribeMCL. Each cluster file will be named "basename.n"
where n is the cluster identifier.

=head1 AUTHOR

Keith James <kdj@sanger.ac.uk>

=cut

use strict;
use Getopt::Std;

use Bio::SeqIO;

my (%opts, $clusters, $basename);
my $usage = "Usage: tribe2fasta -c <cluster file> -b <basename> <fasta file>
";

getopts('c:b:h', \%opts);

defined $opts{h} and die $usage;
defined $opts{b} and $basename = $opts{b};
defined $opts{c} and $clusters = $opts{c};

unless (defined $clusters and defined $basename) { die $usage }

my $filename = shift;

unless (defined $filename) { die $usage }

my %peptides;
my %clusters;

###
### Set up IO
###
my $seqi = Bio::SeqIO->new(-file   => "<$filename",
                           -format => 'fasta');

while (my $seq = $seqi->next_seq)
{
    $peptides{$seq->display_name} = $seq;
}

open(IN, "<$clusters") or die "Failed to open cluster data file $clusters: $!\n";
while (<IN>)
{
    chomp;
    next if /^\s*$/;
    my ($cluster_id, $cluster_member) = split;

    unless (exists $clusters{$cluster_id})
    {
        $clusters{$cluster_id} = [];
    }

    push(@{$clusters{$cluster_id}}, $cluster_member);
}
close(IN);

foreach my $cluster_id (keys %clusters)
{
    my $seqo = Bio::SeqIO->new(-file   => ">$basename.$cluster_id",
                               -format => 'fasta');

    foreach my $cluster_member (@{$clusters{$cluster_id}})
    {
        $seqo->write_seq($peptides{$cluster_member});
    }
}
