#!/usr/local/bin/perl5.6.0 -w

# $Header: /nfs/pathsoft/repository/psu/genexec/perl/src/trna-se.pl,v 1.4 2003/12/15 11:24:31 axk Exp $

use strict;

use File::Temp qw/tempfile/;

# purpose: This script will take a sequence run tRNAscan-SE on it and write
# purpose: tab file containing tRNA features

sub usage
{
  die <<EOF;
usage:
  $0 sequence_file_name
    (run in prokaryotic mode)
or
  $0 -euk sequence_file_name
    (run in eukaryotic mode)
EOF
}

my $euk_mode = 0;

if (@ARGV == 2 && $ARGV[0] eq "-euk") {
  shift;
  $euk_mode = 1;
}

if (@ARGV != 1) {
  usage;
}

my $trna_scan_prog = "tRNAscan-SE";
my $trna_scan_args = "-q -Q";

if ($euk_mode) {
  $trna_scan_args .= " -P";
}

my $seqret_prog = "seqret -auto -filter";

my $sequence_file = shift;

my ($sequence_tempfile_fh, $sequence_tempfile) =
  tempfile ("trna-se.$$.seq.XXXXXXXXX", UNLINK => 1);
close ($sequence_tempfile_fh);

my ($structures_tempfile_fh, $structures_tempfile) =
  tempfile ("trna-se.$$.struct.XXXXXXXXX", UNLINK => 1);
close ($structures_tempfile_fh);

my $log_file = "$sequence_file.tRNAscan_SE.log";

system "$seqret_prog $sequence_file > $sequence_tempfile"
  and die "couldn't run $seqret_prog\n";

system "$trna_scan_prog $trna_scan_args -f $structures_tempfile -l $log_file $sequence_tempfile > /dev/null"
  and die "couldn't run $trna_scan_prog\n";

my $hits = 0;

open STRUCTURES, $structures_tempfile
  or die "can't open tRNAscan_SE output\n";

my $seq_id = undef;
my $id;
my $start;
my $end;

while (<STRUCTURES>) {
  if ($_ =~ qr@(\S+)\s+\((\d+)-(\d+)\)\s+Length:@) {
    # first line of a hit
    ($id, $start, $end) = ($1, $2, $3);

    my $seq_id_tmp = $id;
    $seq_id_tmp =~ s/\.trna.*//;
    if (not defined $seq_id) {
      $seq_id = $seq_id_tmp;
      print <<EOF;
ID   $seq_id
FH   Key             Location/Qualifiers
FH
EOF
    }
    if ($seq_id_tmp ne $seq_id) {
      $seq_id = $seq_id_tmp;
      print <<EOF;
//
ID   $seq_id
FH   Key             Location/Qualifiers
FH
EOF
    }
  } else {
    if (/Type:\s+(\S+)\s+Anticodon:\s+(\S+) at \d+-\d+ \((\d+)-(\d+)\)\s+Score: ([\d\.]+)/) {
      my ($trnatype, $anticodon, $anticodon_start,
          $anticodon_end, $score) = ($1, $2, $3, $4, $5);
      my $location;

      ++$hits;

      if ($start < $end) {
        $location = "$start..$end";
      } else {
        $location = "complement($end..$start)";
      }

      my $anticodon_location;

      if ($anticodon_start < $anticodon_end) {
        $anticodon_location = "$anticodon_start..$anticodon_end";
      } else {
        $anticodon_location = "$anticodon_end..$anticodon_start";
      }

      my $gene_qualifier = "tRNA-$trnatype";
      my $anticodon_qualifier = "(pos:$anticodon_location,aa:$trnatype)";
      my $product_qualifier = "transfer RNA-$trnatype";

      print <<EOF;
FT   tRNA            $location
FT                   /gene="$gene_qualifier"
FT                   /anticodon=$anticodon_qualifier
FT                   /product="$product_qualifier"
FT                   /note="tRNA $trnatype anticodon $anticodon, Cove score $score"
EOF
    }
  }
}

if ($hits == 0) {
  warn "no tRNAs found\n";
}
else {
  print <<EOF;
//
EOF
}