#!/usr/local/bin/perl5.6.0 -w

=head1 NAME

trnacsanse2tab

=head1 SYNOPSIS

 trnacsanse2tab < filename

 tRNAscan-SE | trnacsanse2tab

=head1 DESCRIPTION

Converts tabular output from tRNAscan-SE into an EMBL feature
table. Use the -b (brief) option of tRNAscan-SE to create the input.

=head1 AUTHOR

Keith James (kdj@sanger.ac.uk)

=cut

use strict;
use Getopt::Std;
use Bio::PSU::Feature;
use Bio::PSU::IO::FTHandler;

my %opts;

my $usage = "trnacsanse2tab < filename
";

getopts('h', \%opts);
defined $opts{h} and die $usage;

my $fth = Bio::PSU::IO::FTHandler->new;

while (<>)
{
	my ($name, $num, $start, $end, $type, $anticodon,
		$intron_start, $intron_end, $score) = split;

	my $strand = 1;
	if ($start > $end)
	{
		($start, $end) = ($end, $start);
		$strand = -1;
	}

	my $f = Bio::PSU::Feature->new(-key    => "tRNA",
								   -start  => $start,
								   -end    => $end,
								   -strand => $strand);

	$f->qadd('note', "tRNA $type anticodon $anticodon, Cove score $score");
	print $fth->make_feature_block($f);

	if (($intron_start != 0) and ($intron_end != 0))
	{
		$strand = 1;

		if ($intron_start > $intron_end)
		{
			($intron_start, $intron_end) = ($intron_end, $intron_start);
			$strand = -1;
		}

		my $i = Bio::PSU::Feature->new(-key    => "intron",
									   -start  => $intron_start,
									   -end    => $intron_end,
									   -strand => $strand);
		$f->qadd('note', "tRNA $type intron anticodon $anticodon");
		print $fth->make_feature_block($i);
	}
}
