/* GOParser.java
 *
 * created: Tue Aug 28 2001
 *
 * Copyright (C) 2001  Genome Research Limited
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Header: /nfs/pathsoft/repository/psu/genlib/java/src/uk/ac/sanger/psu/goparser/GOParser.java,v 1.3 2003/04/22 15:26:10 kmr Exp $
 */

package uk.ac.sanger.psu.goparser;

import org.xml.sax.*;
import org.apache.xerces.parsers.*;

import java.io.*;
import java.util.Stack;

/**
 *  This program will read the GO XML database from a file or URL and output
 *  a tab delimited file containing the id, name and description.
 *
 * CLASSPATH=/nfs/pathsoft/javalib/xerces.jar javac uk/ac/sanger/psu/goparser/GOParser.java
 *
 *  @author Kim Rutherford <kmr@sanger.ac.uk>
 *  @version $Id: GOParser.java,v 1.3 2003/04/22 15:26:10 kmr Exp $
 **/

public class GOParser extends org.xml.sax.helpers.DefaultHandler {
  private String lastId = null;

  private String lastGoName = null;

  private String lastDefinition = null;

  private Stack currentHierarchy = new Stack ();

  public void characters (char[] text, int start, int length)
      throws SAXException {
    if (currentHierarchy.size () < 2) {
      return;
    }

    final String currentElementName = (String) currentHierarchy.peek ();

    final String parentElementName =
      (String) currentHierarchy.elementAt (currentHierarchy.size () - 2);

    if (!parentElementName.equals ("term")) {
      return;
    }

    final String textString = new String (text, start, length);

    if (currentElementName.equals ("accession")) {
      if (lastId == null) {
        lastId = textString;
      } else {
        lastId = lastId + textString;
      }
    } else {
      if (currentElementName.equals ("name")) {
        if (lastGoName == null) {
          lastGoName = textString;
        } else {
          lastGoName = lastGoName + textString;
        }
      } else {
        if (currentElementName.equals ("definition")) {
          if (lastDefinition == null) {
            lastDefinition = textString;
          } else {
            lastDefinition = lastDefinition + textString;
          }
        }
      }
    }
  }

  public void startElement (String namespaceURI,
                            String localName,
                            String rawName,
                            Attributes atts)
      throws SAXException {
    currentHierarchy.push (localName);
  }

  public void endElement(String namespaceURI,
                         String localName,
                         String rawName) throws SAXException {
    if (lastDefinition == null) {
      lastDefinition = "";
    }

    if (localName.equals ("term")) {
      System.out.println (lastId + "\t" + lastGoName + "\t" + lastDefinition);
      lastId = null;
      lastGoName = null;
      lastDefinition = null;
    }
    currentHierarchy.pop ();
  }

  public static void main (String[] args) {
    if (args.length == 0) {
      System.out.println("usage: goparser URL");
    }

    XMLReader parser = new SAXParser();

    parser.setContentHandler(new GOParser ());

    // start parsing...
    for (int i = 0; i < args.length; i++) {

      // command line should offer URIs or file names
      try {
        parser.parse(args[i]);
      }
      catch (SAXParseException e) { // well-formedness error
        System.out.println(args[i] + " is not well formed.");
        System.out.println(e.getMessage()
         + " at line " + e.getLineNumber()
         + ", column " + e.getColumnNumber());
      }
      catch (SAXException e) { // some other kind of error
        System.out.println(e.getMessage());
      }
      catch (IOException e) {
        System.out.println("Could not report on " + args[i]
         + " because of the IOException " + e);
      }
    }
  }
}
