/*
 * Copyright (c) 2002 Genome Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.sql;

/**
 * <code>NestedSetTreeNode</code>s are used to create trees using the
 * nested set model. A set is represented by a pair of numbers, one
 * low and one high. A set is considered to be contained by another if
 * its low and high numbers lie between the containing set's low and
 * high numbers. This allows efficient storage of trees in a
 * relational database without using recursive SQL statements at the
 * expense of greater complexity in adding/removing nodes and
 * subtrees. See chapter 29 of "Joe Celko's SQL for Smarties" (ISBN
 * 1-55860-576-2).
 *
 * @author Keith James
 */
public interface NestedSetTreeNode
{
    /**
     * <code>getLow</code> returns the node's low number.
     *
     * @return an <code>int</code>.
     */
    public int getLow();

    /**
     * <code>getHigh</code> returns the node's high number.
     *
     * @return an <code>int</code>.
     */
    public int getHigh();
}
