/*
 * Copyright (c) 2002 Genome Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.util;

/**
 * <code>Prototype</code> extends <code>Cloneable</code> making the
 * clone method public. Classes implementing <code>Prototype</code>
 * should use cloning of suitable prototype instances as the default
 * mechanism for creating new objects.
 *
 * @author Keith James
 */
public interface Prototype extends Cloneable
{
    /**
     * <code>clone</code> overrides the clone method of Object.
     *
     * @return an <code>Object</code>.
     *
     * @exception CloneNotSupportedException if an error occurs.
     */
    public Object clone() throws CloneNotSupportedException;
}
