/*
 * This file is part of the Pathogen Sequencing Unit Java library.
 * Copyright (c) 2001 Genome Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.util.gui;

import java.awt.Image;
import java.util.Collection;

/**
 * <code>ImageFactory</code> creates <code>Image</code> instances.
 *
 * @author Keith James
 */
public interface ImageFactory
{
    /**
     * <code>createImage</code> creates a new instance of
     * <code>Image</code> known by an identifier.
     *
     * @param identifier a <code>String</code>.
     *
     * @return an <code>Image</code>.
     *
     * @exception NoSuchImageException if the image requested does not
     * exist.
     */
    public Image createImage(String identifier) throws NoSuchImageException;

    /**
     * <code>createImages</code> creates a collection containing all
     * available images.
     *
     * @return a <code>Collection</code>.
     */
    public Collection createImages();
}
