package uk.ac.sanger.psu.gfmerge.analysis.simfeature_analysis;

import java.util.*;
import uk.ac.sanger.psu.gfmerge.util.getopt.*;
import uk.ac.sanger.psu.gfmerge.analysis.tools.*;
import uk.ac.sanger.psu.gfmerge.analysis.simfeature_analysis.tools.*;
import uk.ac.sanger.psu.gfmerge.analysis.tools.*;

/** Analysis class which calculates the high scoring path in a clustered sequence using the average conditional probability
 *
 * @author Sebastian R. Spiegler
 * @see uk.ac.sanger.psu.gfmerge.util.PredAccuracy
 * @version 1.0
 */

public class AvgCondProbAnalysis extends BasicFeatureAnalysis
{

  ///////////////////////////////////////
  // constructor
  /** constructor of AvgCondProbAnalysis objects
   *
   * @param _arrListOfGMRegions ArrayList of genemodel regions
   * @param _allArguments object which contains all command line
   *     parameters
   */
  public AvgCondProbAnalysis(ArrayList _arrListOfGMRegions,
                             SimpleArg _allArguments) {
    super(_arrListOfGMRegions, _allArguments);
  }


  /** method which returns the high scoring path of genemodels on a sequence using the average conditional probability as scoring system
   * <p>
   * If one genemodel overlaps another, the genemodel with the higher overlap is taken.
   * For further details consult the documentation.
   *
   * @param _arrListOfGMRegionsToAnalyze ArrayList of genemodel regions
   * @return ArrayList of regions with high scoring genemodels
   * @see uk.ac.sanger.psu.gfmerge.datacomp.GeneModel
   * @see uk.ac.sanger.psu.gfmerge.datacomp.GFMergeRegion
   */
  public ArrayList getHighScoringGmRegArr(ArrayList _arrListOfGMRegionsToAnalyze)
  {
    ArrayList highScoringGmRegArr   = new ArrayList();
    int debug                       = this.getAllArguments().getDebug();

    // HashMap of gms pointing to their acp value
    HashMap gmHashOfBestAcp =
        AvgCondProbAnalysisTools.getAcpScoredGmHash(debug, _arrListOfGMRegionsToAnalyze);


    // save scores in biojava features ("note" tag)
    SavScoreInBioFeat.savScore(this.getArrListOfGMRegions(),
                               "AvgCondProbAnalysis",
                               gmHashOfBestAcp);


    // HashMap of high scoring path gms
    // AVERAGE score used for high score path
    boolean isAverage = true;
    HashMap highScorePathGms =
        HighScoringPath.getHighScoringPath(gmHashOfBestAcp,
                                           _arrListOfGMRegionsToAnalyze,
                                           isAverage);

    // remove low scoring gms from gm region array
    highScoringGmRegArr =
        HighScoringPath.removeLowScoreGmsFromRegArr(_arrListOfGMRegionsToAnalyze,
        highScorePathGms);

    return highScoringGmRegArr;
  }
}
