package uk.ac.sanger.psu.gfmerge.analysis.simfeature_analysis;

import java.util.*;
import java.util.regex.*;
import org.biojava.bio.symbol.*;
import uk.ac.sanger.psu.gfmerge.datacomp.*;
import uk.ac.sanger.psu.gfmerge.util.getopt.*;
import uk.ac.sanger.psu.gfmerge.analysis.*;
import uk.ac.sanger.psu.gfmerge.analysis.simfeature_analysis.tools.*;

/** abstract class from which feature analysis class are derived
 *
 * @author Sebastian R. Spiegler
 * @version 1.0
 */
public abstract class BasicFeatureAnalysis implements  BasicFeatureAnalysisAble{

  ///////////////////////////////////////
  // attributes
  /** ArrayList of genemodel regions
   *
   * @see uk.ac.sanger.psu.gfmerge.datacomp.GeneModel
   */
  private ArrayList arrListOfGMRegions;
  /** Object which contains all command line arguments
   */
  private SimpleArg allArguments;

  ///////////////////////////////////////
  // constructor
  /** constructor of abstract BasicFeatureClass, contains partial implementation of constructors of derived classes
   *
   * @param _arrListOfGMRegions ArrayList of genemodel regions
   * @param _allArguments SimpleArg object which contains all command
   *     line arguments
   */
  public BasicFeatureAnalysis(ArrayList _arrListOfGMRegions,
                              SimpleArg _allArguments) {
    setArrListOfGMRegions(_arrListOfGMRegions);
    setAllArguments(_allArguments);
  }

  ///////////////////////////////////////
  // accessor methods

  // get methods

  /** stub for high scoring method which returns the high scoring path of genemodels on a sequence
   *
   * @param _arrListOfGMRegionsToAnalyze ArrayList of genemodel regions
   * @return ArrayList of regions with high scoring genemodels
   */
  public abstract ArrayList getHighScoringGmRegArr(
      ArrayList _arrListOfGMRegionsToAnalyze);

  /** accessor method which retrieves the class name without package information
   *
   * @return class name without package information
   */
  public String getClassName()
  {
    String longClassName = this.getClass().toString();
    String className     = this.getClass().toString();

    Pattern p = Pattern.compile("\\.(\\w+?)$");
    Matcher m = p.matcher(longClassName);

    if(m.find())
    {
      className     = m.group(1);
    }
    else
    {
      className     = "analysis";
    }

    return className;
  }

  /** accessor method which returns ArrayList of genemodel regions
   *
   * @return ArrayList of genemodel regions
   */
  protected ArrayList getArrListOfGMRegions() {
    return arrListOfGMRegions;
  }

  /** accessor method which returns object which contains all command line arguments
   *
   * @return object which contains all command line arguments
   */
  protected SimpleArg getAllArguments() {
    return allArguments;
  }

  // set methods
  /** accessor method which sets ArrayList of genemodel regions
   *
   * @param _arrListOfGMRegions ArrayList of genemodel regions
   */
  private void setArrListOfGMRegions(ArrayList _arrListOfGMRegions) {
    arrListOfGMRegions = _arrListOfGMRegions;
  }

  /** accessor method which sets object of command line arguments
   *
   * @param _allArguments SimpleArg object which contains all command  line
   *     arguments
   */
  private void setAllArguments(SimpleArg _allArguments) {
    allArguments = _allArguments;
  }
}
