package uk.ac.sanger.psu.gfmerge.analysis.simfeature_analysis;

import java.util.*;
import uk.ac.sanger.psu.gfmerge.util.debug.*;
import uk.ac.sanger.psu.gfmerge.util.getopt.*;
import uk.ac.sanger.psu.gfmerge.analysis.tools.*;
import uk.ac.sanger.psu.gfmerge.analysis.simfeature_analysis.tools.*;
import uk.ac.sanger.psu.gfmerge.analysis.tools.*;

/** Analysis class which calculates the high scoring path in a clustered sequence using the cDNA overlap on the exon level
 *
 * @author Sebastian R. Spiegler
 * @version 1.0
 */
public class CDnaOverlapAnalysis extends FeatureOverlapAnalysis
{

  ///////////////////////////////////////
  // constructor
  /** constructor for CDnaOverlapAnalysis objects
   *
   * @param _arrListOfGMRegions ArrayList of genemodel regions
   * @param _arrListOfCDnaRegions ArrayList of cDNA regions
   * @param _allArguments SimpleArg object which contains all command
   *     line arguments
   */
  public CDnaOverlapAnalysis(ArrayList _arrListOfGMRegions,
                             ArrayList _arrListOfCDnaRegions,
                             SimpleArg _allArguments)
  {
    super(_arrListOfGMRegions,
          _arrListOfCDnaRegions,
          _allArguments);
  }

  /** method which returns the high scoring path of genemodels on a sequence using the cDNA overlap on the exon level
   *<p>
   * The overlap score is corresponding to the overlap of cDNAs at the genemodel exon level measured
   * For further details consult the documentation.
   *
   * @param _arrListOfGMRegionsToAnalyze ArrayList of genemodel regions
   * @return ArrayList of regions with high scoring genemodels
   */
  public ArrayList getHighScoringGmRegArr(ArrayList _arrListOfGMRegionsToAnalyze)
  {
    ArrayList highScoringGmRegArr   = new ArrayList();
    ArrayList _arrListOfFeatRegions = this.getArrListOfFeatRegions();
    int debug                       = this.getAllArguments().getDebug();

    // HashMap of gms pointing to their TOTAL OVERLAP SCORE
    boolean considerStrand = this.getAllArguments().doConsiderCDnaStrand();
    boolean isPercentage = false;
    boolean considerIntrons = false;
    HashMap gmHashOfTotalFeatOverlap =
    FeatureOverlapAnalysisTools.getGmHashOfAbsoluteFeatOverlap(
        debug,
        isPercentage,
        considerStrand,
        _arrListOfGMRegionsToAnalyze,
        _arrListOfFeatRegions,
        considerIntrons);


    // save scores in biojava features ("note" tag)
    SavScoreInBioFeat.savScore(this.getArrListOfGMRegions(),
                               "CDnaOverlapAnalysis",
                               gmHashOfTotalFeatOverlap);


    // HashMap of high scoring path gms
    // NO average score used for high score path
    boolean isAverage = false;
    HashMap highScorePathGms =
        HighScoringPath.getHighScoringPath(gmHashOfTotalFeatOverlap,
                                           _arrListOfGMRegionsToAnalyze,
                                           isAverage);

    // remove low scoring gms from gm region array
    highScoringGmRegArr =
        HighScoringPath.removeLowScoreGmsFromRegArr(_arrListOfGMRegionsToAnalyze,
        highScorePathGms);


    return highScoringGmRegArr;
  }
}
