package uk.ac.sanger.psu.gfmerge.analysis.simfeature_analysis;

import java.util.*;
import java.io.*;

import org.biojava.bio.symbol.*;

import uk.ac.sanger.psu.gfmerge.datacomp.*;
import uk.ac.sanger.psu.gfmerge.util.getopt.*;
import uk.ac.sanger.psu.gfmerge.util.debug.*;
import uk.ac.sanger.psu.gfmerge.analysis.*;
import uk.ac.sanger.psu.gfmerge.analysis.tools.*;
import uk.ac.sanger.psu.gfmerge.analysis.simfeature_analysis.*;
import uk.ac.sanger.psu.gfmerge.analysis.simfeature_analysis.tools.*;

/** Abstract class which contains partial implementation of derived analysis classes
 * 
 * @author Sebastian R. Spiegler
 */
public abstract class FeatureAnalysis extends BasicFeatureAnalysis {

  ///////////////////////////////////////
  // attributes
  /** ArrayList of genemodels
   */
  private ArrayList arrListOfGMRegions;
  /** ArrayList of SimFeature regions (cDNA, Blast)
   */
  private ArrayList arrListOfFeatRegions;
  /** SimpleArg object which contains all command line arguments
   */
  private SimpleArg allArguments;


  ///////////////////////////////////////
  // constructor
  /** constructor of abstract FeatureAnalysis class, contains partial implementation of constructors of derived analysis classes
   * 
   * @param _arrListOfGMRegions ArrayList of genemodels
   * @param _arrListOfCDnaRegions ArrayList of cDNA regions
   * @param _allArguments SimpleArg object which contains all commad line
   *      arguments
   */
  public FeatureAnalysis(ArrayList _arrListOfGMRegions,
                         ArrayList _arrListOfCDnaRegions,
                         SimpleArg _allArguments) {
    super(_arrListOfGMRegions, _allArguments);
    setArrListOfFeatRegions(_arrListOfCDnaRegions);
  }


  ///////////////////////////////////////
  // accessor methods

  // get methods
  /** accessor method which gets ArrayList of SimFeature regions (cDNA, Blast)
   * 
   * @return ArrayList of SimFeature regions (cDNA, Blast)
   */
  protected ArrayList getArrListOfFeatRegions() {
    return arrListOfFeatRegions;
  }


  // set methods
  /** accessor method which sets ArrayList of SimFeature regions (cDNA, Blast)
   * 
   * @param _arrListOfFeatRegions ArrayList of SimFeature regions (cDNA, 
   *     Blast)
   */
  private void setArrListOfFeatRegions(ArrayList _arrListOfFeatRegions) {
    arrListOfFeatRegions = _arrListOfFeatRegions;
  }
}

