package uk.ac.sanger.psu.gfmerge.analysis.simfeature_analysis;

import java.util.*;
import uk.ac.sanger.psu.gfmerge.util.getopt.*;
import uk.ac.sanger.psu.gfmerge.analysis.tools.*;
import uk.ac.sanger.psu.gfmerge.analysis.simfeature_analysis.tools.*;
import uk.ac.sanger.psu.gfmerge.analysis.tools.*;

/** Analysis class which calculates the high scoring path in a clustered sequence using the accumulated exon length as a scoring system
 *
 * @author Sebastian R. Spiegler
 * @version 1.0
 */
public class TotalExonLengthAnalysis extends BasicFeatureAnalysis
{

  ///////////////////////////////////////
  // constructor
  /** constructor for TotalExonLengthAnalysis objects
   *
   * @param _arrListOfGMRegions ArrayList of genemodel regions
   * @param _allArguments SimpleArg object which contains all command
   *     line arguments
   */
  public TotalExonLengthAnalysis (ArrayList _arrListOfGMRegions,
                                  SimpleArg _allArguments)
  {
    super(_arrListOfGMRegions, _allArguments);
  }

  ///////////////////////////////////////
  // methods

  /** method which returns the high scoring path of genemodels on a sequence using the accumulated exon length as a scoring system
   * For further details consult the documentation.
   * @param _arrListOfGMRegionsToAnalyze ArrayList of genemodel regions
   * @return ArrayList of regions with high scoring genemodels
   */
  public ArrayList getHighScoringGmRegArr(ArrayList _arrListOfGMRegionsToAnalyze)
  {
    ArrayList highScoringGmRegArr   = new ArrayList();
    int debug                       = this.getAllArguments().getDebug();

    // HashMap of gms pointing to their total exon length
    HashMap gmHashOfBestTlExLength =
        TotalExonLengthAnalysisTools.getTlExLengthScoredGmHash(debug,
        _arrListOfGMRegionsToAnalyze);


    // save scores in biojava features ("note" tag)
    SavScoreInBioFeat.savScore(this.getArrListOfGMRegions(),
                               "TotalExonLengthAnalysis",
                               gmHashOfBestTlExLength);


    // HashMap of high scoring path gms
    // NO average score used for high score path
    boolean isAverage = false;
    HashMap highScorePathGms =
        HighScoringPath.getHighScoringPath(gmHashOfBestTlExLength,
                                           _arrListOfGMRegionsToAnalyze,
                                           isAverage);

    // remove low scoring gms from gm region array
    highScoringGmRegArr =
        HighScoringPath.removeLowScoreGmsFromRegArr(_arrListOfGMRegionsToAnalyze,
        highScorePathGms);

    return highScoringGmRegArr;
  }
}
