package uk.ac.sanger.psu.gfmerge.analysis.tools;

import java.util.*;
import java.awt.List;

import org.biojava.bio.*;
import org.biojava.bio.seq.*;
import org.biojava.bio.symbol.*;

import uk.ac.sanger.psu.gfmerge.util.biotools.*;
import uk.ac.sanger.psu.gfmerge.util.algotools.*;
import uk.ac.sanger.psu.gfmerge.datacomp.*;
import uk.ac.sanger.psu.gfmerge.util.debug.*;

public class SavScoreInBioFeat
{
  public static boolean savScore(ArrayList arrListOfGmRegions,
                                 String callingClass,
                                 HashMap scoredGms)
  {
    // for each region
    for(Iterator itReg = arrListOfGmRegions.iterator(); itReg.hasNext();)
    {
      GFMergeRegion region = (GFMergeRegion) itReg.next();
      ArrayList     gmArr  = (ArrayList) region.getLocateable();

      // for each gm in a single region
      for(Iterator itGm = gmArr.iterator(); itGm.hasNext();)
      {
        GeneModel gm            = (GeneModel) itGm.next();
        StrandedFeature bioFeat = gm.getFeature();

        // put note (score) in biofeature object
        Double score;
        // score is in hash
        if(scoredGms.containsKey(gm))
        {
          score = (Double) scoredGms.get(gm);
        }
        // score is not in hash
        else
        {
          score = new Double(0);
        }

        // put score into annotation
        Annotation annotation  = bioFeat.getAnnotation();
        Object noteObject = null;

        if(annotation.containsProperty("note"))
        {
          noteObject = annotation.getProperty("note");
        }

        String noteString      = callingClass + "=" + score;

        // if note object contains an array
        ArrayList noteArr = new ArrayList();
        if((noteObject != null) &&
          (noteObject.getClass() == noteArr.getClass()))
        {
          noteArr = (ArrayList) noteObject;
          noteArr.add(noteString);
        }
        // if note objects contains a String
        else if(noteObject != null)
        {
          noteArr.add(noteObject);
          noteArr.add(noteString);
        }
        // if there was no note object
        else
        {
          noteArr.add(noteString);
        }


       try{

          annotation.setProperty("note", noteArr);
        }
        catch (Exception ex){}

//        System.out.println(bioFeat.getAnnotation());

      }
    }
    return true;
  }


  public static ArrayList getRemovedFeatures(ArrayList orgArrListOfGmRegions,
                                             ArrayList pickedArrListOfGmRegions)
  {
    ArrayList removedArrListOfGms = new ArrayList();

    HashSet orgGmList =
        SavScoreInBioFeat.putGmsInHash(orgArrListOfGmRegions);

//    System.out.println("orgGmList: " + orgGmList.size());

    HashSet pickedGmList =
        SavScoreInBioFeat.putGmsInHash(pickedArrListOfGmRegions);

//    System.out.println("pickedGmList: " + pickedGmList.size());

    orgGmList.removeAll(pickedGmList);

    ArrayList removedGmArr = new ArrayList();
    removedGmArr.addAll(orgGmList);
//    System.out.println("removedGmArr: " + removedGmArr.size());

    MyTreeMap removedGmTree = new MyTreeMap();
    removedGmTree = BioTools.addToTreeMap(removedGmTree,removedGmArr);

    removedArrListOfGms = BioTools.search(removedGmTree);

    return removedArrListOfGms;
  }






  private static HashSet putGmsInHash(ArrayList arrListOfGmRegions)
  {
    HashSet listOfGms = new HashSet();
    int counter = 0;

    // for each region
    for(Iterator itReg = arrListOfGmRegions.iterator(); itReg.hasNext();)
    {
      GFMergeRegion region = (GFMergeRegion) itReg.next();
      ArrayList gmArr = (ArrayList) region.getLocateable();

      // for each gm in a single region
      for (Iterator itGm = gmArr.iterator(); itGm.hasNext(); )
      {
        GeneModel gm = (GeneModel) itGm.next();
        listOfGms.add(gm);
        counter++;
      }
    }

//    System.out.println(counter);
    return listOfGms;
  }

}