package uk.ac.sanger.psu.gfmerge.datacomp;

import org.biojava.bio.seq.*;
import org.biojava.bio.symbol.*;

/** abstract class which contains the basic implementation of gene components (Intron, Exon).
 *
 * @author Sebastian R. Spiegler
 * @see uk.ac.sanger.psu.gfmerge.datacomp.Exon
 * @see uk.ac.sanger.psu.gfmerge.datacomp.Intron
 * @version 1.0
 */

public abstract class FeatureComponent
    implements FeatureComponentAble {

  ///////////////////////////////////////
  // attributes

  /** Location of genemodel component
   */
  private Location location;
  /** reference to genemodel
   */
  private GFMergeFeature gfmFeature;

  ///////////////////////////////////////
  // constructor

      /** constructor which contains the basic implementation for genemodel components
   *
   * @param _gfmFeature genemodel
   * @param _location Location of genemodel component
   */
  public FeatureComponent(GFMergeFeature _gfmFeature, Location _location) {
    setGfmFeature(_gfmFeature);
    setLocation(_location);

  } // end GeneComponent

  ///////////////////////////////////////
  // operations

  /** accessor method which returns Location of genemodel component
   *
   * @return Location of genemodel component
   */
  public Location getLocation() {
    return location;
  } // end getLocation

  /** accessor method which returns reference to genemodel
   *
   * @return reference to genemodel
   */
  public GFMergeFeature getGfmFeature() {
    return gfmFeature;
  } // end getGeneModel

  /** method which returns length of genemodel component
   *
   * @return length of genemodel component
   */
  public long getLength() {
    long length;
    length = this.getLocation().getMax() - this.getLocation().getMin() + 1;
    return length;
  }

  /** accessor method which sets Location of genemodel component
   *
   * @param _location Location of genemodel component
   */
  private void setLocation(Location _location) {
    location = _location;
  } // end setLocation

  /** accessor method which sets reference to genemodel genemodel component belongs to
   *
   * @param _gfmFeature reference to genemodel
   */
  private void setGfmFeature(GFMergeFeature _gfmFeature) {
    gfmFeature = _gfmFeature;
  } // end setGeneModel

} // end GeneComponent