package uk.ac.sanger.psu.gfmerge.datacomp;

import java.util.*;
import org.biojava.bio.seq.*;
import org.biojava.bio.symbol.*;
import uk.ac.sanger.psu.gfmerge.util.biotools.*;
import uk.ac.sanger.psu.gfmerge.util.algotools.*;

/** class is representing a genemodel.
 * GeneModel wraps the BioJava feature class.
 * <p>
 * For example:
 * <pre>
 * GeneModel gm = new GeneModel(prediction, feature);
 * </pre>
 * 
 * @author Sebastian R. Spiegler
 * @version 1.0
 */

public class GeneModel extends GFMergeFeature implements GeneModelAble {

  ///////////////////////////////////////
  // attributes

    /** reference to prediction object.
     */
    private Prediction prediction;

  ///////////////////////////////////////
  // constructor

    /** constructor for GeneModel objects.
     * 
     * @param prediction reference to prediction object
     * @param feature reference to BioJava feature object
     */
    public  GeneModel(Prediction prediction, StrandedFeature feature)
    {
      super(feature);
      setPrediction(prediction);
    } // end GeneModel

    ///////////////////////////////////////
    // operations

    /** accessor method which returns reference to prediction object.
     * 
     * @return reference to prediction
     */
    public Prediction getPrediction()
    {
      return prediction;
    }


    /** accessor method which sets reference to prediction.
     * 
     * @param _prediction reference to prediction object
     */
    private void setPrediction(Prediction _prediction)  //done
    {
      prediction = _prediction;
    }

} // end GeneModel
