package uk.ac.sanger.psu.gfmerge.util.debug;

import uk.ac.sanger.psu.gfmerge.*;
import uk.ac.sanger.psu.gfmerge.datacomp.*;
import uk.ac.sanger.psu.gfmerge.util.getopt.*;
import uk.ac.sanger.psu.gfmerge.util.biotools.*;
import uk.ac.sanger.psu.gfmerge.util.algotools.*;
import java.util.*;
import org.biojava.bio.symbol.*;

/** class which contains debugging methods.
 * Logging can be done seperately, on different levels or as a whole.
 *
 * @author Sebastian R. Spiegler
 * @version 1.0
 */
public class Debug {
  ///////////////////////////////////////
  // Debugging modes

  /*
   Debugging mode 1xx - pre processing -
   100 - all pre processing
   101 - Command line arguments
   102 - GenePredicter objects, Prediction objects, GeneModel objects
   103 - gene model regions - MyTreeMap for GMs and Blast hits
   104 - gene model regions - regions
   105 - feature regions - MyTreeMap for cDNA/Blast hits
   106 - feature regions - regions of cDNA/Blast hits

   Debugging mode 2xx - cDNA splice site -
   200 - all cDNA splice site debugging
   201 - hash of by cDNAs overlapped gms
   202 - CDnaSpliceSite - PostionTree
   203 - hash of cDNA scored gms
   204 - hash of high scoring path
   205 - cleaned array of gm regions without lowscoring gms

   Debugging mode 3xx - cDNA overlap -
   300 - all cDNA overlap debugging
   301 - HashMap of genemodels pointing to total feature overlap (203)
   302 - HashMap of genemodels pointing to total percentage feature overlap (203)

   Debugging mode 4xx - Blast overlap -
   400 - all Blast overlap debugging
   401 - HashMap of gms pointing to array of overlapping features
   402 - HashMap of exons pointing to array of overlapping features
   403 - protein HashMap for a single gene model
   404 - HashMap of proteins pointing to overlap length

   Debugging mode 5xx - equal gms
   500 -
   501 - hash of by equality scored gene models

   Debugging mode 6xx - average conditional probability

   Debugging mode 7xx - general debugging
   700 -
   701 - array of analyzed genemodels
   */
// parameters -a /nfs/team81/ss7/project/artemis/tdata1/use/annotated.tab  -t "/nfs/team81/ss7/project/artemis/tdata1/use/hmmgene.tab /nfs/team81/ss7/project/artemis/tdata1/use/gfseq.tab /nfs/team81/ss7/project/artemis/tdata1/use/geneid.tab" -p "/nfs/team81/ss7/project/artemis/tdata1/use/hmmgene.tab /nfs/team81/ss7/project/artemis/tdata1/use/gfseq.tab /nfs/team81/ss7/project/artemis/tdata1/use/geneid.tab" -b /nfs/team81/ss7/project/artemis/tdata1/use/blastx.tab -c /nfs/team81/ss7/project/artemis/tdata1/use/cDNAs.tab -d 1000
// parameters -a /nfs/team81/ss7/project/perl/annotation.tab  -t "/nfs/team81/ss7/project/perl/genepredicter1.tab /nfs/team81/ss7/project/perl/genepredicter2.tab /nfs/team81/ss7/project/perl/genepredicter3.tab" -p "/nfs/team81/ss7/project/perl/genepredicter1.tab /nfs/team81/ss7/project/perl/genepredicter2.tab /nfs/team81/ss7/project/perl/genepredicter3.tab" -b /nfs/team81/ss7/project/perl/blastx.tab -c /nfs/team81/ss7/project/perl/cDNAs.tab -d 1
// parameters -a /nfs/team81/ss7/project/perl/annotation.tab -t "/nfs/team81/ss7/project/perl/genepredicter1.tab /nfs/team81/ss7/project/perl/genepredicter2.tab /nfs/team81/ss7/project/perl/genepredicter3.tab" -p "/nfs/team81/ss7/project/perl/genepredicter1.tab /nfs/team81/ss7/project/perl/genepredicter2.tab /nfs/team81/ss7/project/perl/genepredicter3.tab" -b /nfs/team81/ss7/project/perl/blastx.tab -c /nfs/team81/ss7/project/perl/cDNA.tab -f /nfs/team81/ss7/project/artemis/tdata1/use/mergedGms -r -x 15 -d 1
// parameters -a /nfs/team81/ss7/project/artemis/tdata1/use/annotated.tab  -t "/nfs/team81/ss7/project/artemis/tdata1/use/hmmgene.tab /nfs/team81/ss7/project/artemis/tdata1/use/gfseq.tab /nfs/team81/ss7/project/artemis/tdata1/use/geneid.tab" -p "/nfs/team81/ss7/project/artemis/tdata1/use/hmmgene.tab /nfs/team81/ss7/project/artemis/tdata1/use/gfseq.tab /nfs/team81/ss7/project/artemis/tdata1/use/geneid.tab" -b /nfs/team81/ss7/project/artemis/tdata1/use/blastx.tab -c /nfs/team81/ss7/project/artemis/tdata1/use/cDNAs.tab -f /nfs/team81/ss7/project/artemis/tdata1/use/mergedGmsUse -r -x 15 -d 1
  ///////////////////////////////////////
  // attributes
  static int debug_202_1 = 0;
  static int debug_202_2 = 0;

////////////////////////////////////////////////////////////////////////
// Debugging Command line arguments
// Debugging mode 101
//
  /** Debugging Command line arguments
   * Debugging mode 101
   * - pre processing -
   *
   * @param debug
   * @param allArguments
   */
  public static void debug_101(int debug, SimpleArg allArguments) {
    if ( (debug == 1) || (debug == 100) || (debug == 101)) {
      System.out.println("GFMerge - Command line arguments:");
      System.out.println("GFMerge - Debug mode            : " +
                         allArguments.getDebug());
      System.out.println("GFMerge - Annotation            : " +
                         allArguments.getAnnotatedTrainSet());
      System.out.println("GFMerge - Training sets         : " +
                         allArguments.getPredictedTrainSet());
      System.out.println("GFMerge - Predictions           : " +
                         allArguments.getPredictions());
      System.out.println("GFMerge - Output filename       : " +
                         allArguments.getOutputFilename());
      System.out.println("GFMerge - Do file of removed gms: " +
                         allArguments.doFileOfRemovedGms());
      System.out.println("GFMerge - Fuzzy cDNA boundaries : " +
                         allArguments.getFuzzyBoundariesValue());
    }
  }




////////////////////////////////////////////////////////////////////////
// Debugging GenePredicter and Prediction objects
// Debugging mode 102
//

  // Debug GenePredicter objects in GFMerge
  /** Debugging GenePredicter and Prediction objects
   * Debugging mode 102
   * - pre processing -
   *
   * @param debug
   * @param gPredicter
   */
  public static void debug_102_1(int debug, ArrayList gPredicter)
  {
    if ( (debug == 1) || (debug == 100) || (debug == 102)) {
      for(Iterator it = gPredicter.iterator();it.hasNext();)
      {
        GenePredicter gp = (GenePredicter) it.next();
        System.out.println("GFMerge_Analysis - GenePredicter " + gp.getName() +
                           " acp: " + gp.getAcp());
        System.out.println();
      }
    }
  }

  // Debug Prediction objects in GFMerge
  /** Debug Prediction objects in GFMerge
   * Debugging mode 102
   * - pre processing -
   *
   * @param debug
   * @param gPredictionName
   * @param fName
   */
  public static void debug_102_2(int debug, String gPredictionName,
                                   String fName) {
    if ( (debug == 1) || (debug == 100) || (debug == 102)) {
      System.out.println("GFMerge_Analysis - Prediction " + gPredictionName + " " +
                         fName);
      System.out.println();
    }
  }

  // Debug GeneModel objects in GFMerge
  /** Debug GeneModel objects in GFMerge
   * Debugging mode 102
   * - pre processing -
   *
   * @param debug
   * @param predictions
   */
  public static void debug_102_3(int debug, ArrayList predictions) {
    if ( (debug == 1) || (debug == 100) || (debug == 102)) {
      for (Iterator itPred = predictions.iterator(); itPred.hasNext(); ) {
        Prediction pred = (Prediction) itPred.next();
        for (Iterator itGM = pred.getGeneModels().iterator(); itGM.hasNext(); ) {
          GeneModel gm = (GeneModel) itGM.next();
          System.out.println("GFMerge_Analysis - GeneModel "
                             + " Prediction Name "
                             + gm.getPrediction().getGenePredicterName()
                             + " Predicter Name "
                             + gm.getPrediction().getGenePredicter().getName()
                             + " acp "
                             + gm.getPrediction().getGenePredicter().getAcp());
          System.out.println("GFMerge_Analysis - GeneModel " + gm.getLocation());
          for (Iterator exIt = gm.getExons().iterator(); exIt.hasNext(); ) {
            Exon exon = (Exon) exIt.next();
            System.out.println("GFMerge_Analysis - Exon (" + exon.getExonType() +
                               ") of " +
                               ((GeneModel) exon.getGfmFeature()).getPrediction().
                               getGenePredicterName());
          }

          for (Iterator inIt = gm.getIntrons().iterator(); inIt.hasNext(); ) {
            Intron intron = (Intron) inIt.next();
            System.out.println("GFMerge_Analysis - Intron: " + intron.getLocation());
          }
          System.out.println();
        }
      }
    }
  }





////////////////////////////////////////////////////////////////////////
// Debugging MyTreeMap for GMs and Blast hits
// Debugging mode 103
//

  /** Debugging MyTreeMap for GMs and Blast hits
   * Debugging mode 103
   * - pre processing -
   *
   * @param debug
   * @param calledByClass
   * @param tMap
   */
  public static void debug_103(int debug, String calledByClass, MyTreeMap tMap)
  {
    if ( (debug == 1) || (debug == 100) || (debug == 103)) {
      System.out.println();
      System.out.println(calledByClass);
      System.out.println();

      for (Iterator itMap = tMap.keySet().iterator(); itMap.hasNext(); ) {
        Long key = (Long) itMap.next();
        ArrayList arrList = (ArrayList) tMap.get(key);

        System.out.println("node has " + arrList.size() + " elements");

        for (Iterator itArr = arrList.iterator(); itArr.hasNext(); ) {
          LocateAble loc = (LocateAble) itArr.next();
          System.out.println("locateable has location " + loc.getLocation());
        }
      }
    }
  }






////////////////////////////////////////////////////////////////////////
// Debugging regions of gene models
// Debugging mode 104
//

  /** Debugging regions of gene models
   * Debugging mode 104
   * - pre processing -
   *
   * @param debug
   * @param arrListOfRegions
   * @param tMap
   */
  public static void debug_104(int debug, ArrayList arrListOfRegions,
                                 MyTreeMap tMap) {
    if ( (debug == 1) || (debug == 100) || (debug == 104)) {
      System.out.println("GFMerge_Analysis - Regions of overlapping gene models:");
      System.out.println("GFMerge_Analysis - items in tMap: " + tMap.size());
      int i = 0;
      System.out.println("GFMerge_Analysis - ArrSize: " + arrListOfRegions.size());
      for (Iterator it = arrListOfRegions.iterator(); it.hasNext(); ) {
        i++;
        GFMergeRegion reg = (GFMergeRegion) it.next();
        System.out.println("GFMerge_Analysis - Region " + i + " " + reg.getLocation() +
                           " size: " + reg.getLocateable().size());
        for (Iterator itArr = reg.getLocateable().iterator(); itArr.hasNext(); ) {
          GeneModel gm = (GeneModel) itArr.next();
          System.out.println("GFMerge_Analysis - Region.GM " + i + " " + gm.getLocation());
        }
        System.out.println();
        System.out.println();
      }
    }
  }





////////////////////////////////////////////////////////////////////////
// Debugging MyTreeMap for cDNA hits
// Debugging mode 105
//

  /** Debugging MyTreeMap for cDNA hits
   * Debugging mode 105
   * - pre processing -
   *
   * @param debug
   * @param calledByClass
   * @param tMap
   */
  public static void debug_105(int debug, String calledByClass, MyTreeMap tMap)
  {
    if ( (debug == 1) || (debug == 100) || (debug == 105)) {
      System.out.println();
      System.out.println(calledByClass);
      System.out.println();

      for (Iterator itMap = tMap.keySet().iterator(); itMap.hasNext(); ) {
        Long key = (Long) itMap.next();
        ArrayList arrList = (ArrayList) tMap.get(key);

        System.out.println("node has " + arrList.size() + " elements");

        for (Iterator itArr = arrList.iterator(); itArr.hasNext(); ) {
          LocateAble loc = (LocateAble) itArr.next();
          System.out.println("locateable has location " + loc.getLocation());
        }
      }
    }
  }




////////////////////////////////////////////////////////////////////////
// Debugging regions of cDNA/Blast hits
// Debugging mode 106
//

  /** Debugging regions of cDNA/Blast hits
   * Debugging mode 106
   * - pre processing -
   *
   * @param debug
   * @param callingClass
   * @param arrListOfRegions
   * @param tMap
   */
  public static void debug_106(int debug, String callingClass,
                               ArrayList arrListOfRegions,
                               MyTreeMap tMap) {
    if ( (debug == 1) || (debug == 100) || (debug == 106)) {
      System.out.println("GFMerge_Analysis - Regions of overlapping "
                         + callingClass +" hits:");
      System.out.println("GFMerge_Analysis - items in tMap: " + tMap.size());
      int i = 0;
      System.out.println("GFMerge_Analysis - ArrSize: " + arrListOfRegions.size());
      for (Iterator it = arrListOfRegions.iterator(); it.hasNext(); ) {
        i++;
        GFMergeRegion reg = (GFMergeRegion) it.next();
        System.out.println("GFMerge_Analysis - Region " + i + " " + reg.getLocation()
                           + " size: " + reg.getLocateable().size());
        for (Iterator itArr = reg.getLocateable().iterator(); itArr.hasNext(); ) {
          SimFeature feat = (SimFeature) itArr.next();
          System.out.println("GFMerge_Analysis - Region." + callingClass
                             +" " + i + " " + feat.getLocation());
        }
        System.out.println();
        System.out.println();
      }
    }
  }




////////////////////////////////////////////////////////////////////////
// Debugging hash of by cDNAs overlapped gms
// Debugging mode 201
//

  /** Debugging hash of by cDNAs overlapped gms
   * Debugging mode 201
   * - cDNA splice site -
   *
   * @param debug
   * @param hashCDnaOverlappedGms
   */
  public static void debug_201(int debug, HashMap hashCDnaOverlappedGms)
  {
    if ( (debug == 1) || (debug == 200) || (debug == 201))
    {
      System.out.println("CDnaSpliceSite - Hash of by cDNAs overlapped gm ");
      int count = 0;

      for(Iterator itGm = hashCDnaOverlappedGms.keySet().iterator();
          itGm.hasNext();)
      {
        GeneModel gm      = (GeneModel) itGm.next();
        ArrayList featArr = (ArrayList) hashCDnaOverlappedGms.get(gm);
        count += featArr.size();

        System.out.println("CDnaSpliceSite - gene model: " + gm.getLocation());

        for(Iterator itFeat = featArr.iterator(); itFeat.hasNext();)
        {
          SimFeature feature = (SimFeature)  itFeat.next();
          System.out.println("CDnaSpliceSite - overlapping cDNA: "
                             + feature.getLocation());
        }
        System.out.println();

      }
      System.out.println("total cDNA overlaps: " + count);
    }
  }



////////////////////////////////////////////////////////////////////////
// Debugging CDnaSpliceSite - PostionTree
// Debugging mode 202
//

  /** Debugging CDnaSpliceSite - PostionTree
   * Debugging mode 202
   * - cDNA splice site -
   *
   * @param debug
   * @param gm
   * @param isMinPosTree
   * @param positionTree
   */
  public static void debug_202(int debug, GeneModel gm,
                               boolean isMinPosTree, MyTreeMap positionTree)
  {
    if ( (debug == 1) || (debug == 200) || (debug == 202))
    {
      System.out.println("CDnaSpliceSite - PostionTree");
      if(isMinPosTree == true)
      {
        System.out.println("Minimum Position Tree!");
      }
      else
      {
        System.out.println("Maximum Position Tree!");
      }

      for(Iterator itNode = positionTree.keySet().iterator(); itNode.hasNext();)
      {
        Integer        position = (Integer) itNode.next();
        RecordGmCDna   record   = (RecordGmCDna) positionTree.get(position);
        System.out.println("Node position: " + position);

        if(record.gmBoundary != null)
        {
          System.out.println("     gm bound: " + record.gmBoundary);
          debug_202_1++;
        }
        else
        {
          System.out.println("     gm bound: " + "NONE");
        }

        if(record.cDnaBoundaries.size() > 0)
        {
          for(Iterator itArr = record.cDnaBoundaries.iterator(); itArr.hasNext();)
          {
            Integer cDnaBound = (Integer) itArr.next();
            System.out.println("     cDNA bound: " + cDnaBound);
            debug_202_2++;
          }
        }
        else
        {
          System.out.println("     cDNA bound: " + "NONE");
        }
      }
      System.out.println();
      System.out.println("number of gm   bounds: " + debug_202_1);
      System.out.println("number of cDna bounds: " + debug_202_2);
      System.out.println();
      System.out.println();
    }

  }





////////////////////////////////////////////////////////////////////////
// Debugging hash of cDNA scored gms
// Debugging mode 203
//
    /** Debugging hash of cDNA scored gms
     * Debugging mode 203
     * - cDNA splice site -
     *
     * @param debug
     * @param callingClass
     * @param cDnaScoredHash
     */
    public static void debug_203(int debug,
                                 String callingClass,
                                 HashMap cDnaScoredHash)
    {
      if ( (debug == 1) || (debug == 200) || (debug == 203))
      {
        System.out.println(callingClass);
        System.out.println("Hash size: " + cDnaScoredHash.size());

        for(Iterator itGm = cDnaScoredHash.keySet().iterator(); itGm.hasNext();)
        {
          GeneModel gm = (GeneModel) itGm.next();
          Object score = (Object) cDnaScoredHash.get(gm);

          System.out.println("GeneModel       : " + gm.getLocation());
          System.out.println("  from predicter: " + gm.getPrediction().getGenePredicter().getName());
          System.out.println("  with score    : " + score );
          System.out.println();
        }
      }
    }




////////////////////////////////////////////////////////////////////////
// Debugging hash of high scoring path
// Debugging mode 204
//

    //debug 204 - hash of high scoring path
    /** Debugging hash of high scoring path
     * Debugging mode 204
     * - cDNA splice site -
     *
     * @param debug
     * @param callingClass
     * @param highScoringHash
     */
    public static void debug_204(int debug,
                                 String callingClass,
                                 HashMap highScoringHash)
    {
      if ( (debug == 1) || (debug == 200) || (debug == 204))
      {
       Debug.debug_203(203,callingClass,highScoringHash);
      }
    }





////////////////////////////////////////////////////////////////////////
// Debugging cleaned array of gm regions without lowscoring gms
// Debugging mode 205
//
    /** Debugging cleaned array of gm regions without lowscoring gms
     * Debugging mode 205
     * - cDNA splice site -
     *
     * @param debug
     * @param arrListOfGmRegs
     */
    public static void debug_205(int debug,
                                 ArrayList arrListOfGmRegs)
    {
      if ( (debug == 1) || (debug == 200) || (debug == 205))
      {

        System.out.println("CDnaSpliceSite - cleaned array of gm regions without lowscoring gms");

        int j = 0;
        int i = 0;

        for (Iterator it = arrListOfGmRegs.iterator(); it.hasNext(); )
        {
          j++;
          GFMergeRegion reg = (GFMergeRegion) it.next();
          System.out.println("CDnaSpliceSite - Region " + j + " " + reg.getLocation() +
                             " size: " + reg.getLocateable().size());

          for (Iterator itArr = reg.getLocateable().iterator(); itArr.hasNext(); )
          {
            i++;
            GeneModel gm = (GeneModel) itArr.next();
            System.out.println("CDnaSpliceSite - Region.GM " + gm.getLocation());
          }
          System.out.println();
          System.out.println();
        }
        System.out.println("number of regions  : " + j);
        System.out.println("total number of gms: " + i);
      }
    }



///////////////////////////////////////////////////////////////////////
// Debugging HashMap of genemodels pointing to total feature overlap
// Debugging mode 301 (203)
//
     /** Debugging HashMap of genemodels pointing to total feature overlap
      * Debugging mode 301 (203)
      * - cDNA overlap -
      *
      * @param debug
      * @param callingClass
      * @param cDnaScoredHash
      */
     public static void debug_301(int debug,
                                  String callingClass,
                                  HashMap cDnaScoredHash)

     {
       if ( (debug == 1) || (debug == 300) || (debug == 301))
       {
         Debug.debug_203(203, callingClass, cDnaScoredHash);
       }

     }





///////////////////////////////////////////////////////////////////////
// Debugging hash of cDNA overlap scored gms
// Debugging mode 302 (203)
//
	/** Debugging hash of cDNA overlap scored gms
	 * Debugging mode 302 (203)
	 * - cDNA overlap -
	 *
	 * @param debug
	 * @param callingClass
	 * @param cDnaScoredHash
	 */
        public static void debug_302(int debug,
                                     String callingClass,
                                     HashMap cDnaScoredHash)

        {
          if ( (debug == 1) || (debug == 300) || (debug == 302))
          {
            Debug.debug_203(203, callingClass, cDnaScoredHash);
          }
        }



////////////////////////////////////////////////////////////////////////
// Debugging HashMap of gms pointing to array of overlapping features
// Debugging mode 401
//
    /** Debugging HashMap of gms pointing to array of overlapping features
     * Debugging mode 401
     * - Blast overlap -
     *
     * @param debug
     * @param callingClass
     * @param hashFeatOverlappedGms
     */
    public static void debug_401(int debug,
                                 String callingClass,
                                 HashMap hashFeatOverlappedGms)
    {
      if ( (debug == 1) || (debug == 400) || (debug == 401))
      {
        System.out.println(callingClass);
        for(Iterator itLoc = hashFeatOverlappedGms.keySet().iterator()
            ;itLoc.hasNext();)
        {
          LocateAble gm      = (LocateAble) itLoc.next();
          ArrayList featArr  = (ArrayList) hashFeatOverlappedGms.get(gm);

          System.out.println("GeneModel/Exon        : " + gm.getLocation());

          for(Iterator itFeat = featArr.iterator(); itFeat.hasNext();)
          {
            SimFeature feature = (SimFeature) itFeat.next();
            System.out.println("   overlapping Feature: " + feature.getLocation());
          }
          System.out.println();
        }

      }
    }





////////////////////////////////////////////////////////////////////////
// Debugging HashMap of exons pointing to array of overlapping features
// Debugging mode 402
//
      /** Debugging HashMap of exons pointing to array of overlapping features
       * Debugging mode 402
       * - Blast overlap -
       *
       * @param debug
       * @param callingClass
       * @param hashFeatOverlappedExons
       */
      public static void debug_402(int debug,
                                   String callingClass,
                                   HashMap hashFeatOverlappedExons)
      {
        if ( (debug == 1) || (debug == 400) || (debug == 402)) {
          debug_401(401, callingClass, hashFeatOverlappedExons);
        }
      }





////////////////////////////////////////////////////////////////////////
// Debugging HashMap protein hash for a single gene model
// Debugging mode 403
//
      /** Debugging HashMap protein hash for a single gene model
       * Debugging mode 403
       * - Blast overlap -
       *
       * @param debug
       * @param gm
       * @param proteinHash
       */
      public static void debug_403(int debug,
                                   GeneModel gm,
                                   HashMap proteinHash)
      {
        if ( (debug == 1) || (debug == 400) || (debug == 403))
        {
           System.out.println("FeatureOverlapTools - ProteinHash for gm");
           System.out.println("genemodel: " + gm.getLocation());
           for(Iterator itProt = proteinHash.keySet().iterator();
               itProt.hasNext();)
           {
             String proteinID  = (String) itProt.next();
             ArrayList featArr = (ArrayList) proteinHash.get(proteinID);
             System.out.println("ProteinID: " + proteinID);

             for(Iterator itFeat = featArr.iterator(); itFeat.hasNext();)
             {
               SimFeature feature = (SimFeature) itFeat.next();
               System.out.println("  Feature: " + feature.getLocation());
             }
             System.out.println();
           }
           System.out.println();
           System.out.println();
        }
      }





////////////////////////////////////////////////////////////////////////
// Debugging HashMap of proteins pointing to overlap length
// Debugging mode 404
//
    /** Debugging HashMap of proteins pointing to overlap length
     * Debugging mode 404
     * - Blast overlap -
     *
     * @param debug
     * @param gm
     * @param proteinOverlapLength
     */
    public static void debug_404(int debug,
                                 GeneModel gm,
                                 HashMap proteinOverlapLength)
    {
      if ( (debug == 1) || (debug == 400) || (debug == 404)) {

        System.out.println("FeatureOverlapTools - ProteinOverlapLength for gm");
        System.out.println("genemodel: " + gm.getLocation());

        for(Iterator itProt = proteinOverlapLength.keySet().iterator();
            itProt.hasNext();)
        {
          String proteinID  = (String) itProt.next();
          Long length = (Long) proteinOverlapLength.get(proteinID);
          System.out.println("ProteinID: " + proteinID);
          System.out.println("  Length: " + length);
          System.out.println();
        }
        System.out.println();
        System.out.println();
      }
    }




////////////////////////////////////////////////////////////////////////
// Debugging length of longest protein overlap of single gm
// Debugging mode 405
//
      /** Debugging length of longest protein overlap of single gm
       * Debugging mode 405
       * - Blast overlap -
       *
       * @param debug
       * @param gm
       * @param proteinID
       * @param gmLongestOverlap
       */
      public static void debug_405(int debug,
                                   GeneModel gm,
                                   String proteinID,
                                   long gmLongestOverlap)
      {
        if ( (debug == 1) || (debug == 400) || (debug == 405)) {
          System.out.println("FeatureOverlapTools - ProteinOverlapLength for gm");
          System.out.println("genemodel    : " + gm.getLocation());
          System.out.println("   protein ID: " + proteinID);
          System.out.println("   overlap   : " + gmLongestOverlap);
        }
      }





////////////////////////////////////////////////////////////////////////
// Debugging hash of by equality scored gene models
// Debugging mode 501
//
	  /** Debugging hash of by equality scored gene models
	   * Debugging mode 501
	   *
	   * @param debug
	   * @param scoredGmHash
	   */
          public static void debug_501(int debug,
                                       HashMap scoredGmHash)
          {
            if ( (debug == 1) || (debug == 500) || (debug == 501))
            {
              System.out.println("EqualLocAnalysisTools - score hash (equality)");
              System.out.println("hash size: " + scoredGmHash.size());
              for(Iterator itGm = scoredGmHash.keySet().iterator();
                  itGm.hasNext();)
              {
                GeneModel gm = (GeneModel) itGm.next();
                Double score = (Double) scoredGmHash.get(gm);
                System.out.println("genemodel    : " + gm.getLocation());
                System.out.println("   predicter : "
                + gm.getPrediction().getGenePredicter().getName());
                System.out.println("   score     : " + score);
                System.out.println("");
              }

            }
          }



////////////////////////////////////////////////////////////////////////
// Debugging array of analyzed genemodels
// Debugging mode 701
//
	  /** Debugging array of analyzed genemodels
	   * Debugging mode 701
	   * - general debugging -
	   *
	   * @param debug
	   * @param callingClass
	   * @param analyzedGms
	   */
          public static void debug_701(int debug, String callingClass,
                                       ArrayList analyzedGms)
          {
            if ( (debug == 1) || (debug == 700) || (debug == 701)) {
              System.out.println("SimFeature_Analysis - array of analyzed genemodels");
              System.out.println("called from: " + callingClass);
              int counter = 0;
              for(Iterator itReg = analyzedGms.iterator(); itReg.hasNext();)
              {
                GFMergeRegion region = (GFMergeRegion) itReg.next();
                ArrayList gmArr      = (ArrayList) region.getLocateable();
                for(Iterator itGm = gmArr.iterator(); itGm.hasNext();)
                {
                  counter++;
                  GeneModel gm    = (GeneModel) itGm.next();
                  System.out.println("genemodel    : " + gm.getLocation());
                }
              }
              System.out.println("number of gms: " + counter);
              System.out.println();
              System.out.println();
            }
          }






  }
