/*
 * Copyright (c) 2002 Genome Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.jcon.dao;

import java.util.Collection;

import uk.ac.sanger.jcon.job.Executable;

/**
 * <code>ExecutableDAO</code> describes data access objects for
 * persistence of <code>Executable</code>s. Possible stores are
 * relational databases or XML files.
 *
 * @author Keith James
 */
public interface ExecutableDAO
{
    /**
     * <code>createExecutable</code> creates a new
     * <code>Executable</code> in the store.
     *
     * @param executable an <code>Executable</code>.
     *
     * @exception Exception if an error occurs.
     */
    public void createExecutable(Executable executable) throws Exception;

    /**
     * <code>readAllExecutables</code> reads all
     * <code>Executable</code>s from the store.
     *
     * @return a <code>Collection</code> of <code>Executable</code>s.
     *
     * @exception Exception if an error occurs.
     */
    public Collection readAllExecutables() throws Exception;

    /**
     * <code>readExecutableById</code> reads a single
     * <code>Executable</code> from the store.
     *
     * @param execId an <code>int</code> ID.
     *
     * @return an <code>Executable</code>.
     *
     * @exception Exception if an error occurs.
     */
    public Executable readExecutableById(int execId) throws Exception;

    /**
     * <code>readExecutablesByName</code> reads
     * <code>Executable</code> whose names match the specified pattern
     * from the store.
     *
     * @param name a <code>String</code> pattern.
     *
     * @return a <code>Collection</code> of <code>Executable</code>s.
     *
     * @exception Exception if an error occurs.
     */
    public Collection readExecutablesByName(String name) throws Exception;
}
