/*
 * Copyright (c) 2002 Genome Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.jcon.dao;

import java.util.Collection;

import uk.ac.sanger.jcon.job.Owner;

/**
 * <code>OwnerDAO</code> describes data access objects for persistence
 * of <code>Owner</code>s. Possible stores are relational databases or
 * XML files.
 *
 * @author Keith James
 */
public interface OwnerDAO
{
    /**
     * <code>createOwner</code> creates a new <code>Owner</code> in
     * the store.
     *
     * @param owner an <code>Owner</code>.
     *
     * @exception Exception if an error occurs.
     */
    public void createOwner(Owner owner) throws Exception;

    /**
     * <code>readAllOwners</code> reads all <code>Owner</code>s from
     * the store.
     *
     * @return a <code>Collection</code> of <code>Owner</code>s.
     *
     * @exception Exception if an error occurs.
     */
    public Collection readAllOwners() throws Exception;

    /**
     * <code>readOwnerById</code> reads a single <code>Owner</code>
     * from the store.
     *
     * @param ownerId an <code>int</code> ID.
     *
     * @return an <code>Owner</code>.
     *
     * @exception Exception if an error occurs.
     */
    public Owner readOwnerById(int ownerId) throws Exception;

    /**
     * <code>readOwnerByUserName</code> reads a single
     * <code>Owner</code> from the store.
     *
     * @param userName a <code>String</code>.
     *
     * @return an <code>Owner</code>.
     *
     * @exception Exception if an error occurs.
     */
    public Owner readOwnerByUserName(String userName) throws Exception;

    /**
     * <code>deleteOwner</code> deletes an <code>Owner</code> from the
     * store.
     *
     * @param owner an <code>Owner</code>.
     *
     * @exception Exception if an error occurs.
     */
    public void deleteOwner(Owner owner) throws Exception;
}
