/*
 * Copyright (c) 2002 Genome Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.jcon.dao;

import java.util.Collection;

import uk.ac.sanger.jcon.job.Status;

/**
 * <code>StatusDAO</code> describes data access objects for
 * persistence of <code>Status</code>. Possible stores are relational
 * databases or XML files.
 *
 * @author Keith James
 */
public interface StatusDAO
{
    /**
     * <code>readAllStatus</code> reads all <code>Status</code> from
     * the store.
     *
     * @return a <code>Collection</code> of <code>Status</code>.
     *
     * @exception Exception if an error occurs.
     */
    public Collection readAllStatus() throws Exception;

    /**
     * <code>readStatusById</code> a single <code>Status</code> from
     * the store.
     *
     * @param statusId an <code>int</code> ID.
     *
     * @return a <code>Status</code>.
     *
     * @exception Exception if an error occurs.
     */
    public Status readStatusById(int statusId) throws Exception;
}
