/*
 * Copyright (c) 2002 Genome Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.jcon.dao;

import java.util.Collection;

import uk.ac.sanger.jcon.job.TaskState;

/**
 * <code>TaskStatisticDAO</code> describes data access objects which
 * report data on <code>Task</code>s in the store.
 *
 * @author Keith James
 */
public interface TaskStatisticDAO
{
    /**
     * <code>readSubmittedTaskId</code> returns the IDs of
     * <code>Task</code>s which have been submitted, but which have
     * not yet completed.
     *
     * @return an <code>int []</code> array of IDs.
     */
    public int [] readSubmittedTaskId() throws Exception;

    /**
     * <code>readNSubmittedTaskIdByTime</code> returns the IDs of the n
     * <code>Task</code>s which have been submitted earliest, but
     * which have not yet completed.
     *
     * @param n an <code>int</code> maximum number to retrieve.
     *
     * @return an <code>int []</code> array of IDs.
     */
    public int [] readNSubmittedTaskId(int n) throws Exception;

    /**
     * <code>readSubmittedTaskIdByOwner</code> returns the IDs of the
     * <code>Task</code>s which have been submitted by the specified
     * <code>Owner</code>, but which have not yet completed.
     *
     * @param ownerId an <code>int</code>.
     *
     * @return an <code>int []</code> array of IDs.
     */
    public int [] readSubmittedTaskIdByOwner(int ownerId) throws Exception;

    /**
     * <code>readSubmittedTaskState</code> reads a summary of data for
     * all submitted <code>Task</code>s.
     *
     * @return a <code>Collection</code> of <code>TaskState</code>s.
     *
     * @exception Exception if an error occurs.
     */
    public Collection readSubmittedTaskState() throws Exception;

    /**
     * <code>readTaskState</code> reads a summary of data for the
     * specified <code>Task</code>.
     *
     * @param taskId an <code>int</code> ID.
     *
     * @return a <code>TaskState</code>.
     *
     * @exception Exception if an error occurs.
     */
    public TaskState readTaskState(int taskId) throws Exception;
}
