
package uk.ac.sanger.jcon.gui;

import java.awt.Component;
import javax.swing.ImageIcon;
import javax.swing.JTree;
import javax.swing.tree.DefaultTreeCellRenderer;

import uk.ac.sanger.jcon.job.Job;
import uk.ac.sanger.jcon.job.Status;
import uk.ac.sanger.util.gui.ImageFactory;
import uk.ac.sanger.util.gui.ImageFactoryDefaultImpl;

public class JobTreeCellRenderer extends DefaultTreeCellRenderer
{
    private ImageIcon cancelledIcon;
    private ImageIcon completedIcon;
    private ImageIcon failedIcon;
    private ImageIcon readyIcon;
    private ImageIcon startedIcon;
    private ImageIcon skippedIcon;
    private ImageIcon waitingIcon;

    private StringBuffer sb;

    public JobTreeCellRenderer() throws Exception
    {
        super();
        ImageFactory imFactory = ImageFactoryDefaultImpl.getInstance();
        
        waitingIcon   = new ImageIcon(imFactory.createImage("amber_light"));
        readyIcon     = new ImageIcon(imFactory.createImage("green_light"));
        startedIcon   = new ImageIcon(imFactory.createImage("blue_ring"));
        completedIcon = new ImageIcon(imFactory.createImage("blue_light"));
        failedIcon    = new ImageIcon(imFactory.createImage("red_light"));
        cancelledIcon = new ImageIcon(imFactory.createImage("amber_light"));
        skippedIcon   = new ImageIcon(imFactory.createImage("yellow_light"));

        sb = new StringBuffer();
    }

    public Component getTreeCellRendererComponent(JTree   tree,
                                                  Object  value,
                                                  boolean sel,
                                                  boolean expanded,
                                                  boolean leaf,
                                                  int     row,
                                                  boolean hasFocus)
    {
        super.getTreeCellRendererComponent(tree, value, selected,
                                           expanded, leaf, row, hasFocus);

        Job job = (Job) value;

        setIcon(selectStatusIcon(job.getStatus()));

        sb.setLength(0);
        sb.append("Job [");
        sb.append(job.getId());
        sb.append("] ");
        sb.append(job.getExecutable().getDescription());
        sb.append(" [");

        String op = job.getOperator();
        if (op.equals(Job.OPERATOR_AND))
            sb.append("OP: &&");
        else
            sb.append("OP: ||");

        sb.append("] ");
        sb.append(job.getStatus().getDescription());
        sb.append(" after ");
        sb.append(job.getRetryCount());
        sb.append(" retries (max. ");
        sb.append(job.getMaxRetryCount());
        sb.append(")");

        setText(sb.substring(0));

        return this;
    }

    protected ImageIcon selectStatusIcon(Status status)
    {
        int statusId = status.getId();

        switch (statusId)
        {
            case Status.WAITING:
                return waitingIcon;
            case Status.READY:
                return readyIcon;
            case Status.STARTED:
                return startedIcon;
            case Status.COMPLETED:
                return completedIcon;
            case Status.FAILED:
                return failedIcon;
            case Status.CANCELLED:
                return cancelledIcon;
            case Status.SKIPPED:
                return skippedIcon;
            default:
                throw new IllegalArgumentException("No Job icon for Status ["
                                                   + statusId
                                                   + "] "
                                                   + status.getDescription());
        }
    }
}
