
package uk.ac.sanger.jcon.gui;

import java.awt.BorderLayout;
import java.awt.Font;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.sql.DataSource;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeSelectionModel;

import uk.ac.sanger.jcon.dao.TaskDAO;
import uk.ac.sanger.jcon.dao.TaskSQLDAO;
import uk.ac.sanger.jcon.job.Job;
import uk.ac.sanger.jcon.job.Task;

public class JobViewer extends JFrame
{
    private Job []           rootJobs;
    private int              currentJobIndex;
    private DefaultTreeModel model;
    private JTree            tree;
    private JScrollPane      treePane;

    public static void main(String [] argv) throws Exception
    {
        Context context = new InitialContext();
        DataSource datasource = (DataSource) context.lookup("jdbc/JobControl");

        JobViewer viewer = new JobViewer(datasource,
                                         Integer.parseInt(argv[0]));
  
        viewer.setSize(400, 600);
        viewer.setVisible(true);
    }

    public JobViewer(DataSource datasource, int taskId) throws Exception
    {
        TaskSQLDAO taskSQLDAO = new TaskSQLDAO(datasource);

        Task task = taskSQLDAO.readTaskById(taskId, TaskDAO.INCLUDE_JOB_TREE);
        rootJobs = task.getJobs();
        currentJobIndex  = rootJobs.length - 1;

        initComponents();
    }

    private void initComponents() throws Exception
    {
        model = new DefaultTreeModel(rootJobs[currentJobIndex]);
        tree = new JTree(model);
        treePane = new JScrollPane(tree);

        tree.putClientProperty("JTree.lineStyle", "Angled");
        tree.getSelectionModel().setSelectionMode
            (TreeSelectionModel.SINGLE_TREE_SELECTION);

        tree.setCellRenderer(new JobTreeCellRenderer());
        tree.setFont(new Font("Sanserif", Font.PLAIN, 11));

        // Listen for changes to the TreeModel
        model.addTreeModelListener(repaintListener);

        getContentPane().add(treePane, BorderLayout.CENTER);

        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    }

    public Job [] getJobs()
    {
        return rootJobs;
    }

    public void setJobs(Job [] rootJobs)
    {
        this.rootJobs = rootJobs;
        setCurrentJobIndex(rootJobs.length - 1);
    }

    public void setCurrentJobIndex(int currentJobIndex)
    {
        this.currentJobIndex = currentJobIndex;

        Job rootJob = rootJobs[currentJobIndex];
        model.setRoot(rootJob);
        // Fire an event to indicate a total tree change
        model.nodeStructureChanged(rootJob);
        // Expand the tree
        for (int i = tree.getRowCount(); i-- >= 0;)
            tree.expandRow(i);
    }

    private TreeModelListener repaintListener = new TreeModelListener()
    {
        public void treeNodesChanged(TreeModelEvent tme)
        {
            repaint();
        }

        public void treeNodesInserted(TreeModelEvent tme)
        {
            treeNodesChanged(tme);
        }
 
        public void treeNodesRemoved(TreeModelEvent tme)
        {
            treeNodesChanged(tme);
        }

        public void treeStructureChanged(TreeModelEvent tme)
        {
            treeNodesChanged(tme);
        }
    };
}





