/*
 * Copyright (c) 2002 Genome Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.jcon.job;

/**
 * <code>BatchJob</code>s are <code>Job</code>s which are assigned to
 * a batch queue.
 *
 * @author Keith James
 */
public interface BatchJob extends Job
{
    /**
     * <code>MAX_QUEUE_LEN</code> is the maximum permitted length of a
     * queue name.
     */
    public static final int MAX_QUEUE_LEN = 12;

    /**
     * <code>getBatchId</code> returns the job ID which was assigned
     * by a batch system when the job was submitted, or 0 if the job
     * has not yet been submitted.
     *
     * @return an <code>int</code>.
     */
    public int getBatchId();

    /**
     * <code>setBatchId</code> sets the job ID which was assigned by a
     * batch system when the job was submitted.
     *
     * @param batchId an <code>int</code>.
     */
    public void setBatchId(int batchId);

    /**
     * <code>getQueue</code> returns the batch queue name.
     *
     * @return a <code>String</code>.
     */
    public String getQueue();

    /**
     * <code>setQueue</code> sets the batch queue name.
     *
     * @param queue a <code>String</code>.
     */
    public void setQueue(String queue);

    /**
     * <code>getHosts</code> returns the host(s) on which the batch
     * system ran the job.
     *
     * @return a <code>String []</code> array of host names, or an
     * empty array if the job has not yet been assigned to a host.
     */
    public String [] getHosts();

    /**
     * <code>setHosts</code> sets the host(s) on which the batch
     * system ran the job.
     *
     * @param hosts a <code>String []</code> array of host names.
     */
    public void setHosts(String [] hosts);
}
