/*
 * Copyright (c) 2002 Genome Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.jcon.job;

import java.io.Serializable;

/**
 * <p>An <code>Executable</code> represents a task to be executed,
 * including a description of its purpose, a name indicating the
 * resource (binary executable etc.), a name indicating the working
 * directory, any command arguments and environment variables.</p>
 *
 * @author Keith James
 */
public interface Executable extends Serializable
{
    /**
     * <code>MAX_NAME_LEN</code> is the maximum length permitted for a
     * name.
     */
    public static final int MAX_NAME_LEN = 64;

    /**
     * <code>MAX_VER_LEN</code> is the maximum length permitted for a
     * version.
     */
    public static final int MAX_VER_LEN = 32;

    /**
     * <code>MAX_DESC_LEN</code> is the maximum length permitted for a
     * description.
     */
    public static final int MAX_DESC_LEN = 128;

    /**
     * <code>getId</code> returns the ID.
     *
     * @return an <code>int</code>.
     */
    public int getId();

    /**
     * <code>getDescription</code> returns the free-text description
     * of the purpose of the executable.
     *
     * @return a <code>String</code>.
     */
    public String getDescription();

    /**
     * <code>setDescription</code> sets the free-text description of
     * the purpose of the executable.
     *
     * @param description a <code>String</code>.
     */
    public void setDescription(String description);

    /**
     * <code>getVersion</code> returns the version of the resource to
     * be executed.
     *
     * @return a <code>String</code>.
     */
    public String getVersion();

    /**
     * <code>setVersion</code> sets the version of the resource to be
     * executed.
     */
    public void setVersion(String version);

    /**
     * <code>getName</code> returns the name used to locate the
     * resource to execute. Currently it is expected that this is
     * simply a file name in the local filesystem. Later this could be
     * applied to other systems such as using a URL or JNDI.
     *
     * @return a <code>String</code>.
     */
    public String getName();

    /**
     * <code>setName</code> sets the name used to locate the resource
     * to execute.
     *
     * @param name a <code>String</code>.
     */
    public void setName(String name);
}
