/*
 * Copyright (c) 2002 Genome Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.jcon.job;

import java.io.Serializable;

/**
 * <code>Owner</code> represents the owner and initiator of a
 * <code>Task</code>.
 *
 * @author Keith James
 */
public interface Owner extends Serializable
{
    /**
     * <code>MAX_USERNAME_LEN</code> is the maximum permitted username
     * length.
     */
    public static final int MAX_USERNAME_LEN = 8;

    /**
     * <code>MAX_DETAIL_LEN</code> is the maximum permitted user
     * detail length.
     */
    public static final int MAX_DETAIL_LEN = 32;

    /**
     * <code>getId</code> returns the ID.
     *
     * @return an <code>int</code>.
     */
    public int getId();

    /**
     * <code>getFirstName</code> returns the owner's first name.
     *
     * @return a <code>String</code>.
     */
    public String getFirstName();

    /**
     * <code>getLastName</code> returns the owner's last name.
     *
     * @return a <code>String</code>.
     */
    public String getLastName();

    /**
     * <code>getUserName</code> returns the owner's user name.
     *
     * @return a <code>String</code>.
     */
    public String getUserName();

    /**
     * <code>getEMail</code> returns the owner's email address.
     *
     * @return a <code>String</code>.
     */
    public String getEMail();

    /**
     * <code>setEMail</code> sets the owner's email address.
     *
     * @param email a <code>String</code>.
     */
    public void setEMail(String eMail);

    /**
     * <code>getTelephone</code> returns the owner's telephone number.
     *
     * @return a <code>String</code>.
     */
    public String getTelephone();

    /**
     * <code>setTelephone</code> sets the owner's telephone number.
     *
     * @param telephone a <code>String</code>.
     */
    public void setTelephone(String telephone);
}
