/*
 * Copyright (c) 2002 Genome Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.jcon.job;

/**
 * <code>OwnerDefaultImpl</code> represents the owner and initiator of
 * a <code>Task</code>.
 *
 * @author Keith James
 */
public class OwnerDefaultImpl implements Owner
{
   /**
     * The unique ID of an <code>Owner</code> instance.
     */
    private int id;

    /**
     * <code>userName</code> is the owner's user name.
     */
    protected String userName;

    /**
     * <code>firstName</code> is the owner's first name.
     */
    protected String firstName;

   /**
     * <code>lastName</code> is the owner's last name.
     */
    protected String lastName;

    /**
     * <code>eMail</code> is the owner's email address.
     */
    protected String eMail;

    /**
     * <code>telephone</code> is the owner's telephone number.
     */
    protected String telephone;

    /**
     * Creates a new <code>OwnerDefaultImpl</code>.
     *
     * @param firstName a <code>String</code>.
     * @param lastName a <code>String</code>.
     * @param userName a <code>String</code>.
     * @param eMail a <code>String</code>.
     * @param telephone a <code>String</code>.
     */
    public OwnerDefaultImpl(String firstName, String lastName,
                            String userName, String eMail, String telephone)
    {
        if (firstName == null)
            throw new IllegalArgumentException("Failed to set firstName as it was null");
        if (lastName == null)
            throw new IllegalArgumentException("Failed to set lastName as it was null");
        if (userName == null)
            throw new IllegalArgumentException("Failed to set userName as it was null");
        if (eMail == null)
            throw new IllegalArgumentException("Failed to set eMail as it was null");
        if (telephone == null)
            throw new IllegalArgumentException("Failed to set telephone as it was null");

        checkFieldLength(firstName, Owner.MAX_DETAIL_LEN);
        checkFieldLength(lastName, Owner.MAX_DETAIL_LEN);
        checkFieldLength(userName, Owner.MAX_USERNAME_LEN);
        checkFieldLength(eMail, Owner.MAX_DETAIL_LEN);
        checkFieldLength(telephone, Owner.MAX_DETAIL_LEN);

        this.firstName = firstName;
        this.lastName  = lastName;
        this.userName  = userName;
        this.eMail     = eMail;
        this.telephone = telephone;
    }

    public int getId()
    {
        return id;
    }

    public String getFirstName()
    {
        return firstName;
    }

    public String getLastName()
    {
        return lastName;
    }

    public String getUserName()
    {
        return userName;
    }

    public String getEMail()
    {
        return eMail;
    }

    public void setEMail(String eMail)
    {
        if (eMail == null)
            throw new IllegalArgumentException("Failed to set eMail as it was null");
        checkFieldLength(eMail, Owner.MAX_DETAIL_LEN);
        this.eMail = eMail;
    }

    public String getTelephone()
    {
        return telephone;
    }

    public void setTelephone(String telephone)
    {
        if (telephone == null)
            throw new IllegalArgumentException("Failed to set telephone as it was null");
        checkFieldLength(telephone, Owner.MAX_DETAIL_LEN);
        this.telephone = telephone;
    }

    public String toString()
    {
        StringBuffer sb = new StringBuffer("Owner [");
        sb.append(id);
        sb.append("] '" );
        sb.append(firstName);
        sb.append(" ");
        sb.append(lastName);
        sb.append(", Username: ");
        sb.append(userName);
        sb.append(", Email: ");
        sb.append(eMail);
        sb.append("'");

        return sb.substring(0);
    }

    protected void checkFieldLength(String field, int maxLength)
    {
        if (field.length() > maxLength)
            throw new IllegalArgumentException("Field "
                                               + field
                                               + " is longer than the permitted maximum of "
                                               + maxLength
                                               + " characters");
    }
}
