/*
 * Copyright (c) 2002 Genome Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.jcon.job;

import java.io.Serializable;

/**
 * <code>Status</code> represents the run status of a
 * <code>Job</code>.
 *
 * @author Keith James
 */
public interface Status extends Serializable
{
    /**
     * <code>WAITING</code> is a valid ID for a WAITING
     * <code>Status</code>. This status means that nothing has been
     * done and it is not yet known that anything can be done.
     */
    public static final int WAITING = 1;

    /**
     * <code>READY</code> is a valid ID for a READY
     * <code>Status</code>. This status means that nothing has been
     * done but it is known that operations may be started without
     * error.
     */
    public static final int READY = 2;

    /**
     * <code>STARTED</code> is a valid ID for a STARTED
     * <code>Status</code>. This status means that an operation has
     * been started and is not yet known to have succeeded or failed.
     */
    public static final int STARTED = 3;

    /**
     * <code>COMPLETED</code> is a valid ID for a COMPLETED
     * <code>Status</code>. This status means that an operation has
     * completed successfully and no further actions are expected to
     * take place.
     */
    public static final int COMPLETED = 4;

    /**
     * <code>FAILED</code> is a valid ID for a FAILED
     * <code>Status</code>. This status means that an operation has
     * failed.
     */
    public static final int FAILED = 5;

    /**
     * <code>CANCELLED</code> is a valid ID for a CANCELLED
     * <code>Status</code>. This status means that a request has been
     * issued for the operation to halt and it is known to have done
     * so. This is explictly set by user intervention. The system does
     * not CANCEL and operation; it SKIPs them.
     */
    public static final int CANCELLED = 6;

    /**
     * <code>SKIPPED</code> is a valid ID for a SKIPPED
     * <code>Status</code>. This status means that an operation has
     * been marked as no longer necessary by the system. This has been
     * done without explicit user intervention.
     */
    public static final int SKIPPED = 7;

    /**
     * <code>getId</code> returns the ID.
     *
     * @return an <code>int</code>.
     */
    public int getId();

    /**
     * <code>getDescription</code> returns the status description.
     *
     * @return a <code>String</code>.
     */
    public String getDescription();
}
