/*
 * Copyright (c) 2002 Genome Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.jcon.job;

/**
 * <code>StatusDefaultImpl</code> represents the run status of a
 * <code>Job</code>.
 *
 * @author Keith James
 */
public class StatusDefaultImpl implements Status
{
    private static String [] validDescriptions = { "DUMMY",
                                                   "WAITING",
                                                   "READY",
                                                   "STARTED",
                                                   "COMPLETED",
                                                   "FAILED",
                                                   "CANCELLED",
                                                   "SKIPPED" };

    /**
     * The unique ID of a <code>Status</code> instance.
     */
    private int id;

    /**
     * <code>description</code> is a freetext description of the
     * status.
     */
    private String description;

    /**
     * Creates a new <code>StatusDefaultImpl</code>.
     *
     * @param description a <code>String</code>.
     */
    public StatusDefaultImpl(String description)
    {
        for (int i = 1; i < validDescriptions.length; i++)
        {
            if (description.equals(validDescriptions[i]))
                break;
            else if (i == validDescriptions.length)
                throw new IllegalArgumentException("Invalid description "
                                                   + description);
        }

        this.description = description;
    }

    /**
     * <code>getId</code> returns the ID.
     *
     * @return an <code>int</code>.
     */
    public int getId()
    {
        return id;
    }

    /**
     * <code>getDescription</code> returns the status description.
     *
     * @return a <code>String</code>.
     */
    public String getDescription()
    {
        return description;
    }

    public String toString()
    {
        StringBuffer sb = new StringBuffer("Status [");
        sb.append(id);
        sb.append("] '");
        sb.append(description);
        sb.append("'");

        return sb.substring(0);
    }
}
