/*
 * Copyright (c) 2002 Genome Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.jcon.lsf;

/**
 * <code>LSFBatchException</code> is an exception thrown when a
 * problem occurs accessing the LSF batch system.
 *
 * @author Keith James
 */
public class LSFBatchException extends Exception
{
    private final String lsfMessage;

    /**
     * Creates an <code>LSFBatchException</code> with no detail
     * message.
     */
    public LSFBatchException()
    {
        super();
        lsfMessage = "none";
    }

    /**
     * Creates an <code>LSFBatchException</code> with the specified
     * detail message.
     *
     * @param message a <code>String</code>.
     */
    public LSFBatchException(String message)
    {
        super(message);
        lsfMessage = "none";
    }

    /**
     * Creates an <code>LSFBatchException</code> with the specified
     * detail message and an additional message from the LSF C API.
     *
     * @param message a <code>String</code>.
     * @param lsfMessage a <code>String</code>.
     */
    public LSFBatchException(String message, String lsfMessage)
    {
        super(message);
        this.lsfMessage = lsfMessage;
    }

    public String toString()
    {
        return super.toString() + ". LSF message: '" + lsfMessage + "'";
    }
}
