/*
 * Copyright (c) 2002 Genome Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.jcon.manager;

import java.io.IOException;
import java.sql.Timestamp;
import java.util.Collection;
import java.util.List;

import org.apache.log4j.Category;

import uk.ac.sanger.jcon.dao.ExecutableDAO;
import uk.ac.sanger.jcon.dao.OwnerDAO;
import uk.ac.sanger.jcon.dao.StatusDAO;
import uk.ac.sanger.jcon.dao.TaskDAO;
import uk.ac.sanger.jcon.job.Executable;
import uk.ac.sanger.jcon.job.Owner;
import uk.ac.sanger.jcon.job.Status;
import uk.ac.sanger.jcon.job.Task;

/**
 * <code>AdministratorDefaultImpl</code> is a basic implementation of
 * <code>Administrator</code>.
 *
 * @author Keith James
 */
public class AdministratorDefaultImpl implements Administrator
{
    // Log4j setup
    static Category cat = Category.getInstance(AdministratorDefaultImpl.class.getName());

    private OwnerDAO      ownerDAO;
    private ExecutableDAO executableDAO;
    private StatusDAO     statusDAO;
    private TaskDAO       taskDAO;

    /**
     * Creates a new <code>AdministratorDefaultImpl</code> using the
     * specified data access objects.
     *
     * @param ownerDAO an <code>OwnerDAO</code>.
     * @param executableDAO an <code>ExecutableDAO</code>.
     * @param statusDAO a <code>StatusDAO</code>.
     * @param taskDAO a <code>TaskDAO</code>.
     */
    public AdministratorDefaultImpl(OwnerDAO ownerDAO, ExecutableDAO executableDAO,
                                    StatusDAO statusDAO, TaskDAO taskDAO)
    {
        this.ownerDAO      = ownerDAO;
        this.executableDAO = executableDAO;
        this.statusDAO     = statusDAO;
        this.taskDAO       = taskDAO;
    }

    public void createExecutable(Executable executable) throws Exception
    {
        executableDAO.createExecutable(executable);
    }

    public Executable getExecutableById(int execId) throws Exception
    {
        return executableDAO.readExecutableById(execId);
    }

    public Collection getExecutables() throws Exception
    {
        return executableDAO.readAllExecutables();
    }

    public Collection getExecutablesByName(String name) throws Exception
    {
        return executableDAO.readExecutablesByName(name);
    }

    public void createOwner(Owner owner) throws Exception
    {
        ownerDAO.createOwner(owner);
    }

    public Owner getOwnerById(int ownerId) throws Exception
    {
        return ownerDAO.readOwnerById(ownerId);
    }

    public Collection getOwners() throws Exception
    {
        return ownerDAO.readAllOwners();
    }

    public Owner getOwnerByUserName(String userName) throws Exception
    {
        return ownerDAO.readOwnerByUserName(userName);
    }

    public Status getStatusById(int statusId) throws Exception
    {
        return statusDAO.readStatusById(statusId);
    }

    public Collection getStatus() throws Exception
    {
        return statusDAO.readAllStatus();
    }

    public Task getTaskById(int taskId) throws Exception
    {
        return taskDAO.readTaskById(taskId, TaskDAO.INCLUDE_JOB_TREE);
    }

    public Collection getTasksByOwner(Owner owner) throws Exception
    {
        return taskDAO.readTasksByOwner(owner);
    }

    public Collection getTasksByName(String name) throws Exception
    {
        return taskDAO.readTasksByName(name);
    }

    public Collection getTasksByPriority(int priority) throws Exception
    {
        return taskDAO.readTasksByPriority(priority);
    }

    public Collection getTasksSubmittedBefore(Timestamp time) throws Exception
    {
        return taskDAO.readTasksSubmittedBefore(time);
    }

    public Collection getTasksSubmittedAfter(Timestamp time) throws Exception
    {
        return taskDAO.readTasksSubmittedAfter(time);
    }

    public Collection getTasksSubmittedBetween(Timestamp earliest,
                                               Timestamp latest) throws Exception
    {
        return taskDAO.readTasksSubmittedBetween(earliest, latest);
    }
}
