/*
 * Copyright (c) 2002 Genome Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.jcon.manager;

import uk.ac.sanger.jcon.job.Task;

/**
 * <code>TaskManager</code> defines methods for basic
 * <code>Task</code> manipulation.
 *
 * @author Keith James
 */
public interface TaskManager
{
    /**
     * <code>createTask</code> creates a new <code>Task</code> in the
     * store (but does not submit it).
     *
     * @param task a <code>Task</code>.
     *
     * @return an <code>int</code> ID of the new <code>Task</code>.
     *
     * @exception Exception if an error occurs.
     */
    public int createTask(Task task) throws Exception;

    /**
     * <code>submitTask</code> submits a <code>Task</code> for
     * running.
     *
     * @param taskId an <code>int</code>.
     *
     * @exception Exception if an error occurs.
     */
    public void submitTask(int taskId) throws Exception;

    /**
     * <code>cancelTask</code> cancels an existing <code>Task</code>.
     *
     * @param task a <code>Task</code>.
     *
     * @exception Exception if an error occurs.
     */
    public void cancelTask(int taskId) throws Exception;

    /**
     * <code>prioritiseTask</code> assigns a new priority to an
     * existing <code>Task</code>.
     *
     * @param taskId an <code>int</code>, the ID of an existing
     * <code>Task</code>.
     * @param priority an <code>int</code> priority.
     * @exception Exception if an error occurs.
     */
    public void prioritiseTask(int taskId, int priority) throws Exception;
}
