/*
 * Copyright (c) 2002 Genome Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.jcon.manager;

import uk.ac.sanger.jcon.JobControlException;
import uk.ac.sanger.jcon.dao.JobDAO;
import uk.ac.sanger.jcon.dao.TaskDAO;
import uk.ac.sanger.jcon.job.Job;
import uk.ac.sanger.jcon.job.Task;

/**
 * <code>TaskManagerDefaultImpl</code> is a basic implementation of
 * <code>TaskManager</code>.
 *
 * @author Keith James
 */
public class TaskManagerDefaultImpl implements TaskManager
{
    private TaskDAO taskDAO;
    private JobDAO jobDAO;

    /**
     * Creates a new <code>TaskManagerDefaultImpl</code> using the
     * specified data access objects.
     *
     * @param taskDAO a <code>TaskDAO</code>.
     * @param jobDAO a <code>JobDAO</code>.
     */
    public TaskManagerDefaultImpl(TaskDAO taskDAO, JobDAO jobDAO)
    {
        this.taskDAO = taskDAO;
        this.jobDAO  = jobDAO;
    }

    public int createTask(Task task) throws Exception
    {
        return taskDAO.createTask(task);
    }

    public void submitTask(int taskId) throws Exception
    {
        taskDAO.updateTaskSubmitted(taskId);
    }

    public void cancelTask(int taskId) throws Exception
    {
        taskDAO.updateTaskCancelled(taskId);
    }

    public void prioritiseTask(int taskId, int priority) throws Exception
    {
        Task task = taskDAO.readTaskById(taskId, TaskDAO.EXCLUDE_JOB_TREE);
        task.setPriority(priority);
        taskDAO.updateTask(task);
    }
}
