/*
 * Copyright (c) 2002 Genome Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.jcon.protocol;

import java.util.Map;

/**
 * <code>Protocol</code> is a definition for creation and/or
 * submission of <code>Task</code>s. This is really a temporary
 * measure until we get a LIMS-type system to define analyses and
 * states of data.
 *
 * @author Keith James
 */
public interface Protocol
{
	/**
	 * <code>apply</code> applies the <code>Protocol</code> using the
	 * supplied parameters.
	 *
	 * @param ownerName a <code>String</code> identifying the owner,
	 * usually a username.
     * @param config a <code>Map</code> of configuration parameters
     * and corresponding values.
	 * @param data a <code>String []</code> array of data parameters.
     *
	 * @exception Exception if an error occurs.
	 */
	public void apply(String ownerName, Map config, String [] data)
        throws Exception;
}
