/*
 * Copyright (c) 2002 Genome Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.jcon.run;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

/**
 * <code>BufferedStreamEmptier</code> reads from an
 * <code>InputStream</code> and writes to an <code>OutputStream</code>
 * in its own thread, thus emptying the former into the latter. It
 * wraps both in buffered streams.
 *
 * @author Keith James
 */
public class BufferedStreamEmptier extends Thread
{
    private BufferedInputStream  bis;
    private BufferedOutputStream bos;

    /**
     * Creates a new <code>BufferedStreamEmptier</code> reading from
     * the <code>InputStream</code> and writing to the
     * <code>OutputStream</code>.
     *
     * @param is an <code>InputStream</code>.
     * @param os an <code>OutputStream</code>.
     */
    public BufferedStreamEmptier(InputStream is, OutputStream os)
    {
        bis = new BufferedInputStream(is);
        bos = new BufferedOutputStream(os);
    }

    public void run()
    {
        try
        {
            int i;
            while ((i = bis.read()) != -1)
            {
                bos.write(i);
            }

            bis.close();
            bos.close();
        }
        catch (IOException ioe)
        {
            ioe.printStackTrace();
        }
    }
}
