/*
 * Copyright (c) 2002 Genome Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.jcon.run;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.StringWriter;

/**
 * <code>BufferedStringSink</code> reads from an
 * <code>InputStream</code> and stores the bytes read in a
 * <code>StringBuffer</code>. A convenience class for saving
 * relatively small amounts of input from streams.
 *
 * @author Keith James
 */
public class BufferedStringSink extends Thread
{
    private BufferedReader br;
    private StringWriter sw;

    /**
     * Creates a new <code>BufferedStringSink</code> instance.
     *
     * @param is an <code>InputStream</code> to read from.
     */
    public BufferedStringSink(InputStream is)
    {
        br = new BufferedReader(new InputStreamReader(is));
        sw = new StringWriter();
    }

    public void run()
    {
        try
        {
            int i;
            while ((i = br.read()) != -1)
            {
                sw.write(i);
            }

            br.close();
            sw.flush();
            sw.close();
        }
        catch (IOException ioe)
        {
            ioe.printStackTrace();
        }
    }

    /**
     * <code>getBuffer</code> returns a buffer containing the
     * accumulated chars.
     *
     * @return a <code>StringBuffer</code>.
     */
    public StringBuffer getBuffer()
    {
        return sw.getBuffer();
    }
}
