/*
 * Copyright (c) 2002 Genome Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.jcon.run;

import uk.ac.sanger.jcon.job.Job;

/**
 * <code>CommandLineFactory</code> defines the interface for classes
 * which construct a command line from a <code>Job</code> object.
 *
 * @author Keith James
 */
public interface CommandLineFactory
{
    /**
     * <code>EXECUTABLE</code> is the token in command line templates
     * which is expanded to the <code>Executable</code> name.
     */
    public static final String EXECUTABLE = "executable";

    /**
     * <code>INPUT_NAME</code> is the token in command line templates
     * which is expanded to the input name.
     */
    public static final String INPUT_NAME = "input_name";

    /**
     * <code>OUTPUT_NAME</code> is the token in command line templates
     * which is expanded to the output name.
     */
    public static final String OUTPUT_NAME = "output_name";

    /**
     * <code>ERROR_NAME</code> is the token in command line templates
     * which is expanded to the error name.
     */
    public static final String ERROR_NAME = "error_name";

    /**
     * <code>JOB_ID</code> s the token in command line templates
     * which is expanded to the <code>Job</code>'s unique ID.
     */
    public static final String JOB_ID = "job_id";

    /**
     * <code>createCommandLine</code> creates a new command line
     * string from the <code>Job</code>. This method should validate
     * the <code>Job</code>'s command template to ensure that it
     * contains the "$executable" token which is essential for
     * associating the <code>Executable</code> with the generated
     * command line.
     *
     * @param job a <code>Job</code>.
     *
     * @return a <code>String</code>.
     *
     * @exception Exception if an error occurs.
     */
    public String createCommandLine(Job job) throws Exception;
}
