/*
 * Copyright (c) 2002 Genome Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.jcon.run;

import org.apache.log4j.Category;

import uk.ac.sanger.jcon.dao.TaskStatisticDAO;

/**
 * <code>PolicyFIFOImpl</code> implements a simple FIFO selection
 * policy where the unfinished <code>Task</code> submitted earliest is
 * selected.
 *
 * @author Keith James
 */
public final class PolicyFIFOImpl implements Policy
{
    // Log4j setup
    static Category cat =
        Category.getInstance(PolicyFIFOImpl.class.getName());

    private TaskStatisticDAO taskStatisticDAO;
    private int count = 1;

    /**
     * Creates a new <code>PolicyFIFOImpl</code> using the specified
     * data access object.
     *
     * @param taskStatisticDAO a <code>TaskStatisticDAO</code>.
     */
    public PolicyFIFOImpl(TaskStatisticDAO taskStatisticDAO)
    {
        this.taskStatisticDAO = taskStatisticDAO;
    }

    public int selectTaskId()
    {
        int [] taskIds;

        try
        {
            cat.debug("Reading Task statistics");

            taskIds = taskStatisticDAO.readNSubmittedTaskId(count);

            cat.debug("Found " + taskIds.length + " submitted Task(s)");
        }
        catch (Exception e)
        {
            cat.error("Caught Exception while fetching Task IDs", e);
            return -1;
        }

        if (taskIds.length > 0)
            return taskIds[0];
        else
            return -1;
    }
}
