/*
 * Copyright (c) 2002 Genome Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.jcon.run;

import uk.ac.sanger.jcon.dao.StatusDAO;

/**
 * <code>ProcessorFactory</code> is an interface for objects which can
 * build <code>Processor</code>s.
 *
 * @author Keith James
 */
public interface ProcessorFactory
{
    /**
     * <code>createProcessor</code> creates a new
     * <code>Processor</code> using the specified factory to generate
     * command lines and data access object to obtain information
     * about valid <code>Job</code> status.
     *
     * @param clFactory a <code>CommandLineFactory</code>.
     * @param statusDAO a <code>StatusDAO</code>.
     * @return a <code>Processor</code>.
     */
    public Processor createProcessor(CommandLineFactory clFactory,
                                     StatusDAO          statusDAO);
}
