/*
 * Copyright (c) 2002 Genome Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.jcon.servlets;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.ServletConfig;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import org.apache.log4j.Category;
import org.apache.velocity.Template;
import org.apache.velocity.app.Velocity;
import org.apache.velocity.context.Context;
import org.apache.velocity.servlet.VelocityServlet;

/**
 * <code>JobControlServlet</code> provides web access to the job
 * control system state. Database access must be provided via JNDI
 * under the name "jdbc/JobControl".
 *
 * @author Keith James
 */
public class JobControlServlet extends VelocityServlet
{
    // Log4j setup
    static Category cat = Category.getInstance(JobControlServlet.class.getName());

    private Map factories;

    public JobControlServlet()
    {
        factories = new HashMap(3);
        factories.put("owners",      new TemplateFactoryOwnerImpl(this));
        factories.put("executables", new TemplateFactoryExecutableImpl(this));
        factories.put("taskstate",   new TemplateFactoryTaskStateImpl(this));
    }

    public Template handleRequest(HttpServletRequest  request, 
                                  HttpServletResponse response,
                                  Context             context)
    {
        Template template = null;

        String action = request.getParameter("action");

        cat.info("Recieved request for template '"
                 + action
                 + "'");
        try
        {
            template = ((TemplateFactory) factories.get(action)).createTemplate(context);
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }

        return template;
    }

    protected Properties loadConfiguration(ServletConfig config)
        throws FileNotFoundException, IOException
    {
        String propsFile =
            config.getInitParameter(VelocityServlet.INIT_PROPS_KEY);

        Properties p = new Properties();

        if (propsFile != null)
        {
            String realPath = getServletContext().getRealPath(propsFile);

            if (realPath != null)
                propsFile = realPath;

            p.load(new FileInputStream(propsFile));
        }

        String log = p.getProperty(Velocity.RUNTIME_LOG);

        if (log != null)
        {
            log = getServletContext().getRealPath(log);
            
            if (log != null)
                p.setProperty(Velocity.RUNTIME_LOG, log);
        }

        String path = p.getProperty(Velocity.FILE_RESOURCE_LOADER_PATH);

        if (path != null)
        {
            path = getServletContext().getRealPath(path);

            if (path != null)
                p.setProperty(Velocity.FILE_RESOURCE_LOADER_PATH, path);
        }

        return p;
    }

    protected DataSource getDataSource() throws NamingException
    {
        javax.naming.Context initContext = new InitialContext();
        javax.naming.Context servContext =
            (javax.naming.Context) initContext.lookup("java:comp/env");

        return (DataSource) servContext.lookup("jdbc/JobControl");
    }
}

