/*
 * Copyright (c) 2002 Genome Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.jcon.servlets;

import java.util.Collection;
import java.util.Iterator;
import javax.sql.DataSource;

import org.apache.log4j.Category;
import org.apache.velocity.Template;
import org.apache.velocity.context.Context;
import org.apache.velocity.exception.ParseErrorException;
import org.apache.velocity.exception.ResourceNotFoundException;

import uk.ac.sanger.jcon.dao.ExecutableDAO;
import uk.ac.sanger.jcon.dao.ExecutableSQLDAO;
import uk.ac.sanger.jcon.job.Executable;

/**
 * <code>TemplateFactoryExecutableImpl</code> creates a template
 * populated with <code>Executable</code> data.
 *
 * @author Keith James
 */
class TemplateFactoryExecutableImpl implements TemplateFactory
{
    // Log4j setup
    static Category cat = Category.getInstance(TemplateFactoryExecutableImpl.class.getName());

    private JobControlServlet servlet;

    TemplateFactoryExecutableImpl(JobControlServlet servlet)
    {
        this.servlet = servlet;
    }

    public Template createTemplate(Context context) throws Exception
    {
        Template template = null;

        try
        {
            DataSource datasource = servlet.getDataSource();
            ExecutableDAO executableDAO = new ExecutableSQLDAO(datasource);
            Collection executables = executableDAO.readAllExecutables();
            Table execTable = new Table();

            for (Iterator ei = executables.iterator(); ei.hasNext();)
            {
                Executable e = (Executable) ei.next();
                int id = e.getId();

                Table.Row row = execTable.new Row(4);

                Table.Cell cell = execTable.new Cell(Integer.toString(id));
                cell.setMetaData(ServletConstants.CSS_ALIGN_R);

                row.addCell(cell);
                row.addCell(execTable.new Cell(e.getName()));
                row.addCell(execTable.new Cell(e.getVersion()));
                row.addCell(execTable.new Cell(e.getDescription()));

                if (id % 2 == 0)
                    row.setMetaData(ServletConstants.CSS_DARK_BG);
                else
                    row.setMetaData(ServletConstants.CSS_PALE_BG);

                execTable.addRow(row);
            }

            cat.debug("Result table is:\n" + execTable);

            context.put("execTable",  execTable);
            template = servlet.getTemplate("executables.vm");
        }
        catch (ResourceNotFoundException rnfe)
        {
            System.err.println("JobControlServlet: template not found");
            throw rnfe;
        }
        catch (ParseErrorException pee)
        {
            System.err.println("JobControlServlet: parse error for template");
            throw pee;
        }

        return template;
    }
}
