/*
 * Copyright (c) 2002 Genome Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.jcon.servlets;

import java.util.Collection;
import java.util.Iterator;
import javax.sql.DataSource;

import org.apache.log4j.Category;
import org.apache.velocity.Template;
import org.apache.velocity.context.Context;
import org.apache.velocity.exception.ParseErrorException;
import org.apache.velocity.exception.ResourceNotFoundException;

import uk.ac.sanger.jcon.dao.OwnerDAO;
import uk.ac.sanger.jcon.dao.OwnerSQLDAO;
import uk.ac.sanger.jcon.job.Owner;

/**
 * <code>TemplateFactoryOwnerImpl</code> creates a template
 * populated with <code>Owner</code> data.
 *
 * @author Keith James
 */
class TemplateFactoryOwnerImpl implements TemplateFactory
{
    // Log4j setup
    static Category cat = Category.getInstance(TemplateFactoryOwnerImpl.class.getName());

    private JobControlServlet servlet;

    TemplateFactoryOwnerImpl(JobControlServlet servlet)
    {
        this.servlet = servlet;
    }

    public Template createTemplate(Context context) throws Exception
    {
        Template template = null;

        try
        {
            DataSource datasource = servlet.getDataSource();

            OwnerDAO ownerDAO = new OwnerSQLDAO(datasource);
            Collection owners = ownerDAO.readAllOwners();
            Table ownerTable = new Table();

            for (Iterator oi = owners.iterator(); oi.hasNext();)
            {
                Owner o = (Owner) oi.next();
                int id = o.getId();

                Table.Row row = ownerTable.new Row(5);

                Table.Cell cell = ownerTable.new Cell(Integer.toString(id));
                cell.setMetaData(ServletConstants.CSS_ALIGN_R);

                row.addCell(cell);
                row.addCell(ownerTable.new Cell(o.getFirstName()
                                                    + " "
                                                    + o.getLastName()));
                row.addCell(ownerTable.new Cell(o.getUserName()));
                row.addCell(ownerTable.new Cell(o.getEMail()));
                row.addCell(ownerTable.new Cell(o.getTelephone()));

                if (id % 2 == 0)
                    row.setMetaData(ServletConstants.CSS_DARK_BG);
                else
                    row.setMetaData(ServletConstants.CSS_PALE_BG);

                ownerTable.addRow(row);
            }

            cat.debug("Result table is:\n" + ownerTable);

            context.put("ownerTable", ownerTable);
            template = servlet.getTemplate("owners.vm");

            Object [] keys = context.getKeys();
            for (int i = 0; i < keys.length; i++)
                cat.debug("Factory: key: "
                          + keys[i]
                          + " value: "
                          + context.get((String) keys[i]));

        }
        catch (ResourceNotFoundException rnfe)
        {
            System.err.println("TemplateFactory: template not found");
            throw rnfe;
        }
        catch (ParseErrorException pee)
        {
            System.err.println("TemplateFactory: parse error for template");
            throw pee;
        }

        return template;
    }
}
