/*
 * Copyright (c) 2002 Genome Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.jcon.util;

import java.io.FileOutputStream;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.sql.DataSource;

import org.dbunit.database.DatabaseDataSourceConnection;
import org.dbunit.database.IDatabaseConnection;
import org.dbunit.dataset.IDataSet;
import org.dbunit.dataset.xml.FlatXmlDataSet;
import org.dbunit.dataset.xml.FlatDtdDataSet;

/**
 * <code>DBXMLExporter</code> exports DBUnit "flat XML" datasets from
 * the job control database. Given a base file name "&lt;filename&gt;"
 * it creates "&lt;filename&gt;.xml" and "&lt;filename&gt;.dtd".
 *
 * @author Keith James
 */
public class DBXMLExporter
{
    public static void main(String [] argv) throws Exception
    {
        if (argv.length != 1)
        {
            System.err.println("Usage: java DBXMLExporter <filename>");
            System.exit(0);
        }

        Context context = new InitialContext();
        DataSource datasource = (DataSource) context.lookup("jdbc/TestJobControl");

        IDatabaseConnection con = new DatabaseDataSourceConnection(datasource);
        
        String [] tableNames = { "owner", "executable", "host", "task", "job",
                                 "job_host", "job_history" };

        IDataSet dataSet = con.createDataSet(tableNames);
        FlatXmlDataSet.write(dataSet,
                             new FileOutputStream(argv[0] + ".xml"));
        FlatDtdDataSet.write(dataSet,
                             new FileOutputStream(argv[0] + ".dtd"));
    }
}
