/*
 * Copyright (c) 2002 Genome Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.jcon.util;

import java.util.Enumeration;
import java.util.Stack;

import uk.ac.sanger.jcon.job.Job;

/**
 * <code>PreorderEnumeration</code> implements preorder traversal of a
 * <code>Job</code> tree. Uses the <code>Enumeration</code> interface
 * instead of <code>Iterator</code> because the <code>TreeNode</code>
 * interface specifies the former in some methods. Based on code from
 * <code>DefaultMutableTreeNode</code> from Sun JDK by Rob Davis.
 *
 * @author Keith James
 */
public final class PreorderEnumeration implements Enumeration
{
    private Stack stack;

    /**
     * Creates a new <code>PreorderEnumeration</code>.
     *
     * @param rootNode a <code>Job</code>.
     */
    public PreorderEnumeration(final Job rootNode)
    {
        stack = new Stack();
        stack.push(new Enumeration()
            {
                private boolean hasMore = true;

                public boolean hasMoreElements()
                {
                    return hasMore;
                }

                public Object nextElement()
                {
                    hasMore = false;
                    return rootNode;
                }
            });
    }

    public boolean hasMoreElements()
    {
        return (! stack.isEmpty() &&
                ((Enumeration) stack.peek()).hasMoreElements());
    }

    public Object nextElement()
    {
        Enumeration enum = (Enumeration) stack.peek();

        Job node = (Job) enum.nextElement();
        Enumeration children = node.children();

        if (! enum.hasMoreElements())
            stack.pop();

        if (children.hasMoreElements())
            stack.push(children);

        return node;
    }
}
