/*
 * Copyright (c) 2002 Genome Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.jcon.xml;

import java.util.Enumeration;

import uk.ac.sanger.jcon.job.Job;

/**
 * <code>ParentLinker</code> sets the parent reference of a
 * <code>Job</code>'s children. When unmarshalled from an XML source
 * this reference is null. Setting this prevents illegal addition of
 * child <code>Job</code>s e.g. additions which would create cycles.
 *
 * @author Keith James
 */
class ParentLinker
{
    /**
     * <code>linkParents</code> links children to their parent.
     *
     * @param job a <code>Job</code>.
     */
    void linkParents(Job job)
    {
        link(job, job.children());
    }

    /**
     * <code>link</code> is a recursive method to perform linking. The
     * <code>Job</code> trees are broad and shallow.
     *
     * @param parent a <code>Job</code>.
     * @param children an <code>Enumeration</code> of its children.
     */
    private void link(Job parent, Enumeration children)
    {
        while (children.hasMoreElements())
        {
            Job child = (Job) children.nextElement();
            child.setParent(parent);

            link(child, child.children());
        }
    }
}
