/*
 * Copyright (c) 2002 Genome Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.jcon.xml;

import java.sql.Timestamp;
import javax.xml.bind.MarshallableObject;

import uk.ac.sanger.jcon.job.Job;
import uk.ac.sanger.jcon.job.Owner;
import uk.ac.sanger.jcon.job.Status;
import uk.ac.sanger.jcon.job.Task;
import uk.ac.sanger.jcon.xml.jaxb.BatchJobDO;
import uk.ac.sanger.jcon.xml.jaxb.OwnerDO;
import uk.ac.sanger.jcon.xml.jaxb.SubmissionTime;
import uk.ac.sanger.jcon.xml.jaxb.TaskDO;

/**
 * <code>TaskMarshallableImpl</code> is implementation of
 * <code>Task</code> which may be (un)marshalled using JAXB.
 *
 * @author Keith James
 */
public class TaskMarshallableImpl extends TaskDO implements Task
{
    public Owner getOwner()
    {
        return (Owner) getOwnerDO();
    }

    public void setOwner(Owner owner)
    {
        if (owner instanceof OwnerDO)
            setOwnerDO((OwnerDO) owner);
        else
            throw new IllegalArgumentException("Potential marshalling error. Only JAXB-generated implementations may be used");
    }

    public Status getStatus()
    {
        return getJobs()[0].getStatus();
    }

    public Job [] getJobs()
    {
        MarshallableObject [] jobProperty = getJobProperty();

        Job [] jobs = new Job [jobProperty.length];
        System.arraycopy(jobProperty, 0, jobs, 0, jobProperty.length);
        return jobs;
    }

    public void setJobs(Job [] jobs)
    {
        if (jobs instanceof MarshallableObject [])
            setJobProperty((MarshallableObject []) jobs);
        else
            throw new IllegalArgumentException("Potential marshalling error. Only JAXB-generated implementations may be used");
    }

    public Timestamp getSubmissionTime()
    {
        SubmissionTime st = getSubmissionTimeDO();

        return (st != null) ? st.getContent() : null;
    }

    public void setSubmissionTime(Timestamp submissionTime)
    {
        SubmissionTime st = getSubmissionTimeDO();

        if (st == null)
            st = new SubmissionTime();

        st.setContent(submissionTime);
        setSubmissionTimeDO(st);
    }

    public String toString()
    {
        return "TaskMarshallableImpl: " + super.toString();
    }
}
