/*
 * Copyright (c) 2002 Genome Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.jcon.dao;

import java.util.List;

import junit.framework.TestCase;

import uk.ac.sanger.jcon.job.Job;
import uk.ac.sanger.jcon.job.JobDefaultImpl;

/**
 * <code>JobNestedSetHelperTest</code> tests creation of nested set
 * values from a tree of jobs.
 *
 * @author Keith James
 */
public class JobNestedSetHelperTest extends TestCase
{
    // Test set of jobs
    private JobDefaultImpl [] jobs = { new JobDefaultImpl(),
                                       new JobDefaultImpl(),
                                       new JobDefaultImpl(),
                                       new JobDefaultImpl(),
                                       new JobDefaultImpl(),
                                       new JobDefaultImpl(),
                                       new JobDefaultImpl(),
                                       new JobDefaultImpl(),
                                       new JobDefaultImpl(),
                                       new JobDefaultImpl(),
                                       new JobDefaultImpl(),
                                       new JobDefaultImpl() };

    // Expected values for the left/right integers for each node
    private int [][] expected = { new int [] {0,  23},
                                  new int [] {1,   2},
                                  new int [] {3,  14},
                                  new int [] {4,   5},
                                  new int [] {6,  13},
                                  new int [] {7,   8},
                                  new int [] {9,  10},
                                  new int [] {11, 12},
                                  new int [] {15, 22},
                                  new int [] {16, 21},
                                  new int [] {17, 18},
                                  new int [] {19, 20} };

    /**
     * Creates a new <code>JobNestedSetHelperTest</code>.
     *
     * @param name a <code>String</code>.
     */
    public JobNestedSetHelperTest(String name)
    {
        super(name);
    }

    /**
     * <code>testNestedSetNumbering</code> tests creation of nested
     * set values for a tree.
     *
     * @exception Exception if an error occurs.
     */
    public void testNestedSetNumbering()
    {
        jobs[0].add(jobs[1]);
        jobs[0].add(jobs[2]);
        jobs[0].add(jobs[3]);

        jobs[2].add(jobs[4]);
        jobs[2].add(jobs[5]);

        jobs[3].add(jobs[6]);

        jobs[5].add(jobs[7]);
        jobs[5].add(jobs[8]);
        jobs[5].add(jobs[9]);

        jobs[6].add(jobs[10]);
        jobs[6].add(jobs[11]);

        JobNestedSetHelper setHelper = new JobNestedSetHelper();
        List preorder = setHelper.calculateNestedSets(jobs[0], 0);

        for (int i = 0; i < preorder.size(); i++)
        {
            Job j = (Job) preorder.get(i);
            assertEquals(expected[i][0],  j.getLow());
            assertEquals(expected[i][1],  j.getHigh());
        }
    }
}
