/*
 * Copyright (c) 2002 Genome Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.jcon.dao;

import java.io.File;
import java.util.Enumeration;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.sql.DataSource;

import org.dbunit.DatabaseTestCase;
import org.dbunit.dataset.IDataSet;
import org.dbunit.operation.DatabaseOperation;
import org.dbunit.dataset.xml.FlatXmlDataSet;

import uk.ac.sanger.jcon.JobControlSQLCase;
import uk.ac.sanger.jcon.dao.*;
import uk.ac.sanger.jcon.job.*;

public class PerformanceTest extends JobControlSQLCase
{
    public PerformanceTest(String name) throws Exception
    {
        super(name);
    }

    public void testCreateTasks() throws Exception
    {
        OwnerSQLDAO ownerSQLDAO = new OwnerSQLDAO(datasource);
        Owner owner = ownerSQLDAO.readOwnerById(1);

        ExecutableSQLDAO executableSQLDAO = new ExecutableSQLDAO(datasource);
        Executable executable = executableSQLDAO.readExecutableById(1);

        StatusSQLDAO statusSQLDAO = new StatusSQLDAO(datasource);
        Status status = statusSQLDAO.readStatusById(1);

        TaskSQLDAO taskSQLDAO = new TaskSQLDAO(datasource);

        int iterations = 2;
        int  taskCount = 10;
        int jobL1Count = 100;
        int jobL2Count = 5;
        int jobL3Count = 5;

        for (int iter = 0; iter < iterations; iter++)
        {
            long start = System.currentTimeMillis();

            int totalJobCount = 0;

            for (int i = 0; i < taskCount; i++)
            {
                Task task = new TaskDefaultImpl("Test task " + i,
                                                "A task object for testing",
                                                owner);

                JobDefaultImpl job0 = new JobDefaultImpl();
                totalJobCount++;
                job0.setExecutable(executable);
                job0.setCommandTemplate("$executable");
                job0.setStatus(status);
                task.setJobs(new Job [] { job0 });

                for (int j = 0; j < jobL1Count; j++)
                {
                    JobDefaultImpl job1 = new JobDefaultImpl();
                    totalJobCount++;
                    job1.setExecutable(executable);
                    job1.setCommandTemplate("$executable");
                    job1.setStatus(status);
                    job0.add(job1);

                    for (int k = 0; k < jobL2Count; k++)
                    {
                        JobDefaultImpl job2 = new JobDefaultImpl();
                        totalJobCount++;
                        job2.setExecutable(executable);
                        job2.setCommandTemplate("$executable");
                        job2.setStatus(status);
                        job1.add(job2);

                        for (int m = 0; m < jobL3Count; m++)
                        {
                            JobDefaultImpl job3 = new JobDefaultImpl();
                            totalJobCount++;
                            job3.setExecutable(executable);
                            job3.setCommandTemplate("$executable");
                            job3.setStatus(status);
                            job2.add(job3);
                        }
                    }
                }

                taskSQLDAO.createTask(task);
            }

            long stop = System.currentTimeMillis();
            System.out.println("Total of "
                               + totalJobCount
                               + " jobs in "
                               + (stop - start) + " msec.");
        }
    }

    public void testUpdateJobs() throws Exception
    {
        OwnerSQLDAO ownerSQLDAO = new OwnerSQLDAO(datasource);
        Owner owner = ownerSQLDAO.readOwnerById(1);

        ExecutableSQLDAO executableSQLDAO = new ExecutableSQLDAO(datasource);
        Executable executable = executableSQLDAO.readExecutableById(1);

        StatusSQLDAO statusSQLDAO = new StatusSQLDAO(datasource);
        Status status = statusSQLDAO.readStatusById(1);

        TaskSQLDAO taskSQLDAO = new TaskSQLDAO(datasource);

        JobSQLDAO jobSQLDAO = new JobSQLDAO(datasource);

        int jobL1Count = 1;
        int jobL2Count = 10;
        int jobL3Count = 5;

        Task task = new TaskDefaultImpl("Test task",
                                        "A task object for testing",
                                        owner);

        JobDefaultImpl job0 = new JobDefaultImpl();
        job0.setExecutable(executable);
        job0.setCommandTemplate("$executable");
        job0.setStatus(status);
        task.setJobs(new Job [] { job0 });

        for (int j = 0; j < jobL1Count; j++)
        {
            JobDefaultImpl job1 = new JobDefaultImpl();
            job1.setExecutable(executable);
            job1.setCommandTemplate("$executable");
            job1.setStatus(status);
            job0.add(job1);

            for (int k = 0; k < jobL2Count; k++)
            {
                JobDefaultImpl job2 = new JobDefaultImpl();
                job2.setExecutable(executable);
                job2.setCommandTemplate("$executable");
                job2.setStatus(status);
                job1.add(job2);

                for (int m = 0; m < jobL3Count; m++)
                {
                    JobDefaultImpl job3 = new JobDefaultImpl();
                    job3.setExecutable(executable);
                    job3.setCommandTemplate("$executable");
                    job3.setStatus(status);
                    job2.add(job3);
                }
            }
        }

        int taskId = taskSQLDAO.createTask(task);

        taskSQLDAO.updateTaskSubmitted(taskId);
        task = taskSQLDAO.readTaskById(taskId, TaskDAO.INCLUDE_JOB_TREE);

        Job [] rootJobs = task.getJobs();

        long start = System.currentTimeMillis();
        int totalJobCount = 0;

        for (Enumeration ce = rootJobs[0].children(); ce.hasMoreElements();)
        {
            Job child = (Job) ce.nextElement();

            for (Enumeration gce = child.children(); gce.hasMoreElements();)
            {
                Job gchild = (Job) gce.nextElement();

                for (Enumeration ggce = gchild.children(); ggce.hasMoreElements();)
                {
                    Job ggchild = (Job) ggce.nextElement();

                    jobSQLDAO.updateJobStarted(ggchild);
                    totalJobCount++;
                }
            }
        }

        long stop = System.currentTimeMillis();
        System.out.println("Total of "
                           + totalJobCount
                           + " jobs started in "
                           + (stop - start) + " msec.");

        task = taskSQLDAO.readTaskById(taskId, TaskDAO.INCLUDE_JOB_TREE);
        rootJobs = task.getJobs();

        for (Enumeration ce = rootJobs[0].children(); ce.hasMoreElements();)
        {
            Job child = (Job) ce.nextElement();

            for (Enumeration gce = child.children(); gce.hasMoreElements();)
            {
                Job gchild = (Job) gce.nextElement();

                for (Enumeration ggce = gchild.children(); ggce.hasMoreElements();)
                {
                    Job ggchild = (Job) ggce.nextElement();

                    jobSQLDAO.updateJobCompleted(ggchild);
                }
            }
        }

        stop = System.currentTimeMillis();
        System.out.println("Total of "
                           + totalJobCount
                           + " jobs completed in "
                           + (stop - start) + " msec.");
    }

    protected IDataSet getDataSet() throws Exception
    {
        String fileName = getClass().getResource("/performance_data_oracle.xml").getFile();

        return new FlatXmlDataSet(new File(fileName));
    }
}
