/*
 * Copyright (c) 2002 Genome Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.jcon.dao;

import java.util.Enumeration;

import uk.ac.sanger.jcon.JobControlSQLCase;
import uk.ac.sanger.jcon.job.*;

/**
 * <code>TaskSQLDAOTest</code> tests create, read, update and delete
 * of <code>Task</code>s.
 *
 * @author Keith James
 */
public class TaskSQLDAOTest extends JobControlSQLCase
{
    private TaskSQLDAO taskSQLDAO;

    // Test set of jobs
    private JobDefaultImpl [] jobs = { new JobDefaultImpl(),
                                       new JobDefaultImpl(),
                                       new JobDefaultImpl(),
                                       new JobDefaultImpl(),
                                       new JobDefaultImpl(),
                                       new JobDefaultImpl(),
                                       new JobDefaultImpl(),
                                       new JobDefaultImpl(),
                                       new JobDefaultImpl(),
                                       new JobDefaultImpl(),
                                       new JobDefaultImpl() };

    public TaskSQLDAOTest(String name) throws Exception
    {
        super(name);
    }

    public void testCreateTask() throws Exception
    {
        // First get an Owner
        OwnerSQLDAO ownerSQLDAO = new OwnerSQLDAO(datasource);
        Owner owner = ownerSQLDAO.readOwnerById(1);

        // Get an Executable
        ExecutableSQLDAO executableSQLDAO = new ExecutableSQLDAO(datasource);
        Executable executable = executableSQLDAO.readExecutableById(1);

        // Get a Status
        StatusSQLDAO statusSQLDAO = new StatusSQLDAO(datasource);
        Status status = statusSQLDAO.readStatusById(1);

        Task task = new TaskDefaultImpl("New test task", "A task object for testing", owner);

        jobs[0].add(jobs[1]);
        jobs[0].add(jobs[2]);
        jobs[0].add(jobs[3]);

        jobs[2].add(jobs[4]);
        jobs[2].add(jobs[5]);

        jobs[3].add(jobs[6]);

        jobs[5].add(jobs[7]);
        jobs[5].add(jobs[8]);
        jobs[5].add(jobs[9]);

        jobs[6].add(jobs[10]);

        for (int i = 0; i < jobs.length; i++)
        {
            jobs[i].setExecutable(executable);
            jobs[i].setCommandTemplate("$executable");
            jobs[i].setStatus(status);
        }

        task.setJobs(new Job [] { jobs[0] });

        taskSQLDAO = new TaskSQLDAO(datasource);

        long start = System.currentTimeMillis();
        int taskId  = taskSQLDAO.createTask(task);
        long stop = System.currentTimeMillis();
        System.out.println("testCreateTask " + (stop - start) + " msec");

        Task newTask = taskSQLDAO.readTaskById(taskId,
                                               TaskDAO.INCLUDE_JOB_TREE);
        assertEquals("New test task", newTask.getName());

        Job [] rootJobs = newTask.getJobs();
        Job rootJob = rootJobs[0];

        assertNull(rootJob.getParent());
        assertEquals(6, countLeaves(rootJob));
    }

    public void testReadTaskById() throws Exception
    {
        taskSQLDAO = new TaskSQLDAO(datasource);

        long start = System.currentTimeMillis();
        Task task = taskSQLDAO.readTaskById(1, TaskDAO.INCLUDE_JOB_TREE);
        long stop = System.currentTimeMillis();
        System.out.println("testReadTaskById " + (stop - start) + " msec");

        assertEquals("TestTask1", task.getName());
        assertEquals("A test task", task.getDescription());
        assertEquals(1, task.getOwner().getId());
        assertEquals(4, task.getPriority());

        Job [] rootJobs = task.getJobs();
        Job rootJob = rootJobs[0];

        assertNull(rootJob.getParent());
        assertEquals(7, countLeaves(rootJob));
    }

    private int countLeaves(Job rootJob)
    {
        int leafCount = 0;

        Enumeration enum =  rootJob.preorderEnumeration();
        while (enum.hasMoreElements())
        {
            Job node = (Job) enum.nextElement();
            if (node.isLeaf())
                leafCount++;
        }

        return leafCount;
    }
}
