/*
 * Copyright (c) 2002 Genome Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.jcon.run;

import junit.framework.TestCase;

import uk.ac.sanger.jcon.job.Job;
import uk.ac.sanger.jcon.job.JobDefaultImpl;
import uk.ac.sanger.jcon.job.Executable;
import uk.ac.sanger.jcon.job.ExecutableDefaultImpl;

/**
 * <code>CommandLineFactoryVelocityImplTest</code> tests creation of
 * command lines using Velocity.
 *
 * @author Keith James
 */
public class CommandLineFactoryVelocityImplTest extends TestCase
{
    public CommandLineFactoryVelocityImplTest(String name)
    {
        super(name);
    }

    public void testCreateCommandLine() throws Exception
    {
        CommandLineFactory vFactory = CommandLineFactoryVelocityImpl.getInstance();

        Executable executable = new ExecutableDefaultImpl("fasta",
                                                          "3.3t08",
                                                          "Fasta sequence search");
        String database = "database.dna";

        String commandTemplate =
            "${executable} ${input_name} "
            + database
            + " > ${output_name} 2> ${error_name}";

        Job job = new JobDefaultImpl(executable);
        job.setInputName("seq.dna");
        job.setOutputName("fasta.out");
        job.setErrorName("fasta.err");
        job.setCommandTemplate(commandTemplate);

        String command = vFactory.createCommandLine(job);

        assertEquals("fasta seq.dna database.dna > fasta.out 2> fasta.err", command);
    }
}
