/*
 * Copyright (c) 2002 Genome Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.jcon.run;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;

import uk.ac.sanger.jcon.JobControlSQLCase;
import uk.ac.sanger.jcon.dao.StatusDAO;
import uk.ac.sanger.jcon.dao.StatusSQLDAO;
import uk.ac.sanger.jcon.job.Executable;
import uk.ac.sanger.jcon.job.ExecutableDefaultImpl;
import uk.ac.sanger.jcon.job.Job;
import uk.ac.sanger.jcon.job.JobDefaultImpl;

/**
 * <code>ProcessorTest</code> tests default job processor.
 *
 * @author Keith James
 */
public class ProcessorDefaultImplTest extends JobControlSQLCase
{
    public ProcessorDefaultImplTest(String name) throws Exception
    {
        super(name);
    }

    public void testStartJob() throws Exception
    {
        Executable executable = new ExecutableDefaultImpl("/bin/cat", "", "");
        Job job = new JobDefaultImpl(executable);

        File input = new File(getClass().getResource("TEST.INPUT").getFile());
        job.setInputName(input.getName());
        job.setOutputName("TEST.OUTPUT");
        job.setErrorName("TEST.ERROR");
        job.setCommandTemplate("$executable");
        job.setWorkDirectoryName(input.getParent());

        CommandLineFactory clf = CommandLineFactoryVelocityImpl.getInstance();
        StatusDAO statusDAO = new StatusSQLDAO(datasource);

        Processor processor = new ProcessorDefaultImpl(clf, statusDAO);
        processor.process(job);

        synchronized(this)
        {
            // Should be enough time to run cat
            wait(10000);
        }

        File output = new File(getClass().getResource("TEST.OUTPUT").getFile());
        BufferedReader br = new BufferedReader(new FileReader(input));
        String result = br.readLine();
        assertEquals("A test string", result);
    }
}
