/*
 * Copyright (c) 2002 Genome Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.jcon.run;

import uk.ac.sanger.jcon.JobControl;
import uk.ac.sanger.jcon.JobControlSQLCase;
import uk.ac.sanger.jcon.dao.TaskDAO;
import uk.ac.sanger.jcon.dao.TaskSQLDAO;
import uk.ac.sanger.jcon.job.Job;
import uk.ac.sanger.jcon.job.Status;
import uk.ac.sanger.jcon.job.Task;
import uk.ac.sanger.jcon.run.Coordinator;
import uk.ac.sanger.jcon.run.CoordinatorDefaultImpl;
import uk.ac.sanger.jcon.run.TaskScheduler;
import uk.ac.sanger.jcon.run.TaskSchedulerDefaultImpl;

/**
 * <code>TaskSchedulerRegressionTest</code> runs a simple two-root-job
 * <code>Task</code> to completion using the default scheduler and
 * coordinator.
 *
 * @author Keith James
 */
public class TaskSchedulerRegressionTest extends JobControlSQLCase
{
    public TaskSchedulerRegressionTest(String name) throws Exception
    {
        super(name);
    }

    public void testTaskScheduler() throws Exception
    {
        // Remember to set datasource context to 'jdbc/TestJobControl'
        // or this test will fail
        JobControl jobControl = new JobControl();

        Coordinator coordinator =
            new CoordinatorDefaultImpl(ProcessorDefaultImpl.FACTORY,
                                       jobControl.createJobDAO(),
                                       jobControl.createStatusDAO());

        TaskScheduler scheduler =
            new TaskSchedulerDefaultImpl(jobControl.createTaskDAO(),
                                         coordinator);

        scheduler.setSendMail(false);
        scheduler.setPolicy(new SingleTaskPolicy(4));

        scheduler.startScheduling();
        coordinator.startCoordinating();

        synchronized(this)
        {
            // Should be long enough; coordinator poll interval should
            // be 10000 ms in JobControl config
            wait(240000);
        }

        scheduler.stopScheduling();
        coordinator.stopCoordinating();

        TaskDAO taskDAO = new TaskSQLDAO(datasource);
        Task task = taskDAO.readTaskById(4, TaskDAO.EXCLUDE_JOB_TREE);
        Job [] jobs = task.getJobs();
        assertEquals(Status.COMPLETED, jobs[0].getStatus().getId());
        assertEquals(Status.COMPLETED, jobs[1].getStatus().getId());
    }
}
