/*
 * Copyright (c) 2002 Genome Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.jcon.util;

import java.util.ResourceBundle;
import java.util.Set;

import junit.framework.TestCase;

public class ConfigurationTest extends TestCase
{
	protected static ResourceBundle resource =
		ResourceBundle.getBundle(ConfigurationTest.class.getName());

	protected Configuration config;

	public ConfigurationTest(String name)
    {
        super(name);
    }

	protected void setUp()
	{
		config = new Configuration(resource);
	}

	public void testGetExecutableAliases()
	{
		Set execAliases = config.getExecutableAliases();

		assertEquals(3, execAliases.size());
		assertTrue(execAliases.contains("genefinder"));
		assertTrue(execAliases.contains("geneid"));
		assertTrue(execAliases.contains("hmmgene"));
	}

	public void testGetOrganismAliases()
	{
		Set organismAliases = config.getOrganismAliases();

		assertEquals(1, organismAliases.size());
		assertTrue(organismAliases.contains("d_discoideum"));
	}

	public void testGetExecutableId()
	{
		assertEquals(22, config.getExecutableId("genefinder"));
		assertEquals(23, config.getExecutableId("geneid"));
		assertEquals(25, config.getExecutableId("hmmgene"));
	}

	public void testGetTaxonomyId()
	{
		assertEquals(44689, config.getTaxonomyId("d_discoideum"));
	}

	public void testGetTemplate()
	{
		assertEquals("$executable -maxSeg -minintronlength 32 -minexonlength 3 -maxintronlength 400 -tablenamefile /nfs/disk222/yeastpub/databases/dicty/genefinder_tables/fourth_set/dicty_table_files -seqfile $input_name",
					 config.getTemplate("d_discoideum", "genefinder"));

		assertEquals("$executable -G -P /nfs/disk222/yeastpub/databases/dicty/geneid_table/dicty.param $input_name",
					 config.getTemplate("d_discoideum", "geneid"));

		assertEquals("$executable -L /nfs/pathsoft/external/bio-soft/HMMgene/hmmgene_dicty_OSF/lib -F DICTY_08",
					 config.getTemplate("d_discoideum", "hmmgene"));
	}
}
