/*
 * Copyright (c) 2003 Genome Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.jcon.util;

import java.util.List;
import java.util.ResourceBundle;

import junit.framework.TestCase;

import uk.ac.sanger.jcon.JobControlException;

public class UserConfigurationTest extends TestCase
{
    protected static ResourceBundle resource =
		ResourceBundle.getBundle(UserConfigurationTest.class.getName());

	protected UserConfiguration config;

	public UserConfigurationTest(String name)
    {
        super(name);
    }

	protected void setUp() throws Exception
	{
		config = new UserConfiguration(resource);
	}

    public void testGetProtocolName() throws JobControlException
    {
        assertEquals("example_protocol", config.getProtocolName());
    }

    public void testGetProtocolDescription() throws JobControlException
    {
        assertEquals("An example protocol", config.getProtocolDescription());
    }

    public void testGetJobPrecedenceList()
    {
        List precedence = config.getJobPrecedenceList();

        assertEquals(0, ((Integer) precedence.get(0)).intValue());
        assertEquals(1, ((Integer) precedence.get(1)).intValue());
        assertEquals(2, ((Integer) precedence.get(2)).intValue());
        assertEquals(3, ((Integer) precedence.get(3)).intValue());
    }

    public void testGetExecutableId()
    {
        assertEquals(189, config.getExecutableId(0));
        assertEquals(356, config.getExecutableId(1));
        assertEquals(456, config.getExecutableId(2));
        assertEquals(78, config.getExecutableId(3));
    }

    public void testGetMaxRetryCount()
    {
        assertEquals(0, config.getMaxRetryCount(0));
        assertEquals(1, config.getMaxRetryCount(1));
        assertEquals(2, config.getMaxRetryCount(2));
        assertEquals(3, config.getMaxRetryCount(3));
    }

    public void testGetTemplate()
    {
        assertEquals("$executable -e 4.5 -dp", config.getTemplate(0));
        assertEquals("$executable", config.getTemplate(1));
        assertEquals("$executable -f 12 -c -2", config.getTemplate(2));
        assertEquals("$executable --filter 10 --match 5", config.getTemplate(3));
    }
}
