/*
 * Copyright (c) 2002 Genome Research Limited.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published 
 * by  the Free Software Foundation; either version 2 of the License or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.LIB.  If not, write to 
 * the Free Software Foundation Inc., 59 Temple Place - Suite 330, 
 * Boston, MA  02111-1307 USA
 */

package uk.ac.sanger.jcon.job;

import java.util.ArrayList;
import java.util.List;
import java.sql.Timestamp;

import junit.framework.TestCase;

import uk.ac.sanger.jcon.job.Task;
import uk.ac.sanger.jcon.xml.*;

/**
 * <code>TaskMarshallerTest</code> tests conversion of task data to
 * and from XML defined by the TaskCollection DTD.
 *
 * @author Keith James
 */
public class TaskMarshallerTest extends TestCase
{
    protected Task task;

    public TaskMarshallerTest(String name)
    {
        super(name);
    }

    protected void setUp()
    {
        task = new TaskMarshallableImpl();
        task.setName("Test Task");
        task.setDescription("A test Task");
        task.setPriority(4);
        task.setSubmissionTime(Timestamp.valueOf("2002-09-19 00:38:23.698"));

        OwnerMarshallableImpl owner = new OwnerMarshallableImpl();
        owner.setId(1);
        owner.setUserName("anonymous");

        ExecutableMarshallableImpl exec1 = new ExecutableMarshallableImpl();
        exec1.setId(0);
        exec1.setName("true");
        ExecutableMarshallableImpl exec2 = new ExecutableMarshallableImpl();
        exec2.setId(1);
        exec2.setName("blastp");
        ExecutableMarshallableImpl exec3 = new ExecutableMarshallableImpl();
        exec3.setId(1);
        exec3.setName("blastp");
        ExecutableMarshallableImpl exec4 = new ExecutableMarshallableImpl();
        exec4.setId(1);
        exec4.setName("blastp");

        BatchJobMarshallableImpl rootJob = new BatchJobMarshallableImpl();
        rootJob.setId(0);
        rootJob.setOperator(Job.OPERATOR_AND);
        rootJob.setExecutable(exec1);
        rootJob.setCommandTemplate("$executable");
        rootJob.setMaxRetryCount(0);
        rootJob.setQueue("queue1");

        BatchJobMarshallableImpl job1 = new BatchJobMarshallableImpl();
        job1.setId(1);
        job1.setOperator(Job.OPERATOR_AND);
        job1.setExecutable(exec2);
        job1.setCommandTemplate("$executable -d swall");
        job1.setInputName("sequence1.aa");
        job1.setOutputName("sequence1.out");
        job1.setErrorName("sequence1.err");
        job1.setMaxRetryCount(3);
        job1.setQueue("queue1");

        BatchJobMarshallableImpl job2 = new BatchJobMarshallableImpl();
        job2.setId(2);
        job2.setOperator(Job.OPERATOR_AND);
        job2.setExecutable(exec3);
        job2.setCommandTemplate("$executable -d swall");
        job2.setInputName("sequence2.aa");
        job2.setOutputName("sequence2.out");
        job2.setErrorName("sequence2.err");
        job2.setMaxRetryCount(3);
        job2.setQueue("queue1");

        BatchJobMarshallableImpl job3 = new BatchJobMarshallableImpl();
        job3.setId(3);
        job3.setOperator(Job.OPERATOR_AND);
        job3.setExecutable(exec4);
        job3.setCommandTemplate("$executable -d swall");
        job3.setInputName("sequence3.aa");
        job3.setOutputName("sequence3.out");
        job3.setErrorName("sequence3.err");
        job3.setMaxRetryCount(3);
        job3.setQueue("queue1");

        rootJob.add(job1);
        rootJob.add(job3);
        job1.add(job2);

        task.setOwner(owner);
        task.setJobs(new BatchJobMarshallableImpl [] { rootJob });
    }

    public void testMarshalTask() throws Exception
    {
        TaskMarshaller tm = new TaskMarshaller();
        List tasks = new ArrayList(1);
        tasks.add(task);
        tm.marshal(tasks, System.out);
    }

    public void testUnmarshalTask() throws Exception
    {
        TaskMarshaller tm = new TaskMarshaller();
        List tasks = tm.unmarshal(getClass().getResourceAsStream("/taskcollection.xml"));

        Task unmarshalledTask = (Task) tasks.get(0);

        assertEquals(task, unmarshalledTask);
    }
}
