#!/usr/local/bin/perl -w

use strict;

#


my $dir = "/nfs/disk222/yeastpub/databases/pombe_funct_class"; #config_files
#my $htmldir = "/nfs/disk29/jws/working/pombeoutput";
my $htmldir = "/nfs/WWW/htdocs/Projects/S_pombe/FUNCAT";
my $curator1 = "Valerie Wood";
my $curator1_email = "val";
my $curator2 = "Mike Lyne";
my $curator2_email = "mhl";
my $curator3 = "Rachel McDougall";
my $curator3_email = "rcm";
#my $http = "http://www.sanger.ac.uk/Users/val/"; NOT NEEDED
my $http=""; 
my $contig_dir = "/nfs/disk222/yeastpub/analysis/yeast/pombe/CONTIGS";

if ($#ARGV==-1){
print "\nusage: progname <inputfilename>\n";
print "input file = file of fasta format description lines\n\n";
exit;
}




###############################################################################
#                                                                             #
#                         INITIALISE THE MAIN HASH                            #
#                                                                             #
###############################################################################
#   THE HASH KEYS ARE THE FUNCTIONAL CATEGORIES
#   EACH KEY CONTAINS AN ARRAY OF DATA FOR EACH CATEGORY
#
#   [0] configuration file names 1 per catagory
#   These files contain a set of perl regular expressions for this catagory
#   It can be general descriptions eg CELL CYCLE /RIBOSOMAL
#   Gene Classes eg CDC
#   ..or individual gene names or systematic codes
#   (this variabl is the same as the hash key)
#
#   [1] The name of the curator to appear on the web page (see variable above)
#
#   [2] email of the curator (see variable above)
#
#   [3] Title to appear on the webpage title bar and header
#
#   [4] Class qualifier
#

my %all_data = 

#METABOLISM
  ( "aa_met", ["aa_met", "$curator1", "$curator1_email", "Amino Acid Metabolism",  "01.01.--"],
    "nit_sul_met", ["nit_sul_met", "$curator1", "$curator1_email", "Nitrogen and Sulphur Metabolism", "01.02.--"],
    "nuc_met", ["nuc_met", "$curator1", "$curator1_email", "Nucleotide Metabolism", "01.03.--"],
    "phosphate_met", ["phosphate_met", "$curator1", "$curator1_email", "Phosphate Metabolism", "01.04.--"],
    "carbo_met", ["carbo_met", "$curator1", "$curator1_email", "Carbohydrate Metabolism", "01.05.--"],
    "lipid_met", ["lipid_met", "$curator1", "$curator1_email", "Lipid Metabolism", "01.06.--"],
    "vit_co_met", ["vit_co_met", "$curator1", "$curator1_email", "Vitamin and Cofactor Metabolism", "01.07.--"],
    "amine_met", ["amine_met", "$curator1", "$curator1_email", "Amine Metabolism", "01.20.05"],
    "ox_ac_meth", ["ox_ac_meth", "$curator1", "$curator1_email", "Miscellaneous reductase, acytyl & methytransferase", "01.--.--"],
#ENERGY GENERATION
    "glycolysis", ["glycolysis", "$curator1", "$curator1_email", "Glycolysis", "02.01.--"],
    "citric_ac_cy", ["citric_ac_cy", "$curator1", "$curator1_email", "Citric Acid Cycle", "02.10.--"],
    "respiration", ["respiration", "$curator1", "$curator1_email", "Respiration", "02.13.--"],
    "fermentation", ["fermentation", "$curator1", "$curator1_email", "Fermentation", "01.16.--"],
    "other_en_gen", ["other_en_gen", "$curator1", "$curator1_email", "Other Energy Generation Activities", "02.99.--"],
#CELL GROWTH, CELL DIVISION, DNA SYNTHESIS
    "cell_gr", ["cell_gr", "$curator1", "$curator1_email", "Cell Growth", "03.0-.--"],
    "phero", ["phero", "$curator2", "$curator2_email", "Pheromone Response", "03.07.--"],
    "sporulation", ["sporulation", "$curator2", "$curator2_email", "Sporulation", "03.10.--"],
    "meiosis", ["meiosis", "$curator2", "$curator2_email", "Nitrogen and Sulphur Metabolism", "03.13.--"],
    "dna_syn", ["dna_syn", "$curator1", "$curator1_email", "DNA synthesis Recombination and Repair", "03.1-.--"],
    "cell_cyc", ["cell_cyc", "$curator1", "$curator1_email", "Cell Cycle", "03.22.--"],
    "cytokinesis", ["cytokinesis", "$curator1", "$curator1_email", "Cytokinesis", "03.25.--"],
    "other_cell_cyc", ["other_cell_cyc", "$curator1", "$curator1_email", "Other Cell Cycle activities", "03.99.--"],
    "prot_kin", ["prot_kin", "$curator1", "$curator1_email", "Protein Kinases", "03.PF.04"],
      "prot_phos", ["prot_phos", "$curator1", "$curator1_email", "Protein Phosphatase, and associated", "03.PF.05"],
    
#TRANSCRIPTION
    "rna_syn", ["rna_syn", "$curator1", "$curator1_email", "RNA Synthesis", "04.0-.-1"],
    "rna_proc", ["rna_proc", "$curator3", "$curator3_email", "RNA Processing", "04.0-.0-"],
    "trna_mod", ["trna_mod", "$curator1", "$curator1_email", "tRNA Modification", "04.03.06"],
    "trans_cont", ["trans_cont", "$curator1", "$curator1_email", "Transcriptional Control", "04.02.--"],
    "rna_trans", ["rna_trans", "$curator1", "$curator1_email", "RNA transport", "04.07"],
    "chromatin_mod", ["chromatin_mod", "$curator1", "$curator1_email", "Chromatin Modification", "04.05.01.07"],
    "other_transc", ["other_transc", "$curator1", "$curator1_email", "Other Transcription Activities", "04.99"],
#PROTEIN SYNTHESIS
    "ribosomal", ["ribosomal", "$curator1", "$curator1_email", "Ribosomal Proteins", "05.01"],
    "translation", ["translation", "$curator1", "$curator1_email", "Translation", "05.04"],

    "trna_syn", ["trna_syn", "$curator1", "$curator1_email", "tRNA synthetases", "05.10"],
#PROTEIN DESTINATION MODIFICATION ETC
    "prot_mod", ["prot_mod", "$curator1", "$curator1_email", "Protein Modification", "06.07"],
    "prot_fold", ["prot_fold", "$curator1", "$curator1_email", "Protein Folding and Stabilisation", "06.01"],
    "prot_targ", ["prot_targ", "$curator1", "$curator1_email", "tRNA Targeting", "06.04"],
    "proteolysis", ["proteolysis", "$curator1", "$curator1_email", "Proteolysis", "06.13"],
    "prot_complex_ass", ["prot_complex_ass", "$curator1", "$curator1_email", "Assembly of Protein Complexes", "06.10"],
#TRANSPORT FACILITATION
    "tr_atpases", ["tr_atpases", "$curator1", "$curator1_email", "Transport ATPases", "07.22"],
    "tr_abc", ["tr_abc", "$curator1", "$curator1_email", "ABC Transporters", "07.25"],
    "tr_aa", ["tr_aa", "$curator1", "$curator1_email", "Amino Acid Transporters", "07.10"],
    "tr_met_ion", ["tr_met_ion", "$curator1", "$curator1_email", "Metal Ion Transporters", "07.04.01"],
    "tr_cat_ion", ["tr_cat_ion", "$curator1", "$curator1_email", "Cation Transporters", "07.04.04"],
    "tr_an_ion", ["tr_an_ion", "$curator1", "$curator1_email", "Anion Transporters", "07.04.07"],
    "tr_carbo", ["tr_carbo", "$curator1", "$curator1_email", "Carbohydrate Transporters", "07.07"],
    "tr_drug", ["tr_drug", "$curator1", "$curator1_email", "Drug Transporters", "07.28"],
    "tr_allantoate", ["tr_allantoate", "$curator1", "$curator1_email", "Allantoate Transporters", "07.19"],
    "tr_nucleotide", ["tr_nucleotide", "$curator1", "$curator1_email", "Nucleotide Transporters", "07.16"],
    "tr_misc", ["tr_misc", "$curator1", "$curator1_email", "Miscellaneous transporters", "07.99"],
#INTRACELLULAR EXTRACELLULAR TRANSPORT
    "other_trans", ["other_trans", "$curator1", "$curator1_email", "Intracellular/Extracellular Transport", "08.--"],
    "cell_org", ["cell_org", "$curator1", "$curator1_email", "Cellular Organisation/Biognesis", "09.--"],
    "key_kinase", ["key_kinase", "$curator1", "$curator1_email", "Key Kinases", "10.0-.11"],
    "prot_phosphatase", ["prot_phosphatase", "$curator1", "$curator1_email", "Key Phosphatases", "10.0-.13 "],
    "g_proteins", ["g_proteins","$curator1","$curator1_email","Receptor, 2nd messenger, G-protein Activity", "10.0-.0-"],
    "other_sig", ["other_sig", "$curator1","$curator1_email","Other Signal Transduction Activities", "10.99"],
#CELLULAR RESCUE
    "cell_res", ["cell_res", "$curator1","$curator1_email","Cell Rescue, Stress Response ", "11.--"],
#HOMEOSTASIS OF METAL IONS
    "met_ion_homeo", ["met_ion_homeo", "$curator1", "$curator1_email", "Ion Homeostasis", "13.01"],

#MISCELLANEOUS
    "wtf", ["wtf", "$curator2", "$curator2_email", "wtf with tf type LTRs", "14.00.02"],
    "rna_bind", ["rna_bind", "$curator1", "$curator1_email", "Proteins with RNA binding domains", "04.PF.01"],
    "dna_bind", ["dna_bind", "$curator1", "$curator1_email", "Proteins with DNA binding domains", "99.PF.02"],
    "gtp_bind", ["gtp_bind", "$curator1", "$curator1_email", "Proteins with gtp binding domains", "99.PF.03"],

    "wd", ["wd", "$curator1", "$curator1_email", "WD repeat proteins", "99.PF.08"],
    "dna_j", ["dna_j", "$curator1", "$curator1_email", "DNA J domain proteins", "99.PF.06"],
    "hyp_helicase", ["hyp_helicase", "$curator1", "$curator1_email", "Helicases", "99.PF.07"],

    "retro", ["retro", "$curator1", "$curator1_email", "Transposons", "14.00.01"],
    "very_hypothetical", ["very_hypothetical", "$curator1", "$curator1_email", "Very hypothetical proteins", "99.00.01"],
    "hypothetical_new", ["hypothetical_new", "$curator1", "$curator1_email", "Hypothetical proteins", "99.00.04"],
    "not_sorted", ["not_sorted", "$curator1", "$curator1_email", "Unsorted proteins", "99.00.03"],
    "coiled_coil", ["coiled_coil", "$curator1", "$curator1_email", "Coiled-coil proteins", "99.00.02"],

);  





# IF extra elements are added to this hash they must be done before this, as the number
# of elements in this array varies.

# ADD CLASS QUALIFIER BREAKDOWN

foreach my $key (keys(%all_data)) {
my @class = split( /\./, $all_data{$key}[4]);
push @{$all_data{$key}}, @class;

}

######################################################################################################

#this is used to get mainclass totals, and print main web page

my %class_hash = ();


# PRINT WEBPAGE HEADERS FOR ALL MAIN CLASSES

######################################################################################################

foreach my $key (keys(%all_data)) {

open (WEBPAGE, ">$htmldir/$all_data{$key}[0].shtml");
&print_html_header ($all_data{$key}[1], $all_data{$key}[2], $all_data{$key}[0], 
                    $all_data{$key}[3] ) ;
print WEBPAGE "<h1>$all_data{$key}[4]</h1>";
print WEBPAGE "<TABLE BORDER=0 WIDTH=100%>";
close (WEBPAGE);

}


#create a hash for the unsorted entries, and for the totals

my %tally_hash = ();
my %not_sorted_hash = ();
my $line;


#   MAIN BIT TO PROCESS EACH LINE OF THE INPUT FILE

######################################################################################################


while (defined($line = <>)) {  

my $hit_flag = "false";
my %temp_main_class_hash = ();


   foreach my $key (keys(%all_data)) {
 
   #   Read the regular expressions from the config file

   my @config_data = &get_config_data("$dir/$all_data{$key}[0]");

   #   Compare the current input line to the configuration file
   my $class_positive = &search_config_data ($line, @config_data);


   #   If there is a match put the item into an array of this class
   #   print to a make a description line database for the fun class search.
   #   get main classes to use later to increment main class totals
  
     if ($class_positive ne 0){ 

     print STDOUT ">$all_data{$key}[4] >$class_positive";
     print STDOUT "STUFFERTOMAKEDATABASE\n";
     
 
     #increment the main class counters
     $temp_main_class_hash{$all_data{$key}[5]}++;

     open (WEBPAGE, ">>$htmldir/$all_data{$key}[0].shtml");

     my $tally_hash_key = $all_data{$key}[0];
     $tally_hash{$tally_hash_key}++;

     my $line_to_print = &print_links($class_positive);

     print WEBPAGE "$line_to_print";
     close (WEBPAGE);
     $hit_flag = "true";
     }
 

   }#END FOREACH



#increment the main class hash using the temp class hash

  foreach my $temp_key   (keys(%temp_main_class_hash)){
  $class_hash{$temp_key}++;
  }

#  if the item has not been sorted, put it in a hash to deal with later

   if ($hit_flag eq "false"){
   $not_sorted_hash{$line}++;
   }
   
} # END WHILE LOOP
 

#   PRINT WEB PAGE FOOTERS
###################################################################################################

foreach my $key (keys(%all_data)) {
  
   	open (WEBPAGE, ">>$htmldir/$all_data{$key}[0].shtml");
   	print WEBPAGE '</table>';
   	print WEBPAGE '</br></br>';
	print WEBPAGE '<!--#exec cgi="/cgi-bin/footer" -->';
   	close (WEBPAGE);
}



# DEAL WITH THE UNSORTED ENTRIES
###################################################################################################

# open the html pages
#replace these with the hash refs

open (WEBPAGE, ">$htmldir/hypothetical_new.shtml");

&print_html_header ($curator1, $curator1_email, "hypothetical", "Hypothetical proteins" ) ;
print WEBPAGE "<h1>99.00.04</h1>";
print WEBPAGE '<table border="0" width="100%">';
close (WEBPAGE);

open (WEBPAGE, ">$htmldir/not_sorted.shtml");
&print_html_header ($curator1, $curator1_email, "not_sorted", "Unsorted Proteins" ) ;
print WEBPAGE "<h1>99.00.03</h1>";
print WEBPAGE '<table border="0" width="100%">';
close (WEBPAGE);


foreach my $not_sort (sort(keys %not_sorted_hash)) {

   $not_sort =~ s/^>//g;
   if ($not_sort =~ /HYPOTHETICAL/){

   print STDOUT ">99.00.04 >$not_sort";
   print STDOUT "STUFFERTOMAKEDATABASE\n";

   open (WEBPAGE, ">>$htmldir/hypothetical_new.shtml");
   my $line_to_print = &print_links($not_sort);
   print WEBPAGE "$line_to_print";
   close (WEBPAGE);

   $tally_hash{hypothetical_new}++;

   }else{
   print STDOUT ">99.00.03 >$not_sort";
   print STDOUT "STUFFERTOMAKEDATABASE\n";
   open (WEBPAGE, ">>$htmldir/not_sorted.shtml");
   my $line_to_print = &print_links($not_sort);
   print WEBPAGE "$line_to_print";
   close (WEBPAGE);

   $tally_hash{not_sorted}++;

   }
}

####################################################################################################


#PRINT HYPOTHETICAL & NOT SORTED WEB PAGE FOOTERS

####################################################################################################

open (WEBPAGE, ">>$htmldir/hypothetical_new.shtml");
print WEBPAGE '</table>';
print WEBPAGE '</br></br>';
print WEBPAGE "<!--#exec cgi=\"/cgi-bin/footer\" -->";
close (WEBPAGE);

open (WEBPAGE, ">>$htmldir/not_sorted.shtml");
print WEBPAGE '</table>';
print WEBPAGE '</br></br>';
print WEBPAGE "<!--#exec cgi=\"/cgi-bin/footer\" -->";
close (WEBPAGE);


####################################################################################################

# MAIN CLASS HASH needs hypothetivcal & not sorted adding

#need to add the unsorted totals to the main hash

$class_hash{99} += $tally_hash{hypothetical_new};
$class_hash{99} += $tally_hash{not_sorted};


####################################################################################################

#PRINT THE MAIN WEB PAGE

###################################################################################################

open (WEBPAGE, ">$htmldir/funcat.shtml");
print WEBPAGE <<EOS;
<!--#include virtual="header.shtml" -->
<!--#set var="author" value="$curator1_email" -->
<table width="100%" cellspacing="0" cellpadding="0">
  <tr class="h2bg">
    <td width="100%">
      <br>
      <h2 align="center"><I>Schizosaccharomyces pombe </I>Functional Classification</h2>
    </td>
  </tr>
</table>
<br>
<FONT FACE="Verdana,Arial,Helvetica">
EOS


close (WEBPAGE);


# for each main class get the main data

foreach my $key (sort keys(%class_hash)) {
   
   my @this_class;
   
   foreach my $key_all_data (keys(%all_data)) {
   
      if ($key == $all_data{$key_all_data}[5]) {
      push (@this_class, $all_data{$key_all_data});
      }
   }

   my @this_class_sorted = sort{$a->[4]cmp$b->[4]}@this_class;
   my $main_class;

   if ($key == 01){
   $main_class = "Metabolism";
   }elsif ($key eq "02"){
   $main_class = "Energy";
   }elsif ($key eq "03"){
   $main_class = "Cell Growth, Cell Division and DNA Synthesis";
   }elsif ($key eq "04"){
   $main_class = "Transcription";
   }elsif ($key eq "05"){
   $main_class = "Protein Synthesis";
   }elsif ($key eq "06"){
   $main_class = "Protein Destination Modification etc";
   }elsif ($key eq "07"){
   $main_class = "Transport Facilitation- Membrane Transporters";
   }elsif ($key eq "08"){
   $main_class = "Intracellular-Extracellular transport";
   }elsif ($key eq "09"){
   $main_class = "Cellular Organisation and Biogenesis";
   }elsif ($key eq "10"){
   $main_class = "Signal Transduction activities";
   }elsif ($key eq "11"){
   $main_class = "Cell Rescue";
   }elsif ($key eq "13"){
   $main_class = "Ion Homeostasis";
   }elsif ($key eq "14"){
   $main_class = "Transposons";
   }elsif ($key eq "99"){
   $main_class = "Miscellaneous";
   }
   open (LOGFILE, ">>$contig_dir/fc_log");
   open (WEBPAGE, ">>$htmldir/funcat.shtml");
   print LOGFILE "$main_class $class_hash{$key}\n";
   print WEBPAGE "</P>";
   print WEBPAGE "<H4>$main_class ($class_hash{$key})</H4>\n";  
   print WEBPAGE "<P>";
  close (WEBPAGE);

   foreach my $var (@this_class_sorted){

open (WEBPAGE, ">>$htmldir/funcat.shtml");
if ( exists $tally_hash{$var->[0]}){
      print WEBPAGE "<A HREF= \"$http$var->[0].shtml\">$var->[4] $var->[3] </A> $tally_hash{$var->[0]}<BR>";
   print LOGFILE "$var->[4] $tally_hash{$var->[0]} $var->[3]\n";
    }else {
print WEBPAGE "<A HREF= \"$http$var->[0].shtml\">$var->[4] $var->[3] </A><BR>";
    }



close (WEBPAGE);
   }

} 


open (WEBPAGE, ">>$htmldir/funcat.shtml");
print WEBPAGE ""; 
print WEBPAGE "<!--#exec cgi=\"/cgi-bin/footer\" -->";
close (WEBPAGE);


###############################################################################
#                                                                             #
#                                 SUBROUTINES                                 #
#                                                                             #
###############################################################################


#                                 Get line to print                           #
###############################################################################



sub print_links  { 

my ($data) = @_;  
my @splitall = split(/:/, $data);
my @genes = "";
@genes = split(/,/, $splitall[0]);
my $des = $splitall[1];              
my @accs = split(/;/, $splitall[$#splitall]);                
my $linked_acc = shift(@accs);
$linked_acc =~ s/\s//g;
my $db_link;
     
#debugging
#print STDOUT "class$all_data{$key}[4]\n";
#print STDOUT "full line $class_positive";
#print STDOUT "genes @genes\n";
#print STDOUT "des $des\n";
#print STDOUT "accs @accs\n";

   if (length $linked_acc == 8 ){
   $db_link = "<A HREF=http://www.sanger.ac.uk/srs51bin/cgi-bin/wgetz?-e+[{swissprot%20swissnew%20sptrembl%20tremblnew}-acc:$linked_acc]>$linked_acc</A>";
   }
   else{
   $db_link = "<A HREF=http://www.expasy.ch/cgi-bin/sprot-search-ac?$linked_acc]>$linked_acc</A>";
   }
my $line_to_print = "<TR><TD><font size = -3>@genes</TD><TD><font size = -3>$des</TD><TD><font size = -3>$db_link @accs</TD></TR>";

return $line_to_print;

}



#                                 PRINT WEB PAGES                             #
###############################################################################

sub  print_html_header{
my ($name, $email, $file, $title) = @_;
print WEBPAGE <<EOS;
<!--#config timefmt="%d-%b-%C%y, %I:%M %p" -->
<!--#set var="title" value="$title" -->
<!--#set var="icon" value="euflag.gif" -->
<!--#set var="author" value="$email" -->
<!--#set var="stylesheet" value="/stylesheets/projects.css" -->
<!--#set var="keywords" value="Fission Yeast Schizosaccharomyces pombe Genome
Sequencing Project Functional Classification $title $file" -->
<!--#set var="description" value="The Schizosaccharomyces pombe Genome Sequencing Project Functional Classification" -->
<!--#exec cgi="/cgi-bin/header2" -->

<table width="100%" cellspacing="0" cellpadding="0">
  <tr class="h2bg">
    <td width="100%">
      <br>
      <h2 align="center">$title</h2>
    </td>
  </tr>
</table>
<br>
<FONT FACE="Verdana,Arial,Helvetica">

EOS
}


#                                 GET CONFIG DATA                             #
###############################################################################
sub  get_config_data{
   my ($file) = @_;
   my @temp = "";
   my $tempaa;

   open (TEMPFILEHANDLE,  "$file") || die "cannot open $file";
   while (<TEMPFILEHANDLE>) {
   my $line = $_;
   $tempaa = $line;
   chop $tempaa;
   push (@temp, $tempaa);       
   }
   close  TEMPFILEHANDLE;
   return @temp;
}

#                                 SEARCH CONFIG DATA                          #
###############################################################################

sub  search_config_data{
  my ($line, @array_temp) = @_;
  my $newline = 0;
  shift @array_temp;
  foreach my $var (@array_temp){
    if ($line =~ /$var/ ){ 
    $line =~ s/^>//;
    $newline = $line;
    } 
  }
return $newline;
}


