#!/usr/local/bin/perl -w
# Autogenerates pretty alphabetic tables for all the software packages
# RMP November 1999
#
#       Modified jws 2000-03-15 for use with S.coelicolor cosmid images.
#
# input file should be whitespace delimited:
#                                       _columns        3
#                                       _order  vertical
#                                       AceDB   http://www.acedb.org
#                                       Alfresco        /Software/Alfresco/
#                                       Angler  /Software/Angler/
#                                       Artemis /Software/Artemis/


my %tabledata;          # Hash to hold the data for the table

get_data_from_file();
write_html(\%tabledata);

1;

sub get_data_from_file {
        open (DATAFILE, $ARGV[0]) || die "Cannot open $ARGV[0]";
        while (<DATAFILE>) {
                s/(\S*)\s*(\S*)/$tabledata{$1}=$2/e;
        }
}

sub write_html {
    my ($ref) = @_;
    my %datahash = %$ref;

    my $columns = $datahash{'_columns'};
    delete $datahash{'_columns'};
    my $order = $datahash{'_order'};
    delete $datahash{'_order'};

    my $violet = 0;
    my $column = 1;                             # 1..$columns
    my ($key, $value);
    my $tablecols = ($columns * 2) + 1;         # allow for spaces in-between
    my $tablecols2 = $tablecols-2;                    # poxy colspan
    my $tablerows = ((scalar keys %datahash)/$columns) + ($columns+2);      
        # for grey line rowspan + line, title, line, (content), line

    print STDOUT <<EOT;
<center>
<table border="0" cellspacing="0" cellpadding="0">

  <tr valign="top">
    <td class="grey1" rowspan="$tablerows"><img src="/icons/blank.gif" width="1" height="1"></td>
    <td colspan="$tablecols" class="grey1"><img src="/icons/blank.gif" width="400" height="1"></td>
    <td class="grey1" rowspan="$tablerows"><img src="/icons/blank.gif" width="1" height="1"></td>
  </tr>
EOT

    if($order eq "horizontal") {
        foreach $key (sort keys %datahash) {
            if($column == 1) {
                print STDOUT "  <tr valign=\"top\" class=\"violet", $violet+1, "\">\n";
                print STDOUT "    <td><img src=\"/icons/blank.gif\" width=\"22\" height=\"18\"></td>\n";
            }
            print STDOUT "    <td><a class=\"smarial\" href=\"$datahash{$key}\">$key</a></td>\n",
            "    <td><img src=\"/icons/blank.gif\" width=\"22\" height=\"18\"></td>\n";
            $column++;
            if($column > $columns) {
                $violet = !$violet;
                $column=1;
                print STDOUT "  </tr>\n";
            }
        }
    
        $column--;
        if($column <= 0) {
            $column = 3;
        }
        if($column != 3) {
            for(my $i=1; $i <= ($columns - $column); $i++) {
                print STDOUT "    <td>&nbsp;</td>\n";
                print STDOUT "    <td><img src=\"/icons/blank.gif\" width=\"22\" height=\"18\"></td>\n";
            }
            print STDOUT "  </tr>\n";
        }
    } elsif ($order eq "vertical") {
        my $len = (scalar (keys %datahash));
        if(($len % $columns)!=0) {
            $len = $len + ($columns - ($len % $columns));
        }
        my $mod = $len / $columns;
        my (@sorted) = (sort keys %datahash);

        for(my $rows=0; $rows<$mod; $rows++) {

            print STDOUT "  <tr valign=\"top\" class=\"violet", $violet+1, "\">\n";
            print STDOUT "    <td><img src=\"/icons/blank.gif\" width=\"22\" height=\"18\"></td>\n";

            for(my $cols=0;$cols<$columns; $cols++) {
                my $entry = $rows + ($cols * $mod);
                if(defined $sorted[$entry]) {
                    print STDOUT "    <td><a class=\"smarial\" href=\"$datahash{$sorted[$entry]}\">$sorted[$entry]</a></td>\n";
                } else {
                    print STDOUT "    <td>&nbsp</td>\n";
                }
                print STDOUT "    <td><img src=\"/icons/blank.gif\" width=\"22\" height=\"18\"></td>\n";
            }
            print STDOUT "  </tr>\n";
            $violet = !$violet;
        }
    } else {
        print STDERR "_order not defined \n";

    }

    print STDOUT <<EOT
  <tr valign="top">
    <td colspan="$tablecols" class="grey1"><img src="/icons/blank.gif" width="400" height="1"></td>
  </tr>
</table>
</center>
EOT
}
