
/*
 * Copyright (c) 2001  Genome Research Limited.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package uk.ac.sanger.psu.tigread;

import org.biojava.bio.seq.StrandedFeature;
import org.biojava.bio.seq.StrandedFeature.Strand;
import org.biojava.bio.symbol.Location;
import org.biojava.bio.symbol.AbstractLocationDecorator;

/**
 * <code>StrandedLocation</code> is a standard BioJava
 * <code>Location</code> (which has no notion of strand) decorated
 * with strand information.
 *
 * @author <a href="mailto:kdj@fes1.sanger.ac.uk">Keith James</a>
 */
class StrandedLocation extends AbstractLocationDecorator
{
    private Strand strand;

    public StrandedLocation(Location wrapped)
    {
        super(wrapped);

        strand = StrandedFeature.UNKNOWN;
    }

    public void setStrand(Strand strand)
    {
        this.strand = strand;
    }

    public Strand getStrand()
    {
        return strand;
    }

    protected Location decorate(Location loc)
    {
        return new StrandedLocation(loc);
    }
}
