/*
 * Copyright (c) 2004 Genome Research Limited.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package uk.ac.sanger.psu.tigrutils;

import org.apache.regexp.RE; // Used in /color assingment
import org.apache.commons.lang.StringUtils;


public class TigrUtils {
    public TigrUtils (){ 

    }

    static RE putative;

    static {
        try {
            putative = new RE(",\\s*putative");
        }
        catch(org.apache.regexp.RESyntaxException exp){
            exp.printStackTrace();
        }
    }

    /* <code>getColourFromProduct</code> returns the colour as a String given
     * the /product and /pseudo. It will return 2 if it does not know what the
     * colour should be.
     *
     * @param product <code>String</String> is the value of /product
     * @param isPseudo is a boolean, true if /pseudo is present
     *
     * @return a <code>String</code> containing a number, the value of /colour
     *
     */
    public static String getColourFromProduct(String product, boolean isPseudo){
        String colour = null;

        if (StringUtils.contains(product, "hypothetical protein, unlikely")) {
            colour = "6";
        }
        else if (StringUtils.contains(product, "hypothetical protein, conserved")) {
            colour = "10";
        }
        //else if (StringUtils.contains(product, "hypothetical protein")) {
        else if (product.startsWith("hypothetical protein")) {
            colour = "8";
        }
        else if (product.endsWith("like protein")) {
            colour = "7";
        }

        if (putative.match(product))
            colour = "7";
 
        if (isPseudo)
            colour = "13";

        if (colour == null)
            colour = "2"; // safe default

        return colour;
    }
}
